
C               Program GENPAT.  (Source code, FORTRAN-90)
C
C                    Last revised: 3 August 2002
C
C GENPAT creates an NV-by-NF ASCII raw factor pattern that can be passed to
C program GENSCOR or KWIKCOVS for rescaling as needed after constrained-random
C assignment of communalities and factor correlations.  GENSCOR then outputs
C a population of standardized factor scores augmented by data scores derived
C from those in accord with the rescaled GENPAT pattern together with the
C communalities and factor correlations set in GENSCOR.  In contrast, KWIKCOVS
C produces just a data correlation matrix having the stipulated source
C parameters.  Population scores from GENSCOR or population correlations from
C KWIKCOVS can be sampled and further processed by other programs in the
C Gendata package.

C For details of GENPAT's raw-pattern production, see lines 85-115  below.

C The created pattern's filename has form P?{Kode} in which P is fixed, ? is
C a digit or perhaps a letter,  and  are letters, {Kode} is a pair of digits,
C and  is the lowest letter index A,B,... that will not overwrite any other
C so-named pattern in this subdirectory.  String ?{Kode} codes information
C about this pattern's structure that is decoded to screen when you run GENSCOR.

C The pattern comprises a master JV-by-JF subpattern template repeated KV times
C in rows and KF times in columns, so that NV = JV*KV and NF = JF*KF.  (There is
C no need for KN or KF to be larger than 1. Maximum JF is 7 plus number JG of
C general factors.)  In output-filename P?{Kode}, two-digit factor-complexity
C {Kode} is sum over J=2,..,JF of KMP(J)**(J-1) where KMP(J)=1 (otherwise 0) if
C complexity-level J is included in the pattern, <JF,KF,JV> has two-letter code
C  whose translation into integers 0 to 25*25-1 has composition 100*(JF-1) +
C 10*(KF-1) + (JV-1).  ? codes KMP(1), which is the recurrence count of factor-
C complexity level 1: if KMP(1)  9, ? is that digit; otherwise, for KMP(1)
C up to 36, ? is the letter in position KMP(1)-9 of the alphabetic sequence.
C And  is a combines coding of JG with indexing repetitions of raw pattern
C P?{Kode}: If JG = 0,  is a single-digit numeric repetition index;
C otherwise,  is the upper-case letter such that for K = ICHAR() - 64,
C 1+MOD(K-1,5) = JG while 1+(K-1)/5 is the repetition count for this raw
C pattern.  (Thus neither JG nor the repetition count can exceed 5.)
C See also construction just after "CALL CODE" at line 502, below.
C     For each included complexity level J  2, the variables having complexity
C J on the complexity-controlled factors in the subpattern template comprise
C all different ways to distribute J salient loadings over JF-JG factors.  For
C complexity 1, the template includes KMP(1) variables for each different way to
C assign one salient loading to one of the JF-JG complexity-controlled factors.
C
C ### In brief, P?{Kode} codes block structure in  followed by the higher
C     complexity levels in KODE, complexity-1 count in ?, and a count of
C     general factors combined with number of co-present repetitions of this
C     raw pattern.

C GENPAT output goes to GENSCOR

      PARAMETER (MV=500, MF=50, MB=7)
      LOGICAL RBLK, QY, QP
      CHARACTER F1*7, CLN*8, CH4*4, FMT*70, WORD*99, CF*12
      CHARACTER C11,C12,C21,C22,CL,CR,CT,CB, BAR,LIN
      INTEGER ORDER(MV,MF), LWK(MV), KMP(MB), LST(MB)
      REAL A(MV,MF), B(4,MF), BQ(4)
      CHARACTER,ALLOCATABLE:: IDENT(:)*26
      INTEGER,ALLOCATABLE:: PERM(:,:), NAML(:)
      REAL,ALLOCATABLE :: BLOK(:,:)
C       B(1,J)/B(2,J) holds the upper/lower magnitude bound on the raw factor-J
C       loadings; B(3,J) is the probability these are negative; B(4,J) is the
C       magnitude bound on nonsalient loadings.
      EQUIVALENCE (B,BQ)   ! To enable DATA statement for 1st col of B
      EXTERNAL SCAN
      COMMON /CF/ CF
      DATA JF,RBLK,QP/5,2*.FALSE./, BQ/1.,.25,.2,.0/, KF,KV,KMP1/3*1/
      DATA C11,C12,C21,C22,BAR,LIN,CL,CR,CT,CB/'','','','','','',
     +     '','','',''/
C  Full-pattern index of item I (factor J) in item block IB (factor block JB)
      OPEN(2,STATUS='SCRATCH')
      CALL SYSTEM('cls')
      INQUIRE(FILE='LASTPAT',EXIST=QY)
      IF(QY) THEN
        OPEN(3,FILE='LASTPAT')
        READ(3,*,END=2) JF, KMP1, (B(I,1),I=1,4), X, Y
        IF(Y>0) RBLK = .TRUE. ! Do NOT get QP=T from X
2       CLOSE(3)
      END IF
      QY = .TRUE.

C Ŀ
C   GENPAT creates a well-structured raw factor pattern for NV items (data    
C   variables) on NF factors, where NV (NF) is some small multiple of the     
C   number JV of items (number JF+JG of factors) in a master subpattern block 
C   B optionally augmented by a block G of general factors.  You may choose   
C   JF  7 and any subset of factor-complexity levels up to JF for B, and     
C   flank B by JG  5 general factors.  Thereafter, the program will advise   
C   you of the choices acceptable for NF and NV, and if space remains will    
C   allow some repetitions of the complexity-1 template rows in B.  Each of   
C   the JV different ways to realize one of your chosen factor complexities   
C   (an item's number of salient loadings) on JF factors is taken to define   
C   a row of salient/non-salient template markers in master subpattern B,     
C   with repetitions of complexity-1 rows as declared.  On each general       
C   factor, all loadings are marked as salient.  From there, if you choose    
C   NV > JV and NF > JF+JG, your full pattern template repeats B, or [B G] if 
C   JG > 0, down blocks of item rows and, with optional random permutations   
C   of B's rows, across blocks of factor columns.                             29
C Ĵ
C     Once the full pattern template is defined, its salient/nonsalient       
C     markers are assigned numerical loadings under constraints you set       
C     separately for each block of JV factors.  These constraints are:        
C        a) Lower/upper limits [Lo,Hi] on the size of salient loadings.       
C        b) The probability Pr that a salient loading will be negative.       
C        c) A limit W on the size of nonsalient loadings.                     
C     Nonsalient loadings on factors in this block are generated randomly     
C     between W and -W.  Salient loadings are randomized uniformly between    
C     Lo and Hi, after which they are made negative with probability Pr.      
C     (Special exception: Hi = 0 makes all this block's loadings nonsalient.) 
C                                                                             
C        Finally, you can permute blocks of factor loadings differently       
C        within each item block, thereby choosing where those items'          
C        largest/weakest loadings occur in the factor sequence and            
C        demarking item blocks by clustering of the data covariances.         
C 

      WRITE(6,'(1X,80A)') C11, (LIN,I=1,76), C12
      WRITE(6,'(1X,A,"  GENPAT creates a well-structured raw factor ",
     +  "pattern for NV items (data    ",A/1X,A,"  variables) on NF",
     +  " factors, where NV (NF) is some small multiple of the     ",
     +  A/1X,A,"  number JV of items (number JF+JG of factors) in a ",
     +  "master subpattern block ",A/1X,A,"  B optionally augmented ",
     +  "by a block G of general factors.  You may choose",3X,A/1X,A,
     +  "  JF  7 and any subset of factor-complexity levels up to ",
     +  "JF for B, and",5X,A/1X,A,"  flank B by JG  5 general fact",
     +  "ors.  Thereafter, the program will advise",3X,A/1X,A,"  yo",
     +  "u of the choices acceptable for NF and NV, and if space re",
     +  "mains will",4X,A/1X,A,"  allow some repetitions of the com",
     +  "plexity-1 template rows in B.  Each of",3X,A)') (BAR,I=1,16)
      WRITE(6,'(1X,A,"  the JV different ways to realize one of you",
     +  "r chosen factor complexities",3X,A/1X,A,"  (an item''s num",
     +  "ber of salient loadings) on JF factors is taken to define",
     +  3X,A/1X,A,"  a row of salient/non-salient template markers ",
     +  "in master subpattern B,",5X,A/1X,A,"  with repetitions of ",
     +  "complexity-1 rows as declared.  On each general",7X,A/1X,A,
     +  "  factor, all loadings are marked as salient.  From there,",
     +  " if you choose",4X,A/1X,A,"  NV > JV and NF > JF+JG, your",
     +  " full pattern template repeats B, or [B G] if ",A/1X,A,
     +  "  JG > 0, down blocks of item rows and, with optional rand",
     +  "om permutations",3X,A/1X,A,"  of B''s rows, across blocks",
     +  " of factor columns.",29X,A)') (BAR,I=1,16)
      WRITE(6,'(24X,"Hit RETURN to continue")'); READ(5,'()')
      WRITE(6,'(1X,80A)') CL, (LIN,I=1,76), CR
CC      WRITE(6,'(1X,A,76X,A)') BAR, BAR
      WRITE(6,'(1X,A,4X,"Once the full pattern template is defined, ",
     +  "its salient/nonsalient",7X,A/1X,A,4X,"markers are assigned ",
     +  "numerical loadings under constraints you set",7X,A/1X,A,4X,
     +  "separately for each block of JV factors.  These constraints",
     +  " are:",8X,A/1X,A,7X,"a) Lower/upper limits [Lo,Hi] on the s",
     +  "ize of salient loadings.",7X,A/1X,A,7X,"b) The probability",
     +  " Pr that a salient loading will be negative.",7X,A/1X,A,7X,
     +  "c) A limit W on the size of nonsalient loadings.",21X,A/1X,A,
     +  4X,"Nonsalient loadings on factors in this block are generat",
     +  "ed randomly",5X,A/1X,A,4X,"between W and -W.  Salient loadi",
     +  "ngs are randomized uniformly between",4X,A/1X,A,4X,"Lo and ",
     +  "Hi, after which they are made negative with probability Pr.",
     +  6X,A/1X,A,4X,"(Special exception: Hi = 0 makes all this bloc",
     +  "k''s loadings nonsalient.) ",A)') (BAR,I=1,20)
      WRITE(6,'(1X,A,76X,A)') BAR, BAR
      WRITE(6,'(1X,A,7X,"Finally, you can permute blocks of factor l",
     +  "oadings differently",7X,A/1X,A,7X,"within each item block, t",
     +  "hereby choosing where those items''",10X,A/1X,A,7X,"largest",
     +  "/weakest loadings occur in the factor sequence and",12X,A/1X,
     +  A,7X,"demarking item blocks by clustering of the data covar",
     +  "iances.",9X,A)') (BAR,I=1,8)
      WRITE(6,'(1X,80A)') C21, (LIN,I=1,76), C22
      WRITE(6,'(24X," Hit RETURN to continue ")')
      READ(5,'(A1)')
C
200   WRITE(6,'(/5X,"First, choose the number JF of factors in your",
     +  " master subpattern B,"/5X,"not including general factors.",
     +  "  (limit is 7)")')
5     WRITE(6,'(/" JF is now set at ",A,".  Hit RETURN if OK, or ",
     +  "enter new number of factors in block."/)') CF(:JJF(JF))
      CALL SCAN(J,1,'I')
      IF(J<0) GOTO 5
      IF(J==0) GOTO 10
      READ(2,*) JF
      JF = MAX(1,MIN(7,JF))
      GOTO 5
10    WRITE(6,'(/" The ""factor complexity"" of any row of a factor ",
     +  "pattern is the number of"/" loadings therein that are sali",
     +  "ent, that is, appreciable in size.  For each"/" complexity",
     +  " level K  1, the number of different ways to distribute K ",
     +  "salients"/" over ",A," factors is shown in the following ",
     +  "table. (Level 0 is not allowed.)")') CF(:JJF(JF))
C          Ŀ
C           Factor-complexity level    1   2   3   4   5   6   7  
C           Number of realizations    xx  xx  xx  xx  xx  xx  xx  
C          
      FMT = '(10X,27A,'//CHAR(48+JF)//'(4A),3A)'
      WRITE(6,FMT) C11, (LIN,I=1,25), CT, (LIN,I=1,4*JF), LIN,LIN,C12
      FMT = '(10X,A," Factor-complexity level ",A,'//CHAR(48+JF)//
     +  'I4,"  ",A)'
      WRITE(6,FMT) BAR,BAR, (I,I=1,JF), BAR
      FMT(10:32) = 'Number of realizations '
      WRITE(6,FMT) BAR,BAR, (KOMB(JF,K),K=1,JF), BAR
      FMT = '(10X,27A,'//CHAR(48+JF)//'(4A),3A)'
      WRITE(6,FMT) C21, (LIN,I=1,25), CB, (LIN,I=1,4*JF), LIN,LIN,C22
      LC = 0; JV = 0  ! JV is accumulated number of items
15    LC = LC+1
      LST(LC) = LC
      JV = JV + KOMB(JF,LC)  ! Default setting, seldom if ever wanted
      IF(JV+KOMB(JF,LC+1)<MV .AND. LC<=JF/2) GOTO 15
      WRITE(6,'(" You may choose for your master subpattern''s co",
     +  "mplexities any selection of"/" these levels whose total ",
     +  "number of realizations does not exceed ",A)') CF(:JJF(MV))
17    WRITE(6,'(/" Your selection of factor-complexity levels is ",
     +  "now"/10X,10I4)') (LST(L),L=1,LC)
      WRITE(6,'(" Hit RETURN if OK.  Otherwise, enter new string of",
     +  " complexity levels"/" or any letter to respecify JF."/)')
      CALL SCAN(J,0,'I')
      IF(J<0) GOTO 5
      IF(J==0) GOTO 19
      READ(2,*) (LST(I),I=1,MIN(MB,J))
      CALL SORT(LST,J,1,JF)
      LC = J
19    JV = 0
      DO I = 1,LC
        JV = JV + KOMB(JF,LST(I))
      END DO                     !   ^ List of last index in each item block
      IF(JV<=MV .AND. J==0) GOTO 22
      IF(JV>MV) THEN
        WRITE(6,'(/" Your selected complexity levels have ",A," re",
     +   "alizations on ",A," factors, more than"/" the maximum",
     +   " allowed.  Choose again.")') CF(:JJF(JV)), CF(:JJF(JF))
        CALL SYSTEM('CLS')
        GOTO 10
      END IF
      GOTO 17
22    KMP = 0
      DO I = 1,LC
        KMP(LST(I)) = 1
      END DO

C Allow repetitions of the complexity-1 pattern rows
      MRP = MIN(36,1+(MV-JV)/JF)   ! Need JF items for every repetition
      KMP1 = MAX(1,KMP1)*KMP(1)
      IF(MRP<=1) GOTO 30
      WRITE(6,'(" The complexity-1 template rows in B can recur up",
     +  " to ",A," times.")') CF(:JJF(MRP))
26    CH4(:2) = 's. '
      IF(KMP1==1) CH4(:2) = '. '
      WRITE(6,'(" Each different complexity-1 row of B is now set ",
     +  "to occur ",A," time",A/" Hit RETURN if OK, or enter o",
     +  "ther recurrence count no greater than ",A,"."/)')
     +  CF(:JJF(KMP1)), CH4(:2), CF(:JJF(MRP))
      CALL SCAN(J,1,'I')
      IF(J<0) GOTO 26
      IF(J>0) THEN
        READ(2,*) J
        KMP1 = MAX(1,MIN(MRP,J))
        GOTO 26
      END IF
      KMP(1) = KMP1
      JV = JV + MAX(0,KMP1-1)*JF
C
C Select number JG of general factors if wanted
      JG = 0
      L = MIN(5,10-JF)
      IF(L<1) GOTO 30
      WRITE(6,'(/5X,"Hit RETURN to confirm that you don''t want any ",
     +  "general factors."/5X,"Otherwise, enter anything to consider",
     +  " including some.")')
      CALL SCAN(J,0,'B'); IF(J==0) GOTO 30
      WRITE(6,'(/5X,"You may add to your master subpattern up to",
     +  I2," general"/5X,"factors on which all variables have ",
     +  "salient loadings.")') L
28    WRITE(6,'(/" The number of general factors adjoined to B is",
     +  " now",I2,".  Hit RETURN if OK,"/" or enter another ch",
     +  "oice not over",I2,"."/)') JG, L
      CALL SCAN(J,1,'I')
      IF(J==0) GOTO 30
      READ(2,*) JG
      JG = MAX(0,MIN(L,JG))
      GOTO 28
C
C Initialize pre-General master pattern block
30    A = 0.; I = 0
C Put in pre-G master pattern block all realizations of complexities in KMP.
      IF(KMP(1)==0) GOTO 35
C      I = 0.; WRITE(6,'(" Installing ",A," complexity-",I1," items ",
C     +  "after item ",A)') CF(:JJF(KOMB(JF,1))), 1, CF(:JJF(I))
      DO K = 1,KMP(1); DO J = 1,JF
          I = I+1
          A(I,J) = 1.
      END DO; END DO
35    IF(KMP(2)==0) GOTO 39
C      WRITE(6,'(" Installing ",A," complexity-",I1," items aft",
C     +  "er item ",A)') CF(:JJF(KOMB(JF,2))), 2, CF(:JJF(I))
      DO J = 1,JF-1; DO K = J+1,JF
          I = I+1
          A(I,J)=1.; A(I,K)=1.
      END DO; END DO
39    IF(KMP(3)==0) GOTO 41
C      WRITE(6,'(" Installing ",A," complexity-",I1," items aft",
C     +  "er item ",A)') CF(:JJF(KOMB(JF,3))), 3, CF(:JJF(I))
      DO J = 1,JF-2; DO K = J+1,JF-1
          DO L = K+1,JF
            I = I+1
            A(I,J)=1.; A(I,K)=1.; A(I,L)=1.
          END DO
      END DO; END DO
41    IF(KMP(4)==0) GOTO 44
C      WRITE(6,'(" Installing ",A," complexity-",I1," items aft",
C     +  "er item ",A)') CF(:JJF(KOMB(JF,4))), 4, CF(:JJF(I))
      DO J = 1,JF-3; DO K = J+1,JF-2
          DO L = K+1,JF-1
            DO M = L+1,JF
              I = I+1
              A(I,J)=1.; A(I,K)=1.; A(I,L)=1.; A(I,M)=1.
            END DO
          END DO
      END DO; END DO
44    IF(KMP(5)==0) GOTO 48
C      WRITE(6,'(" Installing ",A," complexity-",I1," items aft",
C     +  "er item ",A)') CF(:JJF(KOMB(JF,5))), 5, CF(:JJF(I))
      DO J = 1,JF-4; DO K = J+1,JF-3
          DO L = K+1,JF-2
            DO L1 = L+1,JF-1
              DO L2 = L1+1,JF
                I = I+1
                A(I,J)=1.; A(I,K)=1.; A(I,L)=1.; A(I,L1)=1.; A(I,L2)=1.
              END DO
            END DO
          END DO
      END DO; END DO
48    IF(KMP(6)==0) GOTO 51
C      WRITE(6,'(" Installing ",A," complexity-",I1," items aft",
C     +  "er item ",A)') CF(:JJF(KOMB(JF,6))), 6, CF(:JJF(I))
      DO J = 1,JF-5; DO K = J+1,JF-4
          DO L = K+1,JF-3
            DO L1 = L+1,JF-2
              DO L2 = L1+1,JF-1
                DO L3 = L2+1,JF
                  I = I+1
                  A(I,J)=1.; A(I,K)=1.; A(I,L)=1.
                  A(I,L1)=1.; A(I,L2)=1.; A(I,L3)=1.
                END DO
              END DO
            END DO
          END DO
      END DO; END DO
51    IF(KMP(7)>0) THEN ! *** If MB is increased, this logic will need change
        DO J = 1,JF
          I = I+1
          DO K = 1,JF
            A(I,K) = 1.
          END DO
        END DO
      END IF
C Install general factors
      DO J = 1,JG; DO I = 1,JV
          A(I,JF+J) = 1.
      END DO; END DO
      JF = JF+JG
      LV = MV/JV; LF = MF/JF; NV = JV*1; NF = JF*1
      LLF = LF+1  ! ^ LV (LF) is the max number of item (factor) blocks
      WORD(:8) = '   NV = ';  L=8; K=0;            ! L is running count of length,
222   IF(K<=LV.AND.L<66 .OR. K>LV-3.AND.K<LV) THEN ! K is running count of entries
        K = K+1; I = JJF(JV*K); WORD(L+1:L+2+I) = CF(:I)//', '
        L = L+2+I; LL = L; GOTO 222
      ELSE IF(K<LV) THEN
        I=JJF(JV*LV); LL = L+9+I; WORD(L+1:LL)= '....., '//CF(:I)//', '
      END IF
      WRITE(6,'(/" Your options for expanding your ",A,"-by-",I1,
     +  " master block template into"/" a full NV-by-NF factor pa",
     +  "ttern are")') CF(:JJF(JV)), JF
      WRITE(6,'(A)') WORD(:LL-2)
      WORD = ' '; WORD(:8) = '   NF = '; L=8; K=0;  ! L and K are running counts
224   IF(K<=LF.AND.L<66 .OR. K>LF-3.AND.K<LF) THEN
        K = K+1; I = JJF(JF*K); WORD(L+1:L+2+I) = CF(:I)//', '
        L = L+2+I; LL = L; GOTO 224
      ELSE IF(K<LF) THEN
        I=JJF(JF*LF); LL = L+9+I; WORD(L+1:LL)= '....., '//CF(:I)//', '
      END IF
      WRITE(6,'(A)') WORD(:LL-2)
      IF(JV==1) GOTO 57
56    WRITE(6,'(/" Total number of items NV is now set at ",A,". ",
     +  "Hit RETURN if OK, or enter"/" another choice from alterna",
     +  "tives ",10(A,:", "))') CF(:JJF(NV)), (CF(:JJF(JV*I)),
     +  I=1,MIN(LV,10))
      IF(LV>10) WRITE(6,'(" or any other multiple of ",A," no larg",
     +  "er than ",A,".")') CF(:JJF(JF)), CF(:JJF(JV*LV))
      WRITE(6,'()')
      CALL SCAN(J,0,'I')
      IF(J==0) GOTO 57
      IF(J<0) GOTO 56
      READ(2,*) J
      NV = MAX(JV,MIN(MV,JV*NINT(J*1./JV)))
      GOTO 56
57    LLF = LLF-1; IF(JF*LLF>NV) GOTO 57
59    WRITE(6,'(/" Total number of factors NF is now set at ",A,".  ",
     +  "Hit RETURN if OK, or enter"/" another choice from alterna",
     +  "tives ",10(A,:", "))') CF(:JJF(NF)), (CF(:JJF(JF*I)),
     +  I=1,MIN(LF,10))
      IF(LF>10) WRITE(6,'(" or any other multiple of ",A," no larg",
     +  "er than ",A,".")') CF(:JJF(JF)), CF(:JJF(JF*LF))
      IF(JF*LLF>JF*LLF) WRITE(6,'(" *** WARNING: NF can be no larg",
     +  "er than ",A," or you will have more common"/15X,"factors ",
     +  "than items.")') CF(:JJF(JF*LLF))
      WRITE(6,'()')
      CALL SCAN(J,0,'I')
      IF(J==0) GOTO 60
      IF(J<0) GOTO 59
      READ(2,*) J
      NF = MAX(JF,MIN(MF,JF*NINT(J*1./JF)))
      GOTO 59
C
C Now repeat this basic simple-structure block throughout rest of pattern
60    KF = NF/JF
      KV = NV/JV
      IF(JF==NF) GOTO 65
62    WORD(:22) = 'second factor block '
      IF(KF>2) WORD(:22 ) = 'other '//CF(:JJF(KF-1))//' factor blocks'
      IF(.NOT.RBLK) WRITE(6,'(/" The master ",A,"-by-",A," subpatter",
     +  "n is set to repeat for the ",A/" WITHOUT row reordering. ",
     +  "Hit RETURN if OK, or enter anything for alternative.")')
     +  CF(:JJF(JV)), CF(:JJF(JF)), WORD(:22)
      IF(RBLK) WRITE(6,'(/" The master ",A,"-by-",A," subpattern is ",
     +  "set to repeat WITH row reordering for ",A," factor"/" blocks",
     +  " after the 1st. Hit RETURN if OK, or enter anything to waive",
     +  " reordering.")') CF(:JJF(JV)), CF(:JJF(JF)), CF(:JJF(KF-1))
      WRITE(6,'(5X,"Note: ""Row reordering"" means random permutation",
     +  " of rows separately"/5X,"in the ",A,"-by-",A," subpattern on",
     +  " each factor block after the 1st.")') CF(:JJF(NV)),
     +  CF(:JJF(JF))
      CALL SCAN(J,0,'B')
      IF(J==0) GOTO 65
      RBLK = .NOT.RBLK
      GOTO 62
65    IF(JV==NV) GOTO 70
      DO K = JV,NV-JV,JV
        DO I = 1,JV; DO J = 1,JF
           A(K+I,J) = A(I,J)
        END DO; END DO
      END DO
70    IF(JF==NF) GOTO 80
      IF(RBLK) THEN
        DO K = 2,KF  ! Randomly reorder all NV row indices
          CALL RANSEL(NV,NV,LWK)
          DO I = 1,NV
            ORDER(I,K) = LWK(I)
          END DO
        END DO
      ELSE
        DO I = 1,NV
          ORDER(I,1) = I
        END DO
      END IF
      DO K = 2,KF
        KK = JF*(K-1)
        L = K; IF(.NOT.RBLK) L = 1
        DO I = 1,NV; DO J = 1,JF
            A(I,KK+J) = A(ORDER(I,L),J)
        END DO; END DO
      END DO
C
C Randomize the salient loading magnitudes within interval [Lo, Hi] and make
C negative with probability Pr for each block of JF factors.

80     WRITE(6,'(/" This pattern''s salient raw loadings will be ran",
     +   "domized within size limits"/" [Lo, Hi] chosen separately f",
     +   "or each block of",I2," factors except that Hi = 0"/" recla",
     +   "ssifies salient loadings in this block as nonsalient rathe",
     +   "r than"/"strictly-zero salient.  (GENSCOR will later resca",
     +   "le each pattern row in"/" conformity to the factor correla",
     +   "tions and uniquenesses it generates.)"/" And the salient ",
     +   "loadings will be made"/" negative with a probability PR ",
     +   "set separately for each factor block.")') JF
      DO L = 1,KF
        IF(L==1) GOTO 85
        DO I = 1,4; B(I,L) = B(I,L-1); END DO
85      WRITE(6,'(/" Salient loadings on factors ",A," - ",A," now ",
     +    "have Lo/Hi size limits [",A4,",",A4,"]."/" Hit RETURN if ",
     +    "OK, or enter revised limits not over 1.0."/)') CF(:JJF(
     +    (L-1)*JF+1)), CF(:JJF(L*JF)), (CLN(B(I,L),4,-2),I=2,1,-1)
        CALL SCAN(J,0,'R')
        IF(J==0) GOTO 90
        IF(J<0) GOTO 85
        IF(J==2) READ(2,*) B1, B2
        IF(J==1) THEN; READ(2,*) B1; B2 = B1; END IF
        B1 = ABS(B1); B2 = ABS(B2)
86      IF(B1>1.0) THEN; B1 = B1/10; GOTO 86; END IF
        IF(B2>1.0) THEN; B2 = B2/10; GOTO 86; END IF
        B(1,L) = MAX(B1,B2); B(2,L) = MIN(B1,B2)
        GOTO 85
90      IF(B(1,L)==0.) THEN; J0 = (L-1)*JF
          DO J = 1,JF; DO I = 1,NV
             A(I,J0+J) = -A(I,J0+J)  ! Flag salients to be made nonsalient
          END DO; END DO
          B(3,L) = 0; GOTO 95
        END IF
        WRITE(6,'(" The probability that salient loadings on factors ",
     +    A," - ",A," will be made"/" negative is now Pr = ",A3,
     +    ". Hit RETURN if OK, or enter preferred Pr."/)') CF(:JJF(
     +    (L-1)*JF+1)), CF(:JJF(L*JF)), CLN(B(3,L),3,2)
        CALL SCAN(J,1,'R')
        IF(J==0) GOTO 95
        IF(J<0) GOTO 90
        READ(2,*) PR
92      IF(ABS(PR)>1.0) THEN
          PR = PR/10
          GOTO 92
        END IF
        B(3,L) = AMAX1(0.,AMIN1(1.,PR))
        GOTO 90
95      WRITE(6,'(" The size limit for nonsalient loadings on facto",
     +    "rs ",A," - ",A," is now W = ",A3,"."/" Hit RETURN if OK, ",
     +    "or enter preferred limit."/)') CF(:JJF((L-1)*JF+1)),
     +    CF(:JJF(L*JF)), CLN(B(4,L),3,2)
        CALL SCAN(J,1,'R')
        IF(J==0) CYCLE
        IF(J<0) GOTO 95
        READ(2,*) W
97      IF(ABS(W)>=1.0) THEN; W = W/10; GOTO 97; END IF
        B(4,L) = MAX(0.,MIN(1.,W))
        GOTO 95
      END DO
100   QY = .FALSE.
      WRITE(6,'(/" In review, your frame specifications for NV = ",
     +  A,"*",A," = ",A,", NF = ",A,"*",A," = ",A," are")')
     +  CF(:JJF(JV)), CF(:JJF(KV)), CF(:JJF(NV)),
     +  CF(:JJF(JF)), CF(:JJF(KF)), CF(:JJF(NF))
      CALL SHOWP(NF,JF,B,FMT,6)
      WRITE(6,'(/" Hit RETURN if OK, or enter anything to revise.")')
      CALL SCAN(J,0,'B')
      IF(J/=0) GOTO 80

C Generate pattern
      DO J = 1,NF
        L = (J-1)/JF + 1
        DO I = 1,NV
          M = 100; IF(A(I,J)<-.1) M = 10  ! Neg loading flags Hi = 0 salients
          IF(A(I,J)>.1) THEN  ! Salients
            A(I,J) = SIGN(B(2,L)+RANDY()*(B(1,L)-B(2,L)),RANDY()-B(3,L))
          ELSE   ! Distingish ordinary nonsalients from Hi=0 case
            A(I,J) = M + RANDY()*2*B(4,L) - B(4,L)
          END IF   ! ^ M added only to nonsalients distinguishes Hi=0 case
        END DO
      END DO

C Rem: JV,JF are blocksize; KF,KV are number of item,factor blocks
C Allow permutation of factor blocks differently within item blocks
      IF(KV==1 .OR. KF==1) GOTO 115  ! Block permutations pointless in this case
102   IF(QP) WRITE(6,'("  To permute blocks of factor loadings with",
     +  "in the pattern for selected item"/"  blocks, enter any let",
     +  "ter.  Otherwise, hit RETURN to waive this option or"/"  en",
     +  "ter any number to see an explanation of it.")')
      IF(.NOT.QP) WRITE(6,'("  To forego permuting blocks of factor",
     +  " loadings within the pattern for selected"/"  item blocks,",
     +  " enter any letter.  Otherwise, hit RETURN to activate this"/
     +  "  pattern-shift option or enter any number to see an expla",
     +  "nation of it.")')
      CALL SCAN(J,0,'R')
      IF(J<0 .AND. QP) GOTO 104
      IF(J<0 .AND. .NOT.QP) GOTO 115
      IF(J==0) THEN; QP = .NOT.QP; GOTO 102; END IF

c  To permute blocks of factor loadings within the pattern for selected item
c  blocks, enter any letter.  Otherwise, hit RETURN to waive this option or
c  enter any number to see an explanation of it.

c  To forego pemuting blocks of factor loadings within the pattern for selected
c  item blocks, enter any letter.  Otherwise, hit RETURN to activate this
c  pattern-shift option or enter any number to see an explanation of it.

C   | When your created factor blocks differ appreciably in their salient      |
C   | loadings' overall size, you can manifest the item blocks in clustering   |
C   | of their correlations by permuting factor-loading blocks differently     |
C   | over the item blocks.  For example, given two item blocks X1,X2 and      |
C   | two factor blocks F1,F2 with salients set high on F1 but low on F2,      |
C   | permuting X2's pattern [A21 A22] on <F1,F2> to [A22 A21] while retaining |
C   | the original pattern order [A11 A12] on X1 will induce item correlations |
C   | that are high within but low between item blocks X1 and X2.              |

      WRITE(6,'(3X,80A)') C11, (LIN,I=1,74), C12
      WRITE(6,'(3X,A," When your created factor blocks differ apprec",
     +  "iably in their salient",6X,A/3X,A," loadings'' overall size,",
     +  " you can manifest the item blocks in clustering",3X,A/3X,A,
     +  " of their correlations by permuting factor-loading blocks d",
     +  "ifferently",5X,A/3X,A," over the item blocks.  For exampl",
     +  "e, given two item blocks X1,X2 and",6X,A/3X,A," two factor ",
     +  "blocks F1,F2 with salients set high on F1 but low on F2,",6X,
     +  A/3X,A," permuting X2''s pattern [A21 A22] on <F1,F2> to [A2",
     +  "2 A21] while retaining ",A/3X,A," the original pattern orde",
     +  "r [A11 A12] on X1 will induce item correlations ",A/3X,A,1X,
     +  "that are high within but low between item blocks X1 and X2.",
     +  14X,A)') (BAR,I=1,16)
      WRITE(6,'(3X,80A)') C21, (LIN,I=1,74), C22
      WRITE(6,'()'); GOTO 102

cc104   IF(.NOT.ALLOCATED(PERM)) ALLOCATE ( PERM(KV,KF) )
104   ALLOCATE ( PERM(KV,KF) )
105   DO J = 1,KF; DO I = 1,KV
          PERM(I,J) = J
      END DO; END DO
106   WRITE(6,'(" Numeral N in row I and column J of this chart says ",
     +  "that the subpattern"/" created for item block I on factor bl",
     +  "ock J will move to cell N in this row."/" (Subpattern blocks",
     +  " for which N = J stay put.)"/)')  ! That is, PERM is a Put
CC      M = 16; IF(KF>9) M =12; IF(KF>12) M=8; IF(J>16) M=4
      M = 12; IF(KF>12) M=8; IF(J>16) M=4
      WORD = ' ';  CH4 = '   '; IF(KF>9) CH4 = ' '
cc      WRITE(6,'(A," Shift subpattern blocks to row positions ")')
      WRITE(6,'(A," Shift subpattern blocks in row to positions ")')
     +  WORD(:M)
      WRITE(6,'(A,"",14(""),"",30(""),"")') WORD(:M)
      DO I = 1,KV
        WRITE(6,'(A," Item block",A2," ",33A3)') WORD(:M), CLN(1.*I,
     +    2,10), (CLN(1.*PERM(I,J),3,10),J=1,KF), ('   ',J=1,
     +    MAX(1,10-KF)), CH4
      END DO
      WRITE(6,'(A,"",14(""),"",30(""),("")') WORD(:M)

C            Shift subpattern blocks in row to positions 
C           Ĵ
C            Item block 1   1  2  3  4  5  6  7  8  9 10
C            Item block 2                               
C           

108   WRITE(6,'(/" To change any row of this table of block-shift ins",
     +   "tructions, enter its"/" row index followed by its column in",
     +   "dices (integers 1 through ",A,") in the"/" order you want t",
     +   "he subpattern blocks to be assigned for this item block."/1X,
     +   "Otherwise hit RETURN to execute these subpattern shifts, or",
     +   " enter any"/" letter to start again."/)')
     +   CF(:JJF(KF))
      CALL SCAN(K,0,'I')
      IF(K==0) GOTO 111; IF(K<0) GOTO 106
      JJ = K-1; READ(2,*) II, (LWK(I),I=1,JJ)
      IF(II<1 .OR. II>KV) THEN
        WRITE(6,'(" ERROR: ",A," is not a row (item block) index.  ",
     +    "Try again.")') CF(:JJF(II)); GOTO 108
      END IF
      IER = 0
      DO I = 1,KF
        L = LWK(I); IF(L<1.OR.L>KF) IER = 1
        DO J = I+1,KF; IF(L==LWK(J)) IER = 1; END DO
        IF(IER>0) THEN
          WRITE(6,'(///)') ! Clear screen above error warning
          WRITE(6,'(" >>> ERROR: Your entry string after the row ind",
     +      "ex does not comprise just"/12X,"the integers from 1 to ",
     +      A,".  Try again."/)') CF(:JJF(J-1)); GOTO 106
        END IF
      END DO
      DO J = 1,KF
        PERM(II,J) = LWK(J)
      END DO
      GOTO 106
111   QP = .FALSE.
      DO J = 1,KF; DO I = 1,KV
          IF(PERM(I,J)/=J) QP = .TRUE.
      END DO; END DO
      IF(.NOT.QP) THEN
        WRITE(6,'("  No block shifts are pending.  If that is your ",
     +    "intent, hit RETURN to finish."/"  Otherwise, enter anyth",
     +    "ing to try setting those again.")')
        CALL SCAN(J,0,'B')
        IF(J==0) GOTO 115; GOTO 106
      END IF
      WRITE(6,'(" Enter anything to confirm that the tabled subpatte",
     +  "rn relocations are what"/" you want.  Otherwise, hit RETURN",
     +  " to make changes.")')
      CALL SCAN(J,0,'B')
      IF(J==0) GOTO 106
      ALLOCATE ( BLOK(JV,NF) )

C Execute block permutations stipulated in PERM
      DO IB = 1,KV; I0 = (IB-1)*JV
        DO JB = 1,KF; J0 = (JB-1)*JF; JP0 = (PERM(IB,JB)-1)*JF
          DO I = 1,JV; DO J = 1,JF
            BLOK(I,JP0+J) = A(I0+I,J0+J)
          END DO; END DO
        END DO
        DO JB = 1,KF; J0 = (JB-1)*JF
          DO I = 1,JV; DO J = 1,JF
            A(I0+I,J0+J) = BLOK(I,J0+J)
          END DO; END DO
        END DO
      END DO
C Define item names coding salient loadings and delete flags from pattern
      ALLOCATE ( IDENT(NV), NAML(NV) )
      DO I = 1,NV; IDENT(I)(:1) = '@'; K = 0
        DO J = 1,NF  ! All salients are under 2, all nonsalients over 4
          IF(A(I,J)<3 .AND. J<27) THEN
            K=K+1; IDENT(I)(K:K) = CHAR(64+J)
          END IF
          NAML(I) = K
        END DO
      END DO
      DO I = 1,NV; DO J = 1,NF
          IF(A(I,J)>50) A(I,J) = A(I,J) - 100
          IF(A(I,J)>5) A(I,J) = A(I,J) - 10
      END DO; END DO

C Store in formatted file P?{Kode}
115   CALL CODE(CH4,JF,KF,KV,KMP,MB)
      N = 48+KMP(1)
      IF(KMP(1)>=10) N = 55+KMP(1)
      J = 0
120   J = J+1  ! Provisional repetition count
      IF(JG==0) K = 48+J
      IF(JG>0) K = 64 + JG + 5*(J-1)
      F1 = 'P'//CHAR(N)//CH4//CHAR(K)
C       2nd character in filename codes number KMP(1) of complexity-1
C       recurrences, 7th character codes repetition count combined with JG.
      INQUIRE(FILE=F1,EXIST=QY)
      IF(QY .AND. J<5) GOTO 120
      OPEN(7,FILE=F1)
      K = JJF(KMP(1)); WORD = 's ('//CF(:K)//' of 1st)'
      M = 11+K; IF(KMP(1)==0) M = 1
      WRITE(7,'(" Raw pattern ",A," for ",A," variables (",A," blocks",
     +  ") on ",A," factors (",A," blocks)")') F1, CF(:JJF(NV)),
     +  CF(:JJF(KV)), CF(:JJF(NF)), CF(:JJF(KF))
      WRITE(7,'(" Row randomizing = ",L1,"; Complexity level",2A,
     +  10(I2,:","))') RBLK, WORD(:M), ':', (LST(I),I=1,LC)
      WRITE(7,'(" General factors in block:",I2)') JG
      WRITE(7,'(/7X,"Pattern parameters")')
      CALL SHOWP(NF,JF,B,FMT,7)
C             Pattern parameters
C    Ŀ
C        Factors  : xx-xx  xx-xx  xx-xx         
C    Ĵ
C     Hi Salient    .xx    .xx    .xx          
C     Lo Salient  -----**-----**----- |        
C     PrNeg Sal.                               
C     Hi nonSal.                               
C    

      IF(QP) THEN
        WRITE(7,'(7X,"Subsequent permutation of factor blocks within",
     +    " item blocks:")')
        WORD = ' ';  CH4 = '   '; IF(KF>9) CH4 = ' '
        WRITE(7,'(4X,"",14(""),"",99A)') ('',J=1,MAX(27,3*KF)),''
CC        WRITE(7,'(4X,"",14(""),"",27(""),"")')
        DO I = 1,KV
          WRITE(7,'(4X," Item block",A2," ",30A3)') CLN(1.*I,2,10),
     +      (CLN(1.*PERM(I,J),3,10),J=1,KF), ('   ',J=1,MAX(1,9-KF)),
     +       CH4
        END DO
        WRITE(7,'(4X,"",14(""),"",99A)') ('',J=1,MAX(27,3*KF)),''
CC        WRITE(7,'(4X,"",14(""),"",27(""),"")')
      END IF
      WRITE(7,'(/" Loadings are rounded to three decimals with ",
     +  "point omitted.")')
      DO I = 1,NV
        WRITE(7,'(50I6)') (NINT(1000*A(I,J)),J=1,NF)
      END DO
      WRITE(7,'(/" Name stems: ")'); LL = 0; NL = 1
      DO I = 1,NV
        LL = LL + 1 + NAML(I)  ! Accumulating line length
        IF(NL>76 .OR. I==NV) THEN
          WRITE(7,'(2X,40(1X,A))') (IDENT(J)(:NAML(J)),J=NL,I)
          LL = 0; NL = I+1
        END IF
      END DO
150   OPEN(3,FILE='LASTPAT')
      X=0.; Y=0.; IF(QP) X=1.; IF(RBLK) Y=1.
      WRITE(3,'(2I4,50A6))') JF, KMP1, (CLN(B(I,1),6,3),I=1,4),
     +  CLN(X,6,1), CLN(Y,6,1)
      CLOSE(3)
C
      IF(ALLOCATED(IDENT)) DEALLOCATE ( IDENT, NAML )
      WRITE(6,'(/" This raw simple-structure pattern has been stored",
     +  " in ASCII file ",A,"."/" To create another, enter anything.",
     +  "  Otherwise, hit RETURN to quit.")') F1
      CALL SCAN(J,1,'B')
      IF(J/=0) THEN
        CALL SYSTEM('cls')
        WRITE(6,'(//6X,65A)') C11,(LIN,I=1,63),C12
C        WRITE(6,'(//6X,"",63(""),"")')
        WRITE(6,'(6X,A,9X,"Generating another raw common-factor ",
     +    "pattern",10X,A)') BAR, BAR
        WRITE(6,'(6X,65A)') C21,(LIN,I=1,63),C22
C        WRITE(6,'(6X,"",63(""),"")')
        GOTO 200
      END IF
      STOP
      END
C
      SUBROUTINE CODE(CH,JF,NFB,NVB,KMP,MB)
C First, convert JF,NFB,NVB into two-letter coding CH(:2) of Block-code KBL =
C 100*(JF-1) + 10*(NFB-1) + (NVB-1) with single-digit constraints on JF-1,
C KF-1, and JV-1. Next, convert string of MB 0/1 KMP(J2) terms into 2-digit
C binary-code integer KC, and write this in ASCII in CH(3:4).
      CHARACTER CH*(*)
      INTEGER KMP(*)
      IF(JF>7 .OR. JF<1 .OR. NFB>10 .OR. NFB<1 .OR.
     +   NVB>10 .OR. NVB<1) THEN
        WRITE(6,'(/" One of your pattern-block specifications is out",
     +    " of bounds.")')
        STOP
      END IF
      KD = 100*(JF-1) + 10*(NFB-1) + (NVB-1)
      CH(1:1) = CHAR(65+KD/26)
      CH(2:2) = CHAR(65+MOD(KD,26))
      KC = 0
      DO I = 2,MB
        KC = KC + MIN(1,KMP(I))*2**(I-2)
      END DO
      CH(3:4) = CHAR(48+MOD(KC/10,10))//CHAR(48+MOD(KC,10))
      RETURN
      END
C
      FUNCTION JJF(N)
C Return character expression of integer N left-justified in field CF; then
C CF(:JJF(N)) writes N with exactly the right length in format specifier A.
C *** Haven't found any way to avoid requiring N to be INTEGER(4).
      CHARACTER(12) CF
      INTEGER(4) K
      COMMON /CF/ CF
C      SAVE /CF/  ! This doesn't appear to be needed
      K = ABS(N)
      CF = '            '
      J = 13
10    J = J-1
      CF(J:J) = CHAR(48+MOD(K,10))
      K = K/10
      IF(K>0) GOTO 10
      IF(N<0) CF(J-1:J-1) = '-'
      CF = ADJUSTL(CF)
      JJF = LEN_TRIM(CF)
      END FUNCTION
C
      FUNCTION CLN(X,NFF,ND)
C Express real number X as a character string in fieldwidth NF with LD decimals
C when room, where LD = ABS(ND).  ND<0 tries to precede all positive numbers
C with a blank. LD > 9 displays zero decimals and no decimal point while
C returning X = 0 as ND-10 zeros right-justified in selected field.
C ***** WARNING: If a call of CLN prints garbage, you have forgotten to
C       declare CLN as CHARACTER*8 in the calling routine.
      PARAMETER (KW=16)
      CHARACTER CLN*8, WK(KW)
      CLN = '        '
      NF = MIN(8,NFF)  ! Limit fieldwidth to 8 chars
      LD = ABS(ND)
      IF(ABS(X)<1.0E-12) THEN !  Special for vanishingly small X
        N = MIN(NF-1,MOD(LD,10))
        IF(LD<10) CLN(NF-N:NF+1-N) = '.0'
        IF(LD<10) RETURN
        DO I = 1,N
          CLN(NF+1-I:NF+1-I) = '0'
        END DO
        RETURN
      END IF
      M = NF; IF(ABS(X)>1.) M = NF-1-INT(LOG10(ABS(X))) !  M is space free for decimal (or M-1 if X<0) )
      IF(M<0 .OR.  M<1.AND.X<0.) GOTO 55
      IF(ND>=10) LD = 0; LD = MIN(LD,M)
      DO I = 1,KW
        WK(I) = ' '
      END DO
      IF(LD==0) N = NINT(ABS(X))         ! Shouldn't be needed, but is
      IF(LD/=0) N = NINT(ABS(X)*10**LD)  ! This can overflow if LD is large
      DO I = KW,KW-LD+1,-1
        WK(I) = CHAR(48+MOD(N,10))
        N = N/10
      END DO
      WK(KW-LD) = '.'
      IF(N==0 .AND. LD==0) WK(KW-LD-1) = '0'
      IF(N==0 .AND. X<0. .AND. LD>0) WK(KW-LD-1) = '-'
      DO I = KW-LD-1,2,-1
        IF(N>0) WK(I) = CHAR(48+MOD(N,10))
        IF(N>0 .AND. X<0.) WK(I-1) = '-'
        N = N/10
      END DO
      KS = KW-NF+1
25    KS = KS-1
C       Field returned will start at WK(KS+1) for initial KS
      IF(KS<=0) GOTO 50
      IF(ND>=0) THEN
        IF(WK(KS)/=' ' .AND. WK(KS+NF)/='.') GOTO 25
      ELSE
        IF(WK(KS+1)/=' '. AND. WK(KS+1)/='-' .AND. WK(KS+NF)/='.')
     +    GOTO 25
      END IF
      IF(ND>=10) KS = MAX(1,KS-1)
      DO I = 1,NF
        CLN(I:I) = WK(KS+I)
      END DO
50    IF(WK(KS)==' ') RETURN
55    CLN = ' *******'
      END FUNCTION
C
      FUNCTION KOMB(K,L0)
C This returns the number of combinations of K things taken L0 at a time.
C If L0 is negative, KOMB returns the number of permutations of K things.
      L = MIN(L0,K-L0)
      IF(L0<0) L = K
      KOMB = KMB(K,L)
      IF(L0<0 .OR. K<=0) RETURN
      KOMB = KOMB/KMB(L,L)
      RETURN
      END
C
      FUNCTION KMB(K0,M)
C Return the number of ordered M-selections from K0 things.
      KMB = 1
      IF(K0<=0 .OR. M<=0 .OR. M>K0) RETURN
      K = K0
10    KMB = KMB*K
      K = K-1
      IF(K>K0-M) GOTO 10
      RETURN
      END
C
      SUBROUTINE RANSEL(NS,NT,LSET)
C This randomly selects NS of integers 1 to NT, and returns the selected
C sequence in array LSET.
C ***** A more expansive version of this subroutine is in file COVSCOR.FOR
C      INTEGER LSET(*), LTAG(*)
      INTEGER LSET(*)
      DO I = 1,NT
        LSET(I) = I
      END DO
      IF(NT<NS) THEN
        WRITE(6,'(/" *** ERROR: The field of integers sent to RAN",
     +   "SEL is smaller than the selection count.")')
        STOP
      END IF
      DO I = 0,NS-1
        I1 = I+1
        K = MIN(NT,I1+INT((NT-I)*RANDY()))
        L = LSET(K)
        LSET(K) = LSET(I1)
        LSET(I1) = L
      END DO
      RETURN
      END
C
      SUBROUTINE SCAN(NL,NS,SEQ)
C This reads the keyboard string, cleans it for list-directed reading of
C the numbers therein, and checks whether it contains NS integers/reals in
C the sequence of Is and Rs received in SEQ if NS>0, or, if NS<1, whether
C all its numbers are of the first I/R kind listed in SEQ. (Integers are
C accepted also as reals.  Termination of an input line by "*" allows up to
C two continuation lines.) NL returns 0 if the input string is blank, -1 if
C this contains only non-numeric characters, -2 if the cleaned number
C string returned in File 2 is non-null but does not match SEQ, and
C gives the total count of numbers in the returned string otherwise.
C *** If SEQ is "B", NL returns 0 if the input line is blank, and
C     returns -1 otherwise.
C
      CHARACTER  AA, SEQ*(*), WA*240, WB*240
      NL = 0
5     NLL = NL + 80
      READ(5,'(A80)') WA(NL+1:NLL)
      IP = NL
      NL = NLL+1
10    NL = NL-1
      IF(NL==0) RETURN
      IF(WA(NL:NL)==' ') GOTO 10
      IF(WA(NL:NL)=='*' .AND. NL>IP) GOTO 5
      IF(SEQ(1:1)=='B') NL = -1
      IF(SEQ(1:1)=='B') RETURN
      IP = 0
      WB(NL+1:NL+1) = ' '
      DO 20 I = 1,NL
       WB(I:I) = ' '
       IF(WA(I:I)<CHAR(48) .OR. WA(I:I)>CHAR(57)) GOTO 20
       WB(I:I) = WA(I:I)
       WA(I:I) = '0'
20     CONTINUE
      IP = 0
      IF(WA(:3)=='-.0') WB(:1) = '-'
      IF(WA(:2)=='-0') WB(:1) = '-'
      IF(WA(:3)=='-.0') WB(:1) = '-'
      IF(WA(:2)=='.0') WB(:1) = '.'
      IF(WA(:2)=='.0') IP = 1
      DO I = 2,NL
        IF(WA(I:I)=='-') THEN
          IF(WB(I-1:I-1)==' ' .AND. (WA(I+1:I+1)=='.'.OR.WA(I+1:I+1)
     +      =='0')) WB(I:I) = '-'
          IP = 0
        ELSE IF (WA(I:I)=='.') THEN
          IF((WA(I-1:I-1)=='0'.OR.WA(I+1:I+1)=='0') .AND. IP==0)
     +      WB(I:I) = '.'
          IF(WB(I:I) == '.') IP = 1
        ELSE IF (WA(I:I) /= '0') THEN
          IP = 0
        END IF
      END DO
C Identify Integer/Real sequence in WB and put in start of WA
      AA = 'I'
      IF(WB(1:1)=='.') AA = 'R'
      NN = 0
      DO I = 2,NL+1
        IF(WB(I:I)==' ' .AND. WB(I-1:I-1)/=' ') THEN
          NN = NN + 1
          WA(NN:NN) = AA
          AA = 'I'
        ELSE IF (WB(I:I)=='.') THEN
          AA = 'R'
        END IF
      END DO
      IF(NN==0) NL = -1
      IF(NN==0) RETURN
      AA = '+'
      IF(NS<=0 .AND. SEQ(1:1)=='R') GOTO 60
      IF(NS<=0) GOTO 50
      IF(NN<NS) GOTO 57
      DO I = 1,NS
        IF(SEQ(I:I)=='I' .AND. WA(I:I)/='I') AA = '0'
      END DO
      GOTO 55
50    DO I = 1,NN
        IF(WA(I:I)/=SEQ(1:1)) AA = '0'
      END DO
55    IF(AA=='+') GOTO 60
57    NL = -2
      IF(NL<=-2) WRITE(6,'(/" Your input does not contain the ",
     +  "number sequence requested. Try again.")')
      RETURN
60    REWIND 2
      WRITE(2,'(A240)') WB(1:NL)
      NL = NN
      REWIND 2
      RETURN
      END
C
      SUBROUTINE SHOWP(NF,JF,B,FMT,KF)
C Write frame-parameter specifications to file KF
      CHARACTER FMT*(*), CLN*8, CH1*1
      REAL B(4,*)
      NFB = NF/JF
      CH1 = CHAR(48+NFB)
      FMT = '(4X,"",'//CH1//'(""),'//'"")'
      WRITE(KF,FMT)
      FMT = '(4X,"    Factors  :",'//CH1//'(I3,"",I2,'//'1X),"")'
      WRITE(KF,FMT) ((J-1)*JF+1,J*JF,J=1,NFB)
      FMT = '(4X,"",'//CH1//'(""),'//'"")'
      WRITE(KF,FMT)
      FMT = '(4X," Hi Salient  ",'//CH1//'(A5,2X),"")'
      WRITE(KF,FMT) (CLN(B(1,J),5,-2),J=1,NFB)
      FMT = '(4X," Lo Salient  ",'//CH1//'(A5,2X),"")'
      FMT(8:17) = 'Lo Salient'
      WRITE(KF,FMT) (CLN(B(2,J),5,-2),J=1,NFB)
      FMT = '(4X," PrNeg Sal.  ",'//CH1//'(A5,2X),"")'
      FMT(8:17) = 'PrNeg Sal.'
      WRITE(KF,FMT) (CLN(B(3,J),5,-2),J=1,NFB)
      FMT = '(4X," Hi nonSal.  ",'//CH1//'(A5,2X),"")'
      FMT(8:17) = 'Hi nonSal.'
      WRITE(KF,FMT) (CLN(B(4,J),5,-2),J=1,NFB)
      FMT = '(4X,"",'//CH1//
     +  '(""),'//'"")'
      WRITE(KF,FMT)
      RETURN
      END
C
      SUBROUTINE SORT(LIST,N,N1,N2)
C This receives in LIST an unordered list of N integers; returns in LIST
C the NN distinct ones in range <N1,N2>, and also resets N = NN.
      INTEGER LIST(*)
      L = 0
      OUTER: DO I = N1,N2
        DO J = L+1,N
          IF(I==LIST(J)) THEN
            L = L+1
            T = LIST(L)
            LIST(L) = I
            LIST(J) = T
            CYCLE OUTER
          END IF
        END DO
      ENDDO OUTER
      N = L
      END SUBROUTINE
C
      FUNCTION RANDY()
C Use the compiler's random generator for a random number in the unit interval
      DATA NEW/1/
      IF(NEW>0) THEN
        NEW = 0
        CALL RANDOM_SEED()
      END IF
      CALL RANDOM_NUMBER(RANDY)
      END FUNCTION
C
      SUBROUTINE WAIT(N)
C  N > 0 calls for N spaces before screen display
      IF(N>0) WRITE(6,'()')
      WRITE(6,'(" Hit RETURN to continue")')
      READ(5,'(A1)')
      RETURN
      END

