C    Program GENSCOR   (Source code, Fortran-90. EIG must also be loaded.)

C                 Last revised:  10 September 2003

C This routine creates a rawdata array (a population of scores on both data
C variables and source factors) from a raw simple-structured factor pattern
C direct from GENPAT.  Recently (October, 2002) it has been upgraded to
C accept SCHEMAS-created schemata for the factor pattern and correlations
C (filenames PAT<i>.<j>, COV<i>.<j>, where <i> is an integer identifying the
C matrice's dimensionality with <j> a list index), as well as covariances
C created previously (filenames KOV<i>.<j>) by SEMCOV.

C *** Twist shaping is explained in the subroutine TWST source code

C  Input GENPAT pattern has name P?ija, where ? is number of complexity-1
C  recurrences in the master pattern block or letter coding thereof if 10,
C   is two-letter code for <JF,NFB,JV> (see GENPAT), ij is two-digit code
C  for complexity levels  2, and a is either a sequential numeric index for
C  repetitions of frame P?ij if JG = 0 or a letter jointly coding JG and
C  number of P?ij repetitions.  (JG is number of general factors in the
C  master pattern block.)
C
CC  [Formerly,] Main output is unformatted rawdata file P?ijab.DAT for the
CC  lowest unused sequential letter index b.  And information about the source
CC  structure is written to ASCII file P?ijab.LOG. [ <- Output of old version;
CC  present output is more abundant.]

C  Programming singular/plural in text.  Declare CHARACTER PL*2, DATA PL/'s ';
C    then PL(3-MIN(2,N):) prints ' ' if N=1 and 's ' if N>1

C File 16 collects text output. F1 uploads a pattern and later  (???)
C F2 gets the LOG-file.  F3 uploads *.COV and later opened as 7 to write *.POP

      PARAMETER (MB=7,ML=40)
      LOGICAL QT, QL, QSCOR, QD, QC   !, QS
      CHARACTER*12 F1, F2, F3, WORD*200, CLN*8, NAME(ML), CF
      CHARACTER CH1, C1, PL1*2, PL2*2, FMT*20
      INTEGER ORT  !  See Labels 355, 360
      REAL LOW, TP(2,4)
      CHARACTER,ALLOCATABLE :: IDENT(:)*26, CWRD(:)*1, WRD(:)*170
      INTEGER,ALLOCATABLE :: LIST(:), LST(:), PERM(:,:)
      REAL,ALLOCATABLE :: CFF(:), CM(:), CV(:), P(:,:), P1(:,:),
     +     PP(:,:), R(:), T(:,:), TWIST(:,:), U(:), W(:)   ! , SV(:)
C       TP(i, ) contains TWIST parameters RAi,RBi,Pi for skewing score dists
C       TWIST(j, ) contains the generated individual-factor twist parameters
C         in cols 1-3, and their skew/kurtosis in cols 4/5

      COMMON /CF/ CF    ! Companion function JF here named JJF
      LO(I,J) = J*(J-1)/2 + I
      LOC(I,J) = LO(MIN(I,J),MAX(I,J))
      DATA NS/5000/, B0,B1,PC/.5,.9,60./, BL,BH,LOW/-.2,.6,.2/
      DATA ORT,QSCOR/1,.TRUE./,  TP/1.,1.,.5,1.,1.,1.,.5,5./  ! Twist parameters
      DATA LLEN,NND,KLT,LWRP/150,3,1,1/ ! Cov-formatting defaults
      OPEN(2,STATUS='SCRATCH') ! KND=1/2 flags schema/genpat pattern upload

CC TEST LINES
C      OPEN(21,FILE='JUNK')

      INQUIRE(FILE='KOVFMT',EXIST=QT)
      IF(QT) THEN ! >>> Blank entry is read as 0 before branching to ERR
        OPEN(7,FILE='KOVFMT')
        READ(7,*,ERR=5) M; LLEN = MAX(10,MIN(240,M))
        READ(7,*,ERR=5) M; NND = MAX(3,MIN(5,M))
        READ(7,*,ERR=5) M; KLT = MAX(0,MIN(1,M))
        READ(7,*,ERR=5) M; LWRP = MAX(0,MIN(1,M))
        GOTO 7
      END IF
5     QT = .FALSE.
7     CLOSE(7)
      MXL = LLEN/(NND+2)  !  Code for reading/writing COV-files
      FMT = '('//CF(:JJF(MXL))//'I'//CF(:JJF(NND+2))//')'
      CALL SYSTEM('cls') ! ^Width needs NND+1 for number, plus 1 for space
      IF(.NOT.QT) THEN
        WRITE(6,'(" ****** WARNING: Parameter file KOVFMT containing",
     +    " your override of"/7X,"Gendata''s default COV-file formatt",
     +    "ing is missing or corrupt."/7X,"Unless you''re sure that t",
     +    "he default format can be read by"/7X,"your SE-solution pro",
     +    "cedure, hit RETURN to exit and run Gendata"/7X,"utility pr",
     +    "ogram COVFORM.  Otherwise, enter anything to continue.")')
        CALL SCN(J); IF(J==0) STOP
      END IF
      WRITE(6,'(9X,"",57(""),"")')
      WRITE(6,'(9X," GENSCOR generates simulation data having a st",
     +  "ipulated   "/9X," factor structure.  If you are unfamili",
     +  "ar with its       "/9X," operation, enter anything to se",
     +  "e a description of this. "/9X," Otherwise, hit RETURN to",
     +  " get on with production.        "))')
      WRITE(6,'(9X,"",57(""),""/)'); KK = 1
      CALL SCN(J); IF(J==0) GOTO 15
c       GENSCOR creates a simulation population of centered, unit-norm scores
c   jointly on NY data variables Y and NF common factors F satisfying SE model
c
c                Y = WF + E,  Cyy = WCffW' + De
c
c   for a pattern W and factor correlations Cff you have sketched previously.
c   (Cyy,W,Cff,De will also be filed; and should you want only those, score
c   production can be waived.)  More precisely, W in output will be the raw
c   input pattern's rescaling entailed by standardizing Y in accord with the
c   received W and Cff after assignment here of measurement noise De.  You will
C   also get options to depart from certain distribution ideals classically
C   posited by factor analysis and more recently structural modelling, namely,
C   that in the population from which an extant dataset is hypothized to be a
C   random sample, the joint distribution of all variables at issue is Normal,
c   while unique (error, noise) factors are strictly orthogonal among
C   themselves and to the common factors.
C                         [wait]
C     Score production starts here with an [NS records]-by-[NF+NY variables]
C array of numbers independently sampled from a unit-Normal random generator.
C After certain optional adjustments, NF dimensions of this random-score array
C are rotated and rescaled to become the common factors on which the created
C data scores have pattern W, while after suitable rescaling the remaining NY
C random-score axes become unique (noise) factors.  If adjustments are waived,
C the resulting joint distribution of common and unique factors is nearly unit-
C Normal with each unique factor nearly orthogonal to all the other created
C factors.  "Nearly" converges to "exactly" as sample size NS becomes huge;
C however, exact orthogonality even with modest NS is an option that can be
C imposed by partialling should you want it.  The other adjustment available
C here - "twist" - is giving the common factors and, separately, unique
C factors degrees of skew and kurtosis that can vary from mild to extreme.
C But twist is imposed on the initially randomized axes and will be degraded
C by subsequent compositing of those.  Exact orthogonalization does not much
C matter so long as sample size is decently large, but if you want strongly
C twisted common factors you should impose only weak correlations on them
C                        [wait]

11     WRITE(6,'(7X,"GENSCOR creates a simulation population of center",
     +  "ed, unit-norm scores"/3X,"jointly on NY data variables Y and",
     +  " NF common factors F satisfying SE model"//16X,"Y = WF + E,",
     +  "  Cyy = WCffW'' + De"//3X,"for a pattern W and factor corr",
     +  "elations Cff you have sketched previously."/3X,"(Cyy,W,Cff,De",
     +  " will also be filed; and should you want only those, score"/3X,
     +  "production can be waived.)  More precisely, W in output will ",
     +  "be the raw"/3X,"input pattern''s rescaling entailed by stand",
     +  "ardizing Y in accord with the"/3X,"received W and Cff after ",
     +  "assignment here of measurement noise De.  You will"/3X,"also ",
     +  "get options to depart from certain distribution ideals class",
     +  "ically"/3X,"posited by factor analysis and more recently stru"
     +  "ctural modelling, namely,"/3X,"that in the population from w",
     +  "hich an extant dataset is hypothesized to be a"/3X,"random s",
     +  "ample, the joint distribution of all variables at issue is N",
     +  "ormal"/3X,"while unique (error, noise) factors are strictly o"
     +  "rthogonal among"/3X,"themselves and to the common factors.")')
      CALL WAIT(1)
      WRITE(6,'(6X,"Score production starts here with an [NS records",
     +  "]-by-[NF+NY variables]"/" array of numbers independently sa",
     +  "mpled from a unit-Normal random generator."/" After certain",
     +  " optional adjustments, NF dimensions of this random-score a",
     +  "rray"/" are rotated and rescaled to become the common facto",
     +  "rs on which the created"/" data scores have pattern W, whil",
     +  "e after suitable rescaling the remaining NY"/" random-score",
     +  " axes become unique (noise) factors.  If adjustments are wai",
     +  "ved,"/" the resulting joint distribution of common and uniqu",
     +  "e factors is nearly unit-"/" Normal with each unique factor ",
     +  "nearly orthogonal to all the other created"/" factors.  ""N",
     +  "early"" converges to ""exactly"" as sample size NS becomes ",
     +  "huge;"/" however, exact orthogonality even with modest NS is",
     +  " an option that can be"/" imposed by partialling should you ",
     +  "want it.  The other adjustment available"/" here - ""twist",
     +  """  - is giving the common factors and, separately, unique"/
     +  " factors degrees of skew and kurtosis that can vary from mil",
     +  "d to extreme."/" But twist is imposed on the initially rando",
     +  "mized axes and will be degraded by"/" subsequent compositing",
     +  " of those.  Exact orthogonalization does not much"/" matter ",
     +  "so long as sample size is decently large, but if you want st",
     +  "rongly"/" twisted common factors you should impose only weak",
     +  " correlations on them.")')
      CALL WAIT(1)
15    WRITE(6,'(3X,"The raw factor pattern needed here for score pro",
     +  "duction can be created"/3X,"either by GENPAT with a more or",
     +  " less complex block structure, or by"/3X,"SCHEMAS as a free",
     +  "-form template whose parameters can be reset here.")')
17    IF(KK==1) WRITE(6,'(/6X,"To look for free-form schemata, enter",
     +  " anything.  Otherwise,"/6X,"hit RETURN to consider the alte",
     +  "rnative.")')
      IF(KK==0) WRITE(6,'(/6X,"To see what GENPAT patterns are avail",
     +  "able, enter anything."/6X,"Otherwise, hit RETURN to conside",
     +  "r the alternative.")')
      CALL SCN(J)
      IF(J==0) THEN; KK = 1-KK; GOTO 17; END IF
      IF(KK==0) GOTO 35

c   To list the available free-form schemata of NY-by-NF factor patterns, enter
c   integers NY and smaller NF.  Otherwise enter any letter to exit GENSCOR
c   and run SCHEMAS to make one, or hit RETURN to repeat the introduction.

20    WRITE(6,'(3X,"To list the available free-form schemata of NY-by",
     +  "-NF factor patterns, enter"/3X,"integers NY and smaller NF. ",
     +  " Otherwise enter any letter to exit GENSCOR"/3X,"and run SCHE",
     +  "MAS to make one, or hit RETURN to repeat the introduction."/)')
      CALL SCANB(J,0,'I',5,WORD,0)
      IF(J<0) STOP; IF(J==0) GOTO 11
      IF(J==1) THEN
        WRITE(6,'(" One number isn''t enough.  Try again.")'); GOTO 20
      END IF   ! Tmp use of TP
      READ(2,*) (TP(I,1),I=1,2); M = NINT(TP(1,1)); N = NINT(TP(2,1))
      NV = MAX(ABS(M),ABS(N)); NF = MIN(ABS(M),ABS(N))
      IF(NV==NF) THEN
        WRITE(6,'(" NY must be larger than NF.  Try again."/)')
        GOTO 20
      END IF

C ************ Start PAT-schema upload ************
24    F1 = 'PAT'//CF(:JJF(NV))//CF(:JJF(NF))//'.*'
ccc      LDOT = LAST(F1)-1; N = -1  !!  N = -41  ! ABS(N) > maxlist calls special filter
      LDOT = LAST(F1)-1; N = -41  ! ABS(N) > maxlist calls special filter
      CALL LOOK(1,F1,NAME,40,N)   ! Neg N suppresses screen display
      IF(N==0) THEN               !
        WRITE(6,'(" >>>> No ",A,"-by-",A," pattern schema with basena",
     +    "me ",A," is available here."/6X,"Call program SCHEMAS and ",
     +    "make one.")') CF(:JJF(NV)), CF(:JJF(NF)), F1(:LDOT-1); STOP
      END IF
      WRITE(6,'(" Pattern schemata responding to your call here are")')
      NLST = 41     ! Flagging by ABS(NLST) > maxlst remains necessary
      CALL LOOK(1,F1,NAME,40,NLST)   ! NAME(NLST) is last in list
27    IF(NLST==1) WRITE(6,'(/4X,"To inspect and maybe use this schem",
     +  "a, hit RETURN.  Otherwise, enter"/4X,"any letter to quit a",
     +  "nd call SCHEMAS to make another.")')
30    IF(NLST>1) WRITE(6,'(/4X,"To upload one of these schemata to i",
     +  "nspect and maybe use, enter its"/4X,"list index.  Otherwise",
     +  ", enter any letter to abort this run."/)')
      CALL SCANB(J,0,'I',5,C1,0); IF(J<0) STOP
      IF(NLST==1) THEN
        IF(J>0) GOTO 27
        F1 = NAME(1)
      ELSE
        IF(J==0) GOTO 27
        READ(2,*) NN; NN = MAX(1,MIN(NN,NLST))
        F1 = NAME(NN)
        WRITE(6,'(5X,"Enter anything to confirm retrieval of schema ",A,
     +    "."/5X,"Otherwise, hit RETURN to reconsider.")') F1(:LAST(F1))
        CALL SCN(J); IF(J==0) GOTO 30
      END IF  !  Input from GENPAT puts the raw pattern in PP
      KND = 1   ! Schema pattern
      IF(ALLOCATED(PP)) DEALLOCATE ( PP, T );
      ALLOCATE ( PP(NV,NF), T(NV,NF) )    ! Schema pattern; extra column for
      CLOSE(15); OPEN(15,FILE='WXYZ'); MSG = -1             ! salients count
      CALL SKEME(NV,NF,F1,NLST,PP,NV,FUZ,CUT,MSG) ! Raw pattern returned in PP
      IF(MSG/=0) THEN  ! End of pattern-schema installation
        WRITE(6,'(4X,"Enter anything to redisplay the schema list ",
     +    "for another pick."/4X,"Otherwise, hit RETURN to quit.")')
          CALL SCN(J); IF(J==0) STOP; GOTO 24
      END IF
      WRITE(15,'(/" The raw pattern derived from this schema under ",
     +  "these parameters was")')
      DO I = 1,NV
        WRITE(15,'(I3,". ",4(5A5,2X),4(:/5X,4(5A5,2X)))')
     +    I, (CLN(PP(I,J),5,-2),J=1,NF)
        IF(MOD(I,5)==0 .AND. I<NV) WRITE(15,'()')
      END DO; KK = 0
      GOTO 70

C *********** Start of GENPAT pattern upload ************
C  Filter out patterns not having the stipulated NV and NF
35    NN = -1 ! Suppress screen output from LOOK
      CALL LOOK(1,'P*',NAME,40,NN)  ! PAT* excluded by special filter in LOOK
      IF(NN==0) THEN
        WRITE(6,'(/3X,"No GENPAT products here.  Hit RETURN to look ",
     +    "for a pattern schema,"/3X,"or enter anything to quit.")')
        CALL SCN(J); IF(J==0) STOP; GOTO 20
      END IF
      IF(.NOT.ALLOCATED(LIST)) ALLOCATE ( LIST(NN) )
C     Ŀ
C        #   P-name    NY    NF   blocksize   complexities  
C     ͵
C       12.  P2AB12C  100    15     25 x 5    1 2 3 4 5 6 7 
C     
      WRITE(6,'(" The raw block-structured patterns fr",
     +  "om GENPAT available here are:")')
      WRITE(6,'(6X,"",
     +  "Ŀ")')
      WRITE(6,'(6X,"   #   P-name    NY    NF   blocksize",
     +  "   complexities  ")')
      WRITE(6,'(6X,"",
     +  "͵")')
      KKRP = 0
      DO I = 1,NN
        F1 = NAME(I)
        KRP = ICHAR(F1(2:2)) - 48
        IF(KRP>9) KRP = KRP -7   !  alphabetic index + 9
        K = ICHAR(F1(7:7)) - 65  ! <0 if numeral, otherwise alphabet-index less 1
        JG = 0; IF(K>=0) JG = 1+MOD(K,5)
        WORD = ' '; WORD(16:16) = ''
        CALL DEKOD(NAME(I)(3:7),KRP,NV1,NF1,JV,JF,JG,WORD,KC,MB)
        LIST(I) = KC   !             ^   ^  Return of NV and NF
        KKRP = KKRP + KRP-1
        JJG = JG+1
C        WRITE(6,'(6X,A,I4,". ",A,1X,A7,1X,A,1X,I3,2X,A,1X,I3,2X,A,2X,
C     +    I3," x",I2,2X,A,1X,A)') '',I,'', NAME(I)(:7), '',
C     +    NV1, '', NF1, '', JV, JF, '', WORD(:20)

        WRITE(6,'(6X,"",I4,".  ",A7,"  ",I3,"   ",I3,"    ",I3,
     +    " x",I2,"   ",A)') I, NAME(I)(:7), NV1,NF1, JV,JF, WORD(:20)

        IF(MOD(I,18)==0) WRITE(6,'(" Hit RETURN to continue")')
        IF(MOD(I,18)==0) WRITE(6,'()')
      END DO
      WRITE(6,'(6X,"",
     +   "")')
      IF(KKRP>0) WRITE(6,'(6X,""" 1*k "" signifies k recurrences ",
     +  "of complexity-1 rows in master block.")')
      IF(JJG>1) WRITE(6,'(6X,"""  Gk "" signifies k General factor",
     +  "s in master block.  Each declared"/14X,"complexity J is ",
     +  "actually within-block complexity J+k.")')
      MPAT = 1
38    F1 = NAME(MPAT)
      WRITE(6,'(/" The raw pattern now chosen is No. ",A," (",A,").",
     +  "  Hit RETURN if OK;")') CF(:JJF(MPAT)), F1(:LAST(F1))
      IF(NN>1) WRITE(6,'(" otherwise enter the index of another sel",
     +  "ection from this list, or"/" any letter to look for a free",
     +  "-form pattern schema.")')
      IF(NN==1) WRITE(6,'(" otherwise enter anything to look for fre",
     +  "e-form pattern schema.")')
      CALL SCANB(J,1,'I',5,WORD,1)
      IF(J==0) GOTO 40; IF(NN==1.OR.J<0) GOTO 20
      READ(2,*) MPAT; MPAT = MAX(1,MIN(MPAT,NN)); GOTO 38
C
40     OPEN(4,FILE=F1)
      KC = LIST(MPAT)
      CALL SCANB(J,0,'I',4,WORD,-20)
      READ(2,*) NV, KV, NF, KF; JV = NV/KV; JF = NF/KF
      IF(F1(:7)/=WORD(13:19)) WRITE(6,'(/" WARNING: This pattern ",
     +  "file was originally named ",A,"."/" Its specifications ",
     +  "may not be correctly reported above.")') WORD(13:19)
      F1(:7) = WORD(13:19)
cc      NF1 = NF+1; NFF = LO(NF,NF); NVV = LO(NV,NV)  ! Done later
cc      NVF = NV+NF ; NVFF = LO(NVF,NVF)              ! Done later
      IF(ALLOCATED ( T )) DEALLOCATE ( T, PP )
      ALLOCATE ( T(NV,NF), PP(NV,NF) ); KND = 2   ! GENPAT pattern
      CALL GETL(4,' Hi',4,1,0,IER1)
      IF(IER>1) THEN
        WRITE(6,'(" >>> Can''t read this pattern file''s frame para",
     +    "meters.")'); STOP
      END IF
      DO I = 1,4
        CALL SCANB(J,0,'R',4,WORD,12)  ! J counts terms in row
        READ(2,*) (T(I,K),K=1,J)       ! Use T for B(4,MB)
      END DO
C       T loads frame parameters displayed by SHOWP call.
      CUT = 1.; DO K = 1,J; CUT = MIN(CUT,T(2,K)); END DO
      IF(J/=NF/JF .OR. WORD(6:9)/='nonS') THEN  ! ^CUT is least salients l.b.
        WRITE(6,'(/" Program is not reading the head of input file ",
     +    A," correctly.")') F1(:LAST(F1)); STOP
      END IF
      CALL SYSTEM('cls'); WRITE(6,'()')
      CLOSE(15); OPEN(15,FILE='WXYZ')
      PL1 = 's,'; PL2 = 's)'
      DO K = 6,15,9  ! WRITE(15) here
        WRITE(K,'(/1X,A," by ",A," raw pattern ",A," (",A," item block",
     +    A,1X,A," factor block",A," has complexity"/" levels ",A,
     +    " and frame parameters")') CF(:JJF(NV)), CF(:JJF(NF)),
     +    F1(:LAST(F1)), CF(:JJF(KV) ),PL1(3-MIN(2,KV):), CF(:JJF(KF)),
     +    PL2(3-MIN(2,KF):), CF(:JJF(KC))
        CALL SHOWP(NF,JF,T,NV,K)
      END DO
      CALL GETL(4,' Item',6,1,0,IER)
      QL = .TRUE.; IF(IER>0) QL = .FALSE.  ! Probably don't need, but set provisionally
      IF(.NOT.QL) WRITE(6,'(6X,"Subsequent permutation of factor ",
     +  "blocks: None")')   ! No Perm table, no permutations
      IF(.NOT.QL) WRITE(15,'(6X,"Subsequent permutation of factor ",
     +  "blocks: None")')
      IF(QL) THEN
        ALLOCATE ( PERM(KV,KF) )
        DO I = 1,KV
          CALL SCANB(J,0,'R',4,WORD,12); IF(J/=KF+1) IER = I
          IF(IER>0) THEN
            WRITE(6,'(" >>> Failure to read line",I2," of the raw",
     +        " pattern permutation table.")') I; STOP
          END IF
          READ(2,*) M, (PERM(I,M),M=1,KF)
        END DO
        WORD(:4) = '   '; IF(KF>9) WORD(:4) = ' '
        DO K = 6,15,9
          WRITE(K,'(4X," Subpattern blocks were shifted thus:     ")')
          WRITE(K,'(4X,"",14(""),"",27(""),"")')
          DO I = 1,KV
            WRITE(K,'(4X," Item block",A2," ",30A3)')
     +        CLN(1.*I,2,10), (CLN(1.*PERM(I,J),3,10),J=1,KF),
     +        ('   ',J=1,MAX(1,9-KF)), WORD(:4)
          END DO
          WRITE(K,'(4X,"",14(""),"",27(""),"")')
        END DO
        DEALLOCATE ( PERM )
      END IF
      REWIND 4
      CALL START(4,F1)
      DO I = 1,NV
        READ(4,*) (PP(I,J),J=1,NF)
        DO J = 1,NF; PP(I,J) = PP(I,J)/1000; END DO
      END DO
C        The raw input pattern is now in PP
      IF(ALLOCATED(IDENT)) DEALLOCATE (IDENT); ALLOCATE (IDENT(NV+NF))
      READ(4,*,END=50) CH1; IDENT(1) = '['
      IF(CH1=='N'.OR.CH1=='n') READ(4,*,ERR=50) (IDENT(I),I=1,NV)
50    CLOSE(4) ! GENPAT passes the items' name stems
      WRITE(6,'(/" To see this pattern, enter any letter to get its ",
     +  "1-digit display or any"/" number for 2-digit details.",
     +  "  Otherwise, hit RETURN to move on.")')
      CALL SCANB(J,0,'R',5,WORD,1); KK = 0  ! To pick advice show below
      IF(J==0) GOTO 70
      IF(J>0) THEN; KK = 1
        DO I = 1,NV
          WRITE(6,'(2X,I3,".",2(2X,5A5)/4(6X,2(2X,5A5)))')
     +      I, (CLN(PP(I,J),5,-2),J=1,NF)
          IF(MOD(I,5)==0 .AND. I<NV) WRITE(6,'()')
          IF(((MOD(I,10)==0.AND.NV-I>5).OR.I==NV)) CALL WAIT(1)
        END DO
      ELSE IF(J<0) THEN
        CALL SCHEM(NV,NF,JV,JF,.1,PP,NV)  ! Show 1st decimal
      END IF !  ^ Not SKEME!
      WRITE(15,'(/" The finished raw pattern was")')
      DO I =1,NV
        WRITE(15,'(2X,I3,".",2(2X,5A5)/4(6X,2(2X,5A5)))')
     +    I, (CLN(PP(I,J),5,-2),K=1,J)
        IF(MOD(I,5)==0 .AND. I<NV) WRITE(15,'()')
      END DO
      WORD(:13) = 'pick another.'; IF(NN==1) WORD(:13) = 'quit.        '
      WRITE(6,'(/" If satisfied that ",A," is the pattern you want, hi",
     +  "t RETURN. Otherwise,"/" enter anything to ",A)') F1(:LAST(F1)),
     +  WORD(:13)
      CALL SCN(J)
      IF(J/=0) THEN
         IF(NN==1) STOP
         DEALLOCATE ( T, PP, LIST ); GOTO 35
      END IF
C ********** End of GENPAT pattern upload *****************
C               Create namestems here
70    IF(ALLOCATED(IDENT)) DEALLOCATE ( IDENT )
      ALLOCATE ( IDENT(NV+NF) ); IDENT = ' ' ! Initializing seems needed (Why??)
      DO I = 1,NV; IDENT(I)(:1) = '@'; K = 0
        DO J = 1,MIN(NF,26)
          IF(ABS(PP(I,J))>CUT) THEN
            K=K+1; IDENT(I)(K:K) = CHAR(64+J)
          END IF
        END DO
      END DO
      KK = 0

C  Re-enter the free-form pattern track.  ! KK=0 set just after setting IDENT stems
101   IF(KK==0) WRITE(6,'("  If you trust the raw pattern just instal",
     +  "led to be satisfactory, hit RETURN"/"  to upload or create ",
     +  "factor correlations.  Otherwise, enter any letter to"/"  see",
     +  " the instantiated pattern rounded to one decimal with points",
     +  " omitted,"/"  or any number to start again.")')
      IF(KK>0) WRITE(6,'("  If you are satisfied with the raw pattern",
     +  " displayed above, hit RETURN"/"  to upload or create factor",
     +  " correlations.  Otherwise, enter anything to"/"  start agai",
     +  "n.")')
      CALL SCN(J)
      IF(J==0) GOTO 105; IF(KK>0.OR.J>0) GOTO 15  ! 15 is start again
      CALL SCHEM(NV,NF,5,5,.1,PP,NV)  ! Show 1st decimal
      K = 1; GOTO 101

C W-matrix is set; now get Cff
105   IF(ALLOCATED(LIST)) DEALLOCATE ( LIST )
      NVV = LO(NV,NV); NFF = LO(NF,NF); NVF = NV+NF; NVFF = LO(NVF,NVF)
      ALLOCATE ( CFF(NFF), CM(NV),CV(NVV), LIST(NFF),LST(NV), P(NV,NF),
     +  P1(NV,MAX(NV,2*NF)), R(NV), TWIST(NVF,7), U(NVF), CWRD(NVF*6) )  ! , SV(NV)
110   CALL SYSTEM('cls'); LB = 0  ! LB>0 iff KOP becomes 3
C   Ŀ
C    You can enter factor correlations Cff in one of three ways:          10
C      1) Randomize Cff here under loose obliquity controls.              14
C      2) Retrieve a COV-schema previously created by program SCHEMAS.    
C      3) Upload the Cff with basename KOVxx derived from a structural    
C           model F = AF + G or EFA model F = BG + U of F's exo-roots   
C           by previous application of Gendata resources.                 17
C   
      WRITE(6,'(4X,"",70(""),"")')
      WRITE(6,'(4X," You can enter factor correlations Cff in ",
     +  "one of three ways:",10X,"")')
      WRITE(6,'(4X,"   1) Randomize Cff here under loose obliquity",
     +  " controls.",14X,"")')
      WRITE(6,'(4X,"   2) Retrieve a COV-schema previously created ",
     +  "by program SCHEMAS.",4X,"")'); K = JJF(NF); WORD = ' '
      WRITE(6,'(4X,"   3) Upload the Cff with basename KOV",A," de",
     +  "rived from a structural",A,""/4X,"",8X,"model F = AF + G",
     +  " or EFA model F = BG + U of F''s exo-roots",3X,""/4X,"",
     +  8X,"by previous application of Gendata resources.",17X,"")')
     +  CF(:JJF(NF)), WORD(:6-K)
      WRITE(6,'(4X,"",70(""),""/)')
      WRITE(6,'(4X,"If you aren''t sure of what you want in Cff and ",
     +  "are willing to let the"/4X,"program arrogate most of that, h",
     +   "it RETURN to accept Option 1.  Otherwise,"/4X,"enter anythi",
     +   "ng to upload a ",A,"-factor Cff created previously.")')
     +   CF(:JJF(NF))
      CALL SCN(J); IF(J==0) GOTO 130  ! If J=0 goto Option (1) old code
      F3 = 'COV'//CF(:JJF(NF))//'*.*'; LF3 = LAST(F3)
      N2 = -1; N3 = -1  ! Suppress screen output from LOOK
      CALL LOOK(1,F3(:LF3),NAME,40,N2)        ! Option (2) calls COV<nf>.*
      CALL LOOK(1,'K'//F3(2:LF3),NAME,40,N3)  ! Option (3) calls KOV<nf>.*
      IF(N3==0) THEN
        WRITE(6,'(9X,"No previously prepared Cff for ",A," factors is",
     +    " available here.  Enter"/9X,"anything to go with Option 1 ",
     +    "or hit RETURN to quit.")') CF(:JJF(NF))
        CALL SCN(J); IF(J==0) STOP; GOTO 130
      END IF
      C1 = '2'; IF(N2==0) C1 = '3'; J = JJF(NF)
      IF((N3-N2)*N2==0 ) WRITE(6,'(" Your ",A,"-factor Cff candidates",
     +  " (Option ",A," files only) are")') CF(:JJF(NF)), C1
      IF((N3-N2)*N2>0 ) WRITE(6,'(" Your ",A,"-factor COV (Opti",
     +   "on2) and KOV (Option 3) candidates for Cff are")')
     +   CF(:JJF(NF)), CF(:J), CF(:J)
115   CALL LOOK(0,F3(:LF3),NAME,40,N2)
      CALL LOOK(3,'K'//F3(2:LF3),NAME,40,N3); NN = 1   ! N3
117   F3 = NAME(NN)
      IF(N3==1) WRITE(6,'(/6X,"Hit RETURN to upload ",A," for your ",
     +  "Cff.  Otherwise enter"/6X,"any letter to go with Option 1,",
     +  " or any numeral to quit.")') F3(:LAST(F3))
      IF(N3>1) WRITE(6,'(/6X,"Hit RETURN to upload ",A," for your Cf",
     +  "f.  Otherwise, enter the"/6X,"index of another listed file",
     +  " or any letter to accept Option 1."/)') F3(:LAST(F3))
      CALL SCANB(J,0,'I',5,C1,0)
      IF(J==-1) GOTO 130; IF(J<-1) GOTO 117
      IF(J==0) GOTO 120; IF(J>0.AND.N3==1) STOP
      READ(2,*) K; NN = MIN(ABS(K),N3); GOTO 117
120   IF(F3(:3)=='COV') THEN; MSG=1 ! Return instantiated COV<nf> in CFF
        CALL SKEME(NF,NF,F3,N2,P,NV,FUZ,CUT,MSG)   ! SKEME starts by uploading F3 and
        IF(MSG/=0.AND.N3>1) THEN        ! returns instantiated Cff in square P
          WRITE(6,'(4X,"Enter anything to redisplay the schema listing",
     +      " for another pick."/4X,"Otherwise, hit RETURN to quit.")')
          CALL SCN(J); IF(J==0) STOP; GOTO 110
        ELSE IF(MSG/=0) THEN
          WRITE(6,'(/6X,"Since Options 2 and 3 aren''t productive on ",
     +      "this run, hit RETURN"/6X,"to create Cff by Option 1.  Ot",
     +      "herwise, enter anything to quit.")')
          CALL SCN(J); IF(J==0) GOTO 130; STOP
        ELSE   ! Upload from schema is OK, so copy from square storage to symm
          DO J = 1,NF; DO I = 1,J
            CFF(LO(I,J)) = P(I,J)
          END DO; END DO
        END IF
        KOP = 2; ZLO = -1.; ZLB = -1.   ! KOP flags source of Cff
        GOTO 132   ! Goto TWEAK
      ELSE IF(F3(:3)=='KOV') THEN
        CLOSE(7); CALL START(7,F3)
        READ(7,*) M, MM   ! Reinstate
        IF(M/=NF) THEN
          WRITE(6,'(" File ",A," gives covariances for ",A," factors;",
     +      " you want NF = ",A,".")') F3(:LAST(F3)), CF(:JJF(M)),
     +      CF(:JJF(NF)); GOTO 115
        END IF
        IF(M/=NF .OR. MM/=NFF) GOTO 125
        READ(7,*,ERR=125) (CFF(I),I=1,NFF)
C Retrieve A-matrix here.   ! ########  Is this used ?????
        CALL GETL(7,'',1,1,1,IER) ! Put F7 at 1st line following '' start
        LL = (NF+1)/25 ! Number of filelines for each table line
        DO I = 1,NF
          CALL SCANB(K,0,'R',7,C1,0); JC = K-1 ! JC is count of A-terms read
          READ(2,*,ERR=124) Z, (P(I,J),J=1,JC)  ! P gets the A-matrix
          IF(NINT(Z)/=I) GOTO 124; IF(JC==NF) CYCLE
122       CALL SCANB(K,0,'R',7,C1,0)   ! Read lines after the 1st
          READ(2,*,ERR=124) (P(I,JC+J),J=1,K)
          JC = JC+K; IF(JC<NF) GOTO 122
        END DO
C         Maximum(LA) = 3+6*26 = 159
cc   Code in SEMCOV:
cc      IF(KOP==3) THEN ! SEMCOV code writes the full path table into WRD(:LA)
cc        WRITE(8,'(/" The causal-path structure (A-matrix) in this ",
cc     +    "Cff''s production is:")')
cc        DO K = 1,NF+3; WRITE(8,'(A)') WRD(K)(:LA); END DO
cc      END IF
        LA = 6+6*MIN(NF,25)+3; LBW = 3+(NF+1)*(1+(NF-1)/25) ! Char width/height of APATH display
        ALLOCATE( WRD(LBW) ); KOP = 3   ! KOP flags source of Cff
        CALL GETL(7,'',1,1,0,IER) ! Position F7 at 1st line starting table
        DO K = 1,LBW; READ(7,'(A)') WRD(K); END DO
        S = CFF(1); CFF = CFF/S; CLOSE(7)
        GOTO 132   ! If upload is successful
      END IF
124   WRITE(6,'(" ERROR: Cannot find the path matrix in File ",A,".  ",
     +  "GENSCOR wants this"/8X,"to include in the archive of this da",
     +  "taset''s production parameters.")') F3(:LAST(F3)); GOTO 115
125   WRITE(6,'(" ERROR: File ",A," is corrupt.")') F3(:LAST(F3))
      GOTO 115
C  Option 1
130   CALL EZCOV(NF,NFF,CFF,NC,BL,BH,IER)  ! Covs returned in CFF which must be sym-storage
      KOP = 1; ZLO = -1.; ZLB = -1.   ! KOP flags source of Cff
132   WORD = 'tweak-check it first.'; IF(KOP==3) WORD = 'waive that.'
      WRITE(6,'(" Hit RETURN to inspect this Cff, or enter anything ",
     +  "to ",A,)') WORD(:LAST(WORD))
      CALL SCN(J); ZZM = -.0001; IF(J/=0) GOTO 136
134   IF(KOP==1) WRITE(6,'(/" The internally created Cff is now")')
      IF(KOP==2) WRITE(6,'(/" The Cff created by specification of ",
     +  "schema ",A," is now")') F3(:LAST(F3))
      IF(KOP==3) WRITE(6,'(/" The SE-modeled Cff uploaded from ",A,
     +  " is")') F3(:LAST(F3))
      DO J = 1,NF
        IF(NF<14) WRITE(6,'(I3,".",3(2X,5A5))') J, (CLN(CFF(LO(I,J)),
     +    5,2),I=1,J)
        IF(NF>13) WRITE(6,'(I3,".",3(2X,5A4),5(:/8X,3(2X,5A4)))') J,
     +    (CLN(ANINT(100*CFF(LO(I,J))),4,10),I=1,J)
        IF(MOD(J,5)==0 .AND. MOD(J,15)/=0) WRITE(6,'()')
        IF(NF>20 .AND. MOD(J,15)==0) CALL WAIT(0)
      END DO
136   IF(KOP<3) THEN
        CALL TWEAK(NF,NFF,CFF,R,ZLO,ZLB,ZZM,'Cff',IER,1); WRITE(6,'()')
C         If ZLO < 0, Tweak aborts without change.  Otherwise, IER/=0 flags
C         that a repeat call fails to make a change.
        IF(IER/=0 .OR. ZLO<0.) THEN
          IF(IER/=0) WRITE(6,'(17X,"Tweak ineffectual this time.")')
          IF(IER==0.AND.ZLO<0.) WRITE(6,'(24X,"Tweak rescinded")')
          GOTO 138
        END IF
        WRITE(6,'(" Enter anything to see the tweaked Cff, or hit ",
     +    "RETURN to move on.")')
        CALL SCN(J); IF(J/=0) GOTO 134
      END IF
138   WRITE(6,'(/5X,"If satisfied with this choice of Cff, hit RETUR",
     +  "N.  Otherwise,"/5X,"enter anything to retrieve your list of",
     +  " Cff alternatives.")')
      CALL SCN(J); IF(J/=0) GOTO 110
      IF(KND==1.AND.LDOT==9) THEN; F1 = 'PA'//F1(4:); LDOT = 8; END IF
      IF(KND==2) LDOT = 8   ! GENPAT raw pattern; overwrite index at name's end
      K = 0
140   K = K+1
      F2 = F1(:LDOT-1)//CHAR(64+K)//'.POP'    !  PATxxxxB
      INQUIRE(FILE=F2,EXIST=QT)
      IF(QT .AND. K<26) GOTO 140
C Store text describing the Cff used
      CLOSE(16); OPEN(16,FILE='ZYXW')
      IF(KOP==3) WRITE(16,'(/" The factor covariances, which have a m",
     +  "odeled structure detailed in ",A,", are")') F3(:LAST(F3))
      IF(KOP==2) WRITE(16,'(/" The factor covariances, which instantia",
     +  "te this ",A," schema, are")') F3(:LAST(F3))
      IF(KOP==1) WRITE(16,'(/" The factor covariances, created inter",
     +  "nally to have ",A," randomly located obliquities, are")')
     +  CF(:JJF(NC))
      WRITE(16,'("  Cff:")')
      DO J = 1,NF
        IF(J<10) WRITE(16,'(3X,"f",I1,":",5(2X,5A5),9(:/8X,5(2X,5A5)))')
     +     J, (CLN(CFF(LO(I,J)),5,2),I=1,J)
        IF(J>9) WRITE(16,'(2X,"f",I2,":",5(2X,5A5),9(:/8X,5(2X,5A5)))')
     +     J, (CLN(CFF(LO(I,J)),5,2),I=1,J)
        IF(MOD(I,5)==0 .AND. I<NF) WRITE(16,'()')
      END DO
      IF(KOP<3.AND.ABS(ZZM-1.)>.01) THEN
        WRITE(16,'(/"  This Cff''s smallest eigenvalue, now ",A4,", ",
     +    "was tweaked by off-diagonal "/"  multiplier ",A4," from ",
     +    A4," in the schema''s raw instantiation.",10X,"")')
     +    CLN(R(NF),4,3), CLN(ZZM,4,3), CLN(ZLB,4,3)
      END IF

C  Initialize unadjusted random communalities over full unit-interval
      X = 1.; Y = 0.
      DO I = 1,NV
        Z = RANDY(); CM(I) = Z
        X = MIN(X,Z); Y = MAX(Y,Z)
      END DO
      D = Y-X
      DO I = 1,NV; CM(I) = (CM(I)-X)/D; END DO
      CALL SYSTEM("cls"); WRITE(6,'()')

c    ͻ
c     It still remains to set each data variable's communality, which 
c     you recall is its proportion of variance accounted for by the   
c     dataset's common factors.  (In the present model, communalities 
c     equal the diagonal elements of Cyy - De.)  Their default values 
C     randomized within your pick of limits <Lo,Hi> can be overridden 
c     by your stipulations for selected variables.                    
C    ͼ

      WRITE(6,'(4X,"",65(""),"")')
      WRITE(6,'(4X," It still remains to set each data variable''s",
     +  " communality, which "/4X," you recall is its proportion",
     +  " of variance accounted for by the   "/4X," dataset''s c",
     +  "ommon factors.  (In the present model, communalities "/4X,
     +  " equal the diagonal elements of Cyy - De.)  Their default ",
     +  "values "/4X," randomized within your pick of limits <Lo,H",
     +  "i> can be overridden "/4X," by your stipulations for sele",
     +  "cted variables.",20X,"")')
      WRITE(6,'(4X,"",65(""),"")')

C  Your range for random default communalities is now set at <.xx, ,xx>.
C  Hit RETURN if OK, or enter two numbers in the unit interval to revise this.

150   WRITE(6,'(/"  Your range for random default communalities is ",
     +  "now set at <",A3,", ",A3,">."/"  Hit RETURN if OK, or enter",
     +  " two numbers in the unit interval to revise this."/)')
     +  CLN(B0,3,2), CLN(B1,3,2)
      CALL SCANB(J,0,'R',5,C1,0)
      IF(J==0) GOTO 154; IF(J<0) GOTO 150
      READ(2,*) (U(I),I=1,MIN(J,2)); IF(J==1) U(2) = U(1)
      B1 = MAX(ABS(U(1)),ABS(U(2))); B0 = MIN(ABS(U(1)),ABS(U(2)))
152   IF(B1>1.) THEN
        IF(B1>1.) B1 = B1/10; IF(B0>1.) B0 = B0/10
        GOTO 152
      END IF
      GOTO 150
154   D = B1-B0; DO I = 1,NV; CM(I) = D*CM(I) + B0; END DO  ! <= Final random comms
      WRITE(6,'(" If you prefer to accept the default communalities ",
     +  "without inspection, hit"/" RETURN.  Otherwise, enter anythi",
     +  "ng to initiate finer communality control.")')
      CALL SCN(J); IF(J==0) GOTO 201; KNT = 0
      IF(NV<41) WRITE(6,'(" Your randomized communalities have been ",
     +  "defaulted to:",20(:/2(2X,4(I3,"(",A4,")"))))')
     +  (I,CLN(CM(I),4,3),I=1,NV)
      IF(NV>40) WRITE(6,'(" Your randomized communalities have been de",
     +  "faulted to:",20(:/10(I3,"(",A3,")")))')
     +  (I,CLN(CM(I),3,2),I=1,NV)
      WRITE(6,'(/" If you are willing to accept all these, enter anyth",
     +  "ing to do so.  Otherwise,"/" make an off-screen list (item i",
     +  "ndex and preferred value) of the communalities"/" you want t",
     +  "o change, and then hit RETURN to see further instructions.")')
      CALL SCN(J); IF(J/=0) GOTO 201

C  Ŀ
C   To assign one or more variables the same communality CM in the unit    
C   interval, enter a line starting with a numeric CM (don't ignore its    
C   decimal), followed either by two integers i,j if you want all items    
C   in index interval [i,j] to have this communality, or by some other     
C   count of indices to assign CM to just the variables explicitly listed. 
c   (To specify CM for just two nonadjacent indices, enter one twice to    
C   change the count.)  You will have opportunity to declare more, as      
C   well as to review and make corrections.                                
C  
      IF(NV<41) WRITE(6,'(" Your randomized communalities have been ",
     +  "defaulted to:",20(:/2(2X,4(I3,"(",A4,")"))))')
     +  (I,CLN(CM(I),4,3),I=1,NV)
      IF(NV>40) WRITE(6,'(" Your randomized communalities have been de",
     +  "faulted to:",20(:/10(I3,"(",A3,")")))')
     +  (I,CLN(CM(I),3,2),I=1,NV)
      WRITE(6,'("  ",72(""),"")')
      WRITE(6,'("   To assign one or more variables the same communa",
     +  "lity CM in the unit    "/"   interval, enter a line starti",
     +  "ng with a numeric CM (don''t ignore its    "/"   decimal),",
     +  " followed either by two integers i,j if you want all items",
     +  4X,""/"   in index interval [i,j] to have this communality,",
     +  " or by some other     "/"   count of indices to assign CM ",
     +  "to just the variables explicitly listed. "/"   (To specify",
     +  " CM for just two nonadjacent indices, enter one twice to    ",
     +  ""/"   change the count.)  You will have opportunity to dec",
     +  "lare more, as",6X,""/"   well as to review and make correc",
     +  "tions.",32X,"")')
      WRITE(6,'("  ",72(""),"")')
160   WORD = ' '; IF(KNT==0) WORD(:7) = ' empty.'
      WRITE(6,'(" Your list of communality revisions is now",A)')
     +  WORD(:7)
      IF(KNT>0) WRITE(6,'(11(:/1X,9(1X,A,"(",A3,")")))') (CF(:JJF(
     +  LIST(I)/1000)),CLN(.001*MOD(LIST(I),1000),3,2),I=1,KNT)
      WORD = 'another line of item'; L = 20
      IF(KNT==0) WORD = 'a line of item      '
      WRITE(6,'(/" If accurate and complete, hit RETURN.  Otherwise, ",
     +  "enter ",A,/" indices preceded by the communality assigned to",
     +  " them.  Including an index"/" entered previously will replac",
     +  "e its earlier specification with the latest"/" one.  To dele",
     +  "te items from this respecification list, enter a negative"/
     +  " number followed by indices to be dropped (or just two if yo",
     +  "u want to wipe"/" all respecifications in that index interva",
     +  "l)."/)') WORD(:20)
      IF(KNT==0) THEN  ! Maybe inferior
        DO I = 1,3-NV/10; WRITE(6,'()'); END DO ! ****** Push frame to top of screen
      END IF
      CALL SCANB(J,0,'R',5,C1,0)
      IF(J==0) GOTO 175  ! Write LIST entries into CM
      IF(J<2) WRITE(6,'(6X,"Huh ???  Enter a real number followed by",
     +  " at least one integer,"/15X,"or hit RETURN to move on."/)')
      IF(J<2) GOTO 160; LL = 0      ! Jth entry in LIST will be J*1000 plus 1st
      READ(2,*) Z, (R(I),I=1,J-1)  ! 3 decimals of its comm with point omitted.
      DO I = 1,J-1; IF(R(I)/=ANINT(R(I))) CYCLE
        N =  MIN(NINT(R(I)),NV); IF(N<1) CYCLE
        LL = LL+1; LST(LL) = N
      END DO
      IF(LL==2 .AND. LST(1)==LST(2)) LL = 1  ! Degenerate two-entry case
      IF(LL==2) THEN   ! Interval reading of just two entries
        K = MIN(LST(1),LST(2)); L = MAX(LST(1),LST(2))
        DO J = K,L; LST(J-K+1) = J; END DO
        LL  = L-K+1
      END IF
      IF(Z<0.) THEN  ! Deletions
        DO K = 1,KNT; DO J = 1,LL
          IF(LST(J)==LIST(K)/1000) LIST(K) = 999999
        END DO; END DO
        CALL ISORT(KNT,LIST)    ! ; LIST(KNT+1)=999999  <=  why this trailer???
165     IF(LIST(KNT)==999999) THEN; KNT = KNT-1; GOTO 165; END IF
        GOTO 160
      END IF  ! Deletions now expunged from LIST
170   IF(Z>1.) THEN; Z = Z/10.; GOTO 170; END IF
      MM = MIN(999,NINT(Z*1000)) ! MM is the mod part of new entries
      OUTR:DO J = 1,LL  ! First check if index NN is already listed
        NN = LST(J)    !; IF(NN<1.OR.NN>NV) CYCLE ! Bounds already enforced
        DO I = 1,KNT
          IF(LIST(I)/1000 == NN) THEN
C            LIST(I) = 1000*(LIST(I)/1000) + MM;  CYCLE OUTR
            LIST(I) = LIST(I) - MOD(LIST(I),1000) + MM; CYCLE OUTR
          END IF ! Cycling on outer loop insures no index duplications in LIST
        END DO
        KNT = KNT+1; LIST(KNT) = 1000*NN + MM
      END DO OUTR
      CALL ISORT(KNT,LIST)
      GOTO 160
175   DO I = 1,KNT    ! Last occurrence of KNT
        Z = MOD(LIST(I),1000)*.001; IF(Z>.998) Z=1.
        CM(LIST(I)/1000) = Z
      END DO
      WRITE(6,'(/4X,"To inspect the provisionally final list of comm",
     +  "unalities,"/4X,"enter anything.  Otherwise, hit RETURN to ",
     +  "waive this.")'); KNT = 0
      CALL SCN(J); IF(J==0) GOTO 201
      WRITE(6,'(" If you are content with these communalities:"/)')
cc      WRITE(6,'(20(8X,2(2X,5A5):/))') (CLN(CM(I),5,3),I=1,NV)
      WRITE(6,'(20(1X,5A5,2(2X,5A5):/))') (CLN(CM(I),5,3),I=1,NV)
      WRITE(6,'(/" hit RETURN.  Otherwise, first make note of the one",
     +  "s in this new list you want"/" to change and then enter anyt",
     +  "hing for return to the respecification process.")')
      CALL SCN(J); IF(J/=0) GOTO 160

201   DO I = 1,NV; SF = 0.
        DO J = 1,NF; DO K = 1,NF
            SF = SF + PP(I,J)*CFF(LOC(J,K))*PP(I,K)
        END DO; END DO
        S = SQRT(CM(I)/SF)
C         CM(I) is the Ith variable's assigned communality. And SF is the Ith
C         diagonal element of the raw true-part covariance matrix PP*CF*PP'.
        DO J = 1,NF
          P(I,J) = S*PP(I,J)  ! PP now free for other use
        END DO
      END DO ! >>>> Diag[PCFFP'] contains the stipulated communalities
      IF(IDENT(1)(:1)=='[') GOTO 210
      CALL NAMS(NV,NF,IDENT,P,NN,MAXL)

c   Each data variable has been assigned a name profiling the factors most
c   salient for it.  With factors f1,f2,..,f7 denoted by letters A,B,..,G,
c   respectively, the name of each variable yi starts with the letter-coded
c   list of factors declared salient for yi in decreasing order of loading
c   size (up to a cutoff of six) followed by the numeric index of this
c   item's salience rank in the subset of items having this same profile.
c   Names of items assigned no salient factors start with "@".

      WRITE(16,'(/3X,"Each data variable has been assigned a name pro",
     +  "filing the factors most"/3X,"salient for it.  With factors ",
     +  "f1,f2,..,f",A," denoted by letters A,B,..,",A/3X,"respective",
     +  "ly, the name of each variable yi starts with the letter-cod",
     +  "ed"/3X,"list of factors declared salient for yi in decreasi",
     +  "ng order of loading"/3X,"size (up to a cutoff of six) follo",
     +  "wed by the numeric index of this"/3X,"item''s salience ra",
     +  "nk in the subset of items having this same profile.")')
     +  CF(:JJF(MIN(26,NF))), CHAR(MIN(90,64+NF))
      IF(NN>0) WRITE(6,'(3X,"Names of items assigned no salient fact",
     +  "ors start with ""@""."/)')

cc      WRITE(16,'(/" Unique variances assigned to the data variables ",
cc     +  "are"/5(2X,5A5),9(:/8X,5(2X,5A5)))') CLN(1.-CM(I),5,2)

C The item names so created, with their unique variances parenthetically prefixed, are
      WRITE(16,'(/" The item names so created, with their unique",
     +  " variances parenthetically prefixed, are")')
      WRITE(16,'(20(5(I4,": (",A3,") ",A),:/))')
     +  (I,CLN(1.-CM(I),3,2),IDENT(I)(:MAXL),I=1,NV)
C ###     P is now the NV-by-NF refined simple-structure factor pattern;
C ###     communalities are in CM; saliences should be coded in item names.
210   WRITE(6,'(/" Your refined simple-structure factor pattern (co",
     +  "mmunalities in parens) is:")')
      LL = (NV+1)/10; LB = 21/LL  ! Break if MOD(I,LB) = 0
      DO I = 1,NV
        IF(MOD(I,5)==1) WRITE(6,'()')
        WRITE(6,'(I4,". (",A3,")",2(2X,5A5),20(:/14X,2(2X,5A5)))')
     +    I, CLN(CM(I),3,2), (CLN(P(I,J),5,2),J=1,NF)
        IF(MOD(I,MIN(LB,10))==0 .OR. I==NV) THEN
          IF(I<NV) WRITE(6,'(/" Hit RETURN to continue, or enter ",
     +      "anything to skip the pattern''s remainder.")')
215       IF(I==NV) WRITE(6,'(/3X,"If you dislike this result, do Ct",
     +      "rl-C to void the run."/3X,"Otherwise, hit RETURN",
     +      " to finish production.")')
          CALL SCN(J); IF(J/=0) GOTO 216
        END IF
      END DO
216   WRITE(16,'(/" Row-wise rescaling the raw pattern reported abo",
     +  "ve to yield unit-norm data variables"/" results in the fol",
     +  "lowing specification of W in production model Y = WF + E."/
     +  " (Communalities entailed by the assigned uniqenesses are ",
     +  "shown in parentheses.)"/"  W:")')
      DO I = 1,NV
        WRITE(16,'(I4,". (",A3,")",4(1X,5A5,1X),20(:/14X,4(1X,5A5,
     +    1X)))') I, CLN(CM(I),3,2), (CLN(P(I,J),5,2),J=1,NF)
        IF(MOD(I,5)==0 .AND. I<NV) WRITE(16,'()')
      END DO

C  Creation of simple-structure pattern ends with communalities in CM,
C  unique-factor SDs in U, Cff in CFF, fInished pattern on SS-factors in P;
C  will solve for T,R eigvecs/vals of Cff so Q = TR is pattern of SS-factors
C  on their prin.factors G and PQ is pattern of common-parts H on orthnorm G

      DO I = 1,NV; U(I) = SQRT(MAX(0.,1.-CM(I))); END DO
C       U now contains the unique-factor SDs
C  Put true-part covars in P1 to solve for its eigenstructure
      DO J = 1,NV             ! ***** Not efficient comp; but probably
        DO I = 1,J; S = 0.    !       not worth fuss to improve
          DO K = 1,NF; DO M = 1,NF
              S = S + P(I,K)*CFF(LOC(K,M))*P(J,M)
          END DO; END DO
          P1(I,J) = S; P1(J,I) = S  ! P1 gets common-part covars Chh
          CV(LO(I,J)) = S
        END DO
      END DO

      WRITE(16,'(/" "/4X," Above is the matrix W of fac",
     +  "tor coefficients that a SEM or EFA method of"/4X," common",
     +  "-source recovery should hope to retrieve from this simulat",
     +  "ion''s"/4X," normalized data covariances Cyy, below:"/
     +  " "//"  Cyy:")')
      DO J = 1,NV
        CV(LO(J,J)) = 1.
        WORD(:5) = '  y'//CF(:JJF(J))
        IF(J<10) WORD(:5) = '   y'//CF(:JJF(J))
        IF(J>99) WORD(:5) = ' y'//CF(:JJF(J))
        WRITE(16,'(A5,":",4(5A5,2X),20(:/9X,4(5A5,2X)))') WORD(:5),
     +    (CLN(CV(LO(I,J)),5,2),I=1,J)
        IF(J<NV.AND.MOD(J,5)==0) WRITE(16,'()')
      END DO  ! Only use of CV until recomputed by SCORS
      IF(ORT<1) WRITE(16,'(/"  NOTE: This is your simulation''s popul",
     +  "ation Cyy.  It differs slightly from  "/" ",7X,"the data c",
     +  "ovariances in your finite score array since you elected to "/
     +  " ",7X,"leave sampling noise in its distribution of unique ",
     +  "components.",7X,"")')
      CALL EIGS(NV,NF,P1,NV,R,T,NV,1,IER,6)
C        The nonzero common-part roots are now in R, while T contains the
C        corresponding eigenvectors. So H = (TL)G
      WRITE(16,'(/" The ",A," salient eigenvalues of Cyy''s common-pa",
     +  "rts covariances (that is, of Cyy - Du) are"/50(4(2X,5(1X,A4))
     +  :/))') CF(:JJF(NF)),(CLN(R(I),4,3),I=1,NF) ! Note: All lower eivals pass zeros test
      IF(R(NF)<.1) WRITE(16,'(/" To the extent the smallest of these ",
     +  "approaches zero, solution for the model''s "/" parameters sh",
     +  "ould approach indeterminacy.  Developing rules of thumb for ",
     +  "how"/" small these can become without crippling model soluti",
     +  "on would be a desirable"/" target of simulation research.")')
220   WRITE(6,'(5X,"It still remains to create a population of joint ",
     +  "data/factor scores that"/5X,"embodies this model''s stipulat",
     +  "ed structure.  If you don''t want scores,"/5X,"enter anythin",
     +  "g to signal that.  Otherwise, hit RETURN to produce them.")')
      CALL SCN(J)
      IF(J/=0) THEN
        WRITE(6,'(9X,">>> Hit RETURN to confirm that you don''t want ",
     +    "scores."/13X,"Otherwise, enter anything to reconsider.")')
        CALL SCN(J); IF(J/=0) GOTO 220
        QSCOR = .FALSE.
        WRITE(16,'(/" >>>> Production of scores satisfying this simu",
     +    "lation structure has been waived. <<<<")')
        GOTO 401
      END IF

C Find pattern matrices that will carry the principal factors G into
C  (a) the true-parts and (b) the simple-structure factors:
CC [OLD:  This took the H-parts' principal factors for orthonormal basis G
CC Situation: Let H be true parts, F the SS factors, G the principal factors:
CC   H = PF = RDG where R,D is eigenstructure of H-covars with R NV-by-NF.
CC   So RDG = PF whence G = WF where W = (Inv[D]R'P), and F = Inv[W]G.

C [NEW]: Use the principal factors of F for orthonormal basis G of H-space
C Alternatively, H = PF, F = TDG where TDT' is eigstructure of Cff;
C so H = (PTd)G and F = TdG where Td =def TD.  P so-named is already computed

      CALL EIGS(NF,NF,CFF,NFF,R,T,NV,1,IER,6) ! Get eigvals/vecs of Cff
      IF(R(NF)<0) THEN
        WRITE(6,'(" Your Cff is non-Gramian; real factor scores so c",
     +    "ovaried cannot be created.")'); QSCOR = .FALSE.; GOTO 401
      END IF
      DO J = 1,NF; S = SQRT(R(J))
        DO I = 1,NF
          T(I,J) = T(I,J)*S
        END DO
      END DO  ! T now holds the F-loadings on their principal factors G
      P1 = 0. ! Put H-pattern PT on G into P1
      DO I = 1,NV; DO J = 1,NF
          DO K = 1,NF
            P1(I,J)  = P1(I,J)  + P(I,K)*T(K,J)
          END DO
      END DO; END DO

C  Head size: 4{ [1]: 4+NV+8. [2]: 6NV. [3]: NVNF. [4]: NVV + NFF + NVNF }
c           = 4{ 7NV + 12 + 2NVNF + (NV + NV + NF + NF)/2 }
C             = 28NV + 48 + 8NVNF + 2NV + 2NV + 2NF + 2NF
c             = 2[NV + NF + 4NVNF + 15NV + NF + 24]
c             = 2[NV + NF + 4NF(NV+1) + 15NV + 24]
C Number of bytes in datafile head:  2[NV + NF + 4NVNF + 15NV + NF + 24]
C                                   = 2[(NV+NF) + 2NFNV + 15NV + NF + 24].
C Number of bytes in datafile score array is: 4*NS*(NV+NF).  Empirically, file
C   overhead appears to increase the 4-multiplier to roughly 4.06.
      LH = 2*((NV+NF)**2 + 2*NV*NF + 15*NV + NF + 24); NS = 1000

      INQUIRE(FILE='LASTNS',EXIST=QT)
      IF(.NOT.QT) GOTO 335
      OPEN(29,FILE='LASTNS'); READ(29,*,END=335,ERR=335) NS; CLOSE(29)
335   X = (LH + 4.06*NS*(NV+NF))/1000000
      WRITE(6,'(/" The number of score records to be generated is no",
     +  "w set at ",A,", needing ",A4/" megabytes of storage space. ",
     +  " Hit RETURN if OK, or enter new population size."/)')
     +  CF(:JJF(NS)), CLN(X,4,2)
      CALL SCANB(J,1,'I',5,C1,0)
      IF(J==0) GOTO 340
      IF(J<0) GOTO 335
      READ(2,*) NS; NS = MAX(10,NS); GOTO 335

C  Generate scores
340   ORT = 1; MTW = 0  ! MTW = 1 if Method TWisted, otherwise Normal
345   IF(MTW==0) WRITE(6,'(/" If you want Normal distributions on a",
     +  "ll the common and unique factors, hit"/" RETURN.  Otherwis",
     +  "e, enter anything to consider setting twist parameters.")')
      IF(MTW==1) WRITE(6,'(/" If you want twisted distributions on ",
     +  "some or all factors, hit RETURN.  (You"/" can initiate twi",
     +  "st to learn more about this but back out before executing ",
     +  "it.)"/" Otherwise, enter anything to get Normal score dis",
     +  "tributions.")')
      CALL SCN(J); IF(J/=0) THEN; MTW = 1-MTW; GOTO 345; END IF
      IF(MTW==0) GOTO 350
      CALL TWST(NF,NV,TWIST,TP,QT); ORT = 0 ! Discourage orthogonalizing twisted dists.
      IF(.NOT.QT) THEN
        MTW = 0; ORT = 1
        WRITE(6,'("  Your brush with twist parameters has left these ",
     +    "set for Normality."/" Hit RETURN to execute that choice, ",
     +    "or enter anything to try again.")')
        CALL SCN(J); IF(J/=0) GOTO 345
      END IF

c The unique-factor orthogonalities created in this dataset can either be
c left approximate or made exact by partialling.  Exact is appropriate if
c you want to view this score array as proxy for an infinite population whose
c zero covariances have exactly zero expectations in random-with-replacement
c samples from that.  But you can also accept it as a size-NS sample from
c NY+NF-dimensional random generator space with unique factors aligned with
c generation axes whose covariances differ from their zero expectations by
c sampling noise that for large NS will be trivial.  It shouldn't much matter
c which way you go on this, but a choice is needed and you may need to say
c something about it if you communicate findings from these simulation data.
C (Since Gendata avails sampling from score populations created here, you
C are advised to waive strict uniqueness orthogonality only if you don't
C want that to dilute the extremities of skew or kurtosis you have twisted
C into the source distributions.)

350   WRITE(6,'(/" The unique-factor orthogonalities created in this ",
     +  "dataset can either be"/" left approximate or made exact by p",
     +  "artialling.  Exact is appropriate if"/" you want to view thi",
     +  "s score array as proxy for an infinite population whose"/" z",
     +  "ero covariances have exactly zero expectations in random-with",
     +  "-replacement"/" samples from this.  But you can also accept ",
     +  "it as a size-NS sample from"/" NF+NY-dimensional random gene",
     +  "rator space with unique factors aligned with"/" generation a",
     +  "xes whose covariances differ from their zero expectations by"/
     +  " sampling noise that for large NS will be trivial.  It shoul",
     +  "dn''t much matter"/" which way you go on this, but a choice",
     +  " is needed and you may need to say"/" something about it if",
     +  " you communicate findings from these simulation data.")')
      WRITE(6,'(" (Since Gendata avails sampling from score populati",
     +  "ons created here, you"/" are advised to waive strict unique",
     +  "ness orthogonality only if you don''t"/" want that to dilut",
     +  "e the extremities of skew or kurtosis you have twisted"/
     +  " into the source distributions.)"/)')
355   IF(ORT==1) WRITE(6,'(4X,"If you want the uniqueness orthogonal",
     +  "ities in this score array to be"/4X,"exact, hit RETURN.  Ot",
     +  "herwise, enter anything to retain their random"/4X,"producti"
     +  "on''s sampling violation of strictly zero Cfe and diagonal ",
     +  "Cee.")')
      IF(ORT<=0) WRITE(6,'(4X,"To retain the sampling noise in this",
     +  " simulation''s random production"/4X,"of orthogonal unique",
     +  " factors, hit RETURN.  Otherwise, enter anything"/4X,"to ",
     +  "make Cfe exactly zero and Cee exactly diagonal.")')
      CALL SCN(J)
      IF(J/=0) THEN; ORT = 1-ORT; GOTO 355; END IF
      IF(ORT>0) GOTO 360   ! ORT options are only 0,+1 here

CCC >>>> Suppress, probably delete, option to standardize Cyy when ORT < 1
CCC    To let sampling noise in the unique factors disturb the strictly standard
CCC    (unit-variance) scaling of this finite population's data variables, hit
CCC    RETURN.  Otherwise, enter any letter to enforce exact data standardizing,
CCC    or any number to reconsider imposing strict unique-factor orthogonality.
CC
CCC    To impose strict unit-variance norming on this population's data variables
CCC    by rescaling the common-factor pattern to compensate for sampling noise in
CCC    the unique-factor orthogonalities, hit RETURN. Otherwise, enter any letter
CCC    to allow chance deviations of the data variances from unit norm, or any
CCC    number to reconsider imposing strict unique-factor orthogonality.
CC
CCC     If ORT=1, U-factors are strictly orthogonal and data strictly unit-normed
CC357  IF(ORT==0) WRITE(6,'(4X,"To let sampling noise in the unique ",
CC     +  "factors disturb the strictly standard"/4X,"(unit-variance) ",
CC     +  "scaling of this finite population''s data variables, hit"/4X,
CC     +  "RETURN.  Otherwise, enter any letter to enforce exact data ",
CC     +  "standardizing,"/4X,"or any number to reconsider imposing st",
CC     +  "rict unique-factor orthogonality."/)')
CC      IF(ORT<0) WRITE(6,'(4X, "To impose strict unit-variance norming",
CC     +  " on this population''s data variables"/4X,"by rescaling the ",
CC     +  "common-factor pattern to compensate for sampling noise in"/4X,
CC     +  "the unique-factor orthogonalities, hit RETURN. Otherwise, en",
CC     +  "ter any letter"/4X,"to allow chance deviations of the data ",
CC     +  "variances from unit norm, or any"/4X,"number to reconsider ",
CC     +  "imposing strict unique-factor orthogonality."/)')
CC      CALL SCANB(J,0,'R',5,WORD,1)
CC      IF(J>0) GOTO 355
CC      IF(J<0) THEN; ORT = -1 - ORT; GOTO 357; END IF

360   WRITE(6,'(/" Computing data/factors scores for ",A," subjects.")')
     +   CF(:JJF(NS))
      OPEN(14,STATUS='SCRATCH',FORM='UNFORMATTED')
C       Initially put data scores followed by factor scores in scratchfile 14.

      DEALLOCATE ( PP ); ALLOCATE ( W(NVV) )
      W = CV  ! T is pattern of F on their prin-facs G; P1 holds pattern of H on G
      DO I = 1,NV; W(LO(I,I)) = 1.; END DO
      CALL SCORS(NS,NF,NV,NVF,P1,T,U,MTW,ORT,TWIST,CV,NVFF) ! CV returns data covars
      W = W - CV  ! Sampling error in data covs due to sampling error in unique factors
      IF(ORT==1) THEN
        ERR = 0.; II = 0; JJ = 0
        DO J = 1,NV; DO I = 1,J
            IF(ABS(W(LO(I,J)))>ERR) THEN;
              ERR = ABS(W(LO(I,J))); II = I; JJ = J
            END IF
        END DO; END DO
        IF(ERR>.01) THEN
          WRITE(6,'(/" Maximum mismatch between stipulated Cyy and ",
     +    "its score realization."/4X,A5," (",A,", ",A,")")')
     +    CLN(ERR,5,4), CF(:JJF(II)), CF(:JJF(JJ)); CALL WAIT(0);
        END IF
      END IF

CC      err= 0
CC      DO I = 1,NV  ! >>> Can reactivate for test of SCORS output when ORT=1
CCC        SV(I) = 1./SQRT(CV(LO(I,I)))  ! Only if ORT = -1 is reinstated
CCC        IF(ABS(SV(I)-1.)>.001) QS = .TRUE.
CC        err = max(err, abs(1.- CV(LO(I,I))))
CC      END DO

C       P is the refined simple-structure pattern; CM the communalities.
C       CV holds data covars

CC      IF(QS .AND. ORT<0) THEN    ! Only if ORT = -1 is reinstated
CC        DO J = 1,NV
CC          CM(J) = CM(J)*SV(J)**2    ! Final communalities
CC          DO I = 1,J
CC            CV(LO(I,J)) = CV(LO(I,J))*SV(I)*SV(J)  ! Final data covs
CC          END DO
CC          DO K = 1,NF
CC            P(J,K) = SV(J)*P(J,K)  ! Final SS patterm
CC          END DO
CC        END DO
CC      END IF

C ======================================================================
CSCORE PRODUCTION (NS = xxxxxxx):
C   In notation below, for any sets X,Z of variables in this model, Cxz is   
C   the matrix of X:Z covariances created by this simulation prior to score  
C   production, whereas Cov(X,Z) comprises the X:Z covariances in the finite 
C   score array actually produced.  If you waived enforced orthogonalization 
C   of unique factors, some Cov(X,Z) differ from Cxz by sampling error.      

      WRITE(16,'(//" SCORE PRODUCTION (NS = ",A,"):")') CF(:JJF(NS))
      WRITE(16,'("   In notation below, for any sets X,Z of variable",
     +  "s in this model, Cxz is   "/"   the matrix of X:Z covaria",
     +  "nces created by this simulation prior to score  "/"   pro",
     +  "duction, whereas Cov(X,Z) comprises the X:Z covariances in t",
     +  "he finite "/"   score array actually produced.  If you wa",
     +  "ived enforced orthogonalization "/"   of unique factors, ",
     +  "some Cov(X,Z) differ from Cxz by sampling error.",6X,"")')
      WRITE(16,'("########")')  ! Replace line with completion of following
CC      WRITE(16,'("  A size-",A," population of joint scores on <Y,F>",
CC     +  " having this stipulated structure has been saved in binary ",
CC     +  "file ",A)') CF(:JJF(NS)), F2  ! Go to "###" below
CC       Rem !!! "##..." here is part of the program, not an editing flag !!!
C      WORD = ' enforcement of strict unique-factor orthogonalities'
C      IF(ORT<1) WORD = 'out enforcing'//WORD(16:52)//', so that'      ! 59
C      IF(MTW==0) WRITE(16,'("  The joint distribution was sampled to",
C     +  " be unit-Normal with",A)') WORD(:60)
C      IF(MTW==1) WRITE(16,'("  The distribution was twisted from joi",
C     +  "nt unit-Normality with",A)') WORD(:60)
C      IF(ORT==1) WRITE(16,'("  to ensure Cov(E,E) = Cee (= De), Cov(",
C     +  "F,E) = Cfe (= 0), Cov(Y,F) = Cyf, and Cov(Y,Y) = Cyy.")')
C      IF(ORT==0) WRITE(16,'("  approximations Cov(E,E)  Cee (= De),",
C     +  " Cov(F,E)  Cfe (= 0), Cov(Y,F)  Cyf, and Cov(Y,Y)  Cyy ",
C     +  "contain sampling noise.")')

      IF(ORT==1.AND.MTW==0) WRITE(16,'("  was sampled to be unit-Norm",
     +  "al with enforcement of strict unique-factor"/"  orthogonalit",
     +  "ies to ensure Cov(E,E) = Cee (= De), Cov(F,E) = Cfe (= 0),"/
     +  "Cov(Y,F) = Cyf, and Cov(Y,Y) = Cyy.")')

      IF(ORT==0.AND.MTW==0) WRITE(16,'("  was sampled to be unit-Nor",
     +  "mal without enforcing strict unique-factor"/"  orthogonalit",
     +  "ies, so that approximations Cov(E,E)  Cee (= De),"/"  Cov(",
     +  "F,E)  Cfe (= 0), Cov(Y,F)  Cyf, and Cov(Y,Y)  Cyy contai",
     +  "n"/"  sampling noise.")')

      IF(ORT==1.AND.MTW==1) WRITE(16,'("  was twisted from joint uni",
     +  "t-Normality with enforcementof strict unique-factor"/"  ort",
     +  "hogonalities to insure Cov(E,E) = Cee (= De), Cov(F,E) = Cfe",
     +  " (= 0),"/"  Cov(Y,F) = Cyf, and Cov(Y,Y) = Cyy.")')

      IF(ORT==0.AND.MTW==0) WRITE(16,'("  was twisted from joint unit",
     +  "-Normality without enforcing strict unique-factor"/"  ortho",
     +  "gonalities, so that approximations Cov(E,E)  Cee (= De),"/
     +  " Cov(F,E)  Cfe (= 0), Cov(Y,F)  Cyf, andCov(Y,Y)  Cyy ",
     +  "contain"/"  sampling noise.")')

cc   was sampled to be unit-Normal with enforcement of strict unique-factor
cc   orthogonalities to ensure Cov(E,E) = Cee (= De), Cov(F,E) = Cfe (= 0),
cc   Cov(Y,F) = Cyf, and Cov(Y,Y) = Cyy.

cc   was sampled to be unit-Normal without enforcing strict unique-factor
cc   orthogonalities, so that approximations Cov(E,E)  Cee (= De),
cc   Cov(F,E)  Cfe (= 0), Cov(Y,F)  Cyf, and Cov(Y,Y)  Cyy contain
cc   sampling noise.

cc   was twisted from unit-Normality with enforcement of strict unique-factor
cc   orthogonalities to insure Cov(E,E) = Cee (= De), Cov(F,E) = Cfe (= 0),
cc   Cov(Y,F) = Cyf, and Cov(Y,Y) = Cyy.

cc   was twisted from unit-Normality without enforcing strict unique-factor
cc   orthogonalities, so that approximations Cov(E,E)  Cee (= De),
cc   Cov(F,E)  Cfe (= 0), Cov(Y,F)  Cyf, and Cov(Y,Y)  Cyy contain
cc   sampling noise.


      IF(ORT<1) THEN
        WRITE(16,'("  Allowing sampling noise to degrade the unique ",
     +    "factors'' stipulated"/"  orthogonalities yielded data-co",
     +    "variance sampling errors"//"  Cov(Y,Y) - Cyy:")')
        DO I = 1,NV
          WORD(:5) = '  y'//CF(:JJF(I))
          IF(I<10) WORD(:5) = '   y'//CF(:JJF(I))
          IF(I>99) WORD(:5) = ' y'//CF(:JJF(I))
          WRITE(16,'(A5,":",4(2X,5A5),20(:/14X,4(2X,5A5)))') WORD(:5),
     +      (CLN(W(LO(I,J)),5,2),J=1,I)
          IF(I<NV.AND.MOD(I,5)==0) WRITE(16,'()')
        END DO
      END IF
C      Twisting by parameters    RA     RB    P  smooth.
C             Common factors:  xx.xx  xx.xx  .xx   n
C             Unique factors:

      IF(MTW==1) THEN
        WRITE(16,'(/" TWIST EFFECTS.")')
        WRITE(16,'(6X,"Twisting by parameters",4X,"RA",5X,"RB",4X,
     +  "P  smooth"/13X,"Common factors:",2A7,A5,3X,L1/13X,"Unique f",
     +  "actors:",2A7,A5,3X,L1)') ( CLN(TP(J,1),7,2), CLN(TP(J,2),7,2),
     +  CLN(TP(J,3),5,2), (TP(J,4)>.5),J=1,2)
        WRITE(16,'(" gave the score population the following higher",
     +    " factor moments (Normal: Skew = 0, Kurtosis = 3.0)")')
        WRITE(16,'(/" Common-factor skew: "/20(5(2X,5(1X,A4)):/))')
     +    (CLN(TWIST(I,4),4,2),I=1,NF)
        WRITE(16,'(/" Common-factor kurtosis:"/20(5(2X,5(1X,A4)):/))')
     +    (CLN(TWIST(I,5),4,2),I=1,NF)
        WRITE(16,'(/" Unique-factor skew:"/20(5(2X,5(1X,A4)):/))')
     +    (CLN(TWIST(I,4),4,2),I=NF+1,NVF)
        WRITE(16,'(/" Unique-factor kurtosis:"/20(5(2X,5(1X,A4)):/))')
     +    (CLN(TWIST(I,5),4,2),I=NF+1,NVF)
      END IF
      OPEN(29,FILE='LASTNS'); WRITE(29,*) NS; CLOSE(29)
C=================================================================

C Record input parameters in head of logfile <<======== Score omission re-enters here
C Start recording results.  Want (a) data/factor covars and names in Hydata-standard
C format; (b) production details (LOG info); (c) data/factor population scores
401   F1 = F1(:LDOT)//'COV'; LL = LDOT-1; L = 0
      F2 = F1(:LDOT)//'LOG'; F3 = F1(:LDOT)//'POP' ! ^ Needed for default name
       WRITE(6,'(/"  Results are ready to save under basename ",A,
     +  " derived from the raw"/"  pattern''s filename.  To accept ",
     +  "this default (not likely best), hit RETURN."/"  Otherwise, ",
     +  "enter a preferred basename.  If you expect this to recur as",
     +  " stem"/"  in the names of other files derived from this bas",
     +  "e, it should be no longer"/"  than six or at most seven cha",
     +  "racters in order to leave branch room."/)') F1(:LL)
405   IF(L/=0) WRITE(6,'("  Your basename for this simulation''s out",
     +  "put is now ",A,".  Hit RETURN"/"  if OK, or enter an altern",
     +  "ative."/)') F1(:LL)
      READ(5,'(A)') WORD; L=MIN(8,LAST(WORD))
      IF(L/=0) THEN; CALL CAP(WORD,L); J = 0
407     J = J+1; IF(J<L.AND.LLT('0',WORD(J+1:J+1))) GOTO 407
        L = J; LL = J; F1 = WORD(:LL); GOTO 405
      END IF
      F1 = F1(:LL)//'.COV'
      F2 = F1(:LL)//'.LOG'; F3 = F1(:LL)//'.POP'
      INQUIRE(FILE=F1,EXIST=QC)
      INQUIRE(FILE=F2,EXIST=QL)
      INQUIRE(FILE=F3,EXIST=QD)
      IF(QC) WRITE(6,'("  A COV-file (ASCII data covariances) with",
     +  " this basename already exists here.")')
      IF(QL) WRITE(6,'("  A LOG-file (ASCII product information) ",
     +  "with this basename already exists here.")')
      IF(QD) WRITE(6,'("  A POP-file (binary score population) with",
     +  " this basename already exists here.")')
      IF(QD.AND..NOT.QSCOR) WRITE(6,'("  NOTE: Even though no new po",
     +  "pulation scores are to be saved, this old"/"  scorefile wi",
     +  "ll be deleted if these new products usurp its basename."/)')
      IF(QC.OR.QL.OR.QD) THEN
410     WRITE(6,'(/"  To overwrite that old material with this sim",
     +    "ulation''s new results,"/"  hit RETURN.  Otherwise, ent",
     +    "er anything to select a new basename.")')
        CALL SCN(L); IF(L/=0) GOTO 405  ! Need L for branch to 405
        WRITE(6,'("  To confirm deleting the old files with basena"
     +    "me ",A,", enter anything."/"  Otherwise, hit RETURN to",
     +    " reconsider.")') F1(:LAST(F1)-4)
        CALL SCN(J); IF(J==0) GOTO 410          !  I goes from 1 to NF,
        OPEN(11,FILE=F3,FORM='UNFORMATTED')     !  J mods over 1 - 26
        CLOSE(11,STATUS='DELETE')               !  M = 1 + I/26    position
      END IF                                    !  J = 1+MOD(I-1,26) letter
      IF(.NOT.QSCOR) GOTO 420
      DO K = 1,4      ! Give the factors sequential lower-case letter names
        M = 26*(K-1)+1; IF(M>NF) EXIT
        DO I = M,NF     !  I starts at 1+26*(K-1)
          IF(K==1) IDENT(NV+I)(K:K) = CHAR(97+MOD(I-1,26))
          IF(K>1) IDENT(NV+I)(K:K) = IDENT(NV+I)(K-1:K-1)
        END DO
      END DO
      NC = 0
      DO I = 1,NV+NF; K = LAST(IDENT(I))
        DO J = 1,K;  NC = NC+1; CWRD(NC) = IDENT(I)(J:J); END DO
        NC = NC+1; CWRD(NC) = '.'
      END DO     ! ^ CWRD is the dot-delimited string of variable names
      OPEN(7,FILE=F3,FORM='UNFORMATTED')   ! F3 is POP-file, CM holds communalities
      WRITE(7) NV, NF, NC, MTW, NS, ORT, KOP, LBW
      WRITE(7) (CM(I),I=1,NV), (CWRD(I),I=1,NC)
      IF(MTW>0) WRITE(7) ((TP(I,J),I=1,2),J=1,4),  ! TWIST and its params TP not
     +           ((TWIST(I,J),I=1,NVF),J=1,7)      ! read if MTW=0 (not twisted
      WRITE(7) ((P(I,J),I=1,NV),J=1,NF)  ! P is the refined simple-structure pattern
      WRITE(7) (CV(I),I=1,NVV), (CFF(I),I=1,NFF)
C       CV holds data covars; CFF, SS-factor covars.
      IF(KOP==3) THEN                     ! ??? Why write P as well as WRD ???
        WRITE(7) LA, (WRD(K)(:LA),K=1,LBW) ! LA is length of each table line
CC        DEALLOCATE( WRD, P )      ! No real point
      END IF
C  **************** Formerly included the data/SS-factor covars: Restore??
414   IF(MTW==1 .AND. NS>9999) THEN
        WRITE(6,'(" This dataset is huge (NS = ",A,").  If you don''t",
     +    " want the individual"/" score records but only their stati",
     +    "stical summary, enter anything to waive"/" score archiving",
     +    ".  Otherwise, hit RETURN to save the complete package.")')
     +    CF(:JJF(NS))
        CALL SCN(J); IF(J==0) GOTO 415
        WRITE(6,'(" Hit RETURN to confirm score omission, or enter ",
     +    "something to reconsider.")')
        CALL SCN(J); IF(J==0) GOTO 420; GOTO 414
      END IF
415   J = JJF(NS); KK = MAX(100,MIN(100000,10**(J-1)))  ! WRITE(6,'()')
      DO NR = 1,NS   ! Scores are on data vars followed by SS factors
        READ(10,REC=NR) (U(I),I=1,NVF)
CC        IF(QS .AND. ORT<0) THEN  ! Only if ORT = -1 is reinstated
CC          DO I = 1,NV; U(I) = SV(I)*U(I); END DO
CC        END IF   !  ^ Only the datascores, not factors
        WRITE(7) (U(I),I=1,NVF)
        IF(MOD(NR,KK)==0) WRITE(6,'(4X,"Archiving record ",A)')
     +    CF(:JJF(NR))
      END DO
      CLOSE(7)

C Save data covariances, Hyball-standard format, in ASCII file *.COV.
420   OPEN(13,FILE=F1)  ! F1 is COV-file
C Save correlations and supporting information in ASCII file *.COV (F1).
C Names are used by MODA and HYBALL
      WRITE(13,'("Standardized data covariances (",I1," decimals, poi"
     +  "nt omitted) produced by simulation documented in textfile ",A,
     +  ":")') NND, F2(:LAST(F2)); KP = 10**NND; M = NVV
      IF(KLT==0) M = NV*NV
      WRITE(13,'(2(1X,A))') CF(:JJF(NV)), CF(:JJF(M))
      IF(LWRP==1 ) THEN    ! Rem: KLT=1 is LowerTriangle
        IF(KLT==1) WRITE(13,FMT) (NINT(KP*CV(J)),J=1,NVV)
        IF(KLT==0) THEN
          DO I = 1,NV
            WRITE(13,FMT) (NINT(KP*CV(LOC(I,J))),J=1,NV)
          END DO
        END IF
      END IF
      IF(LWRP==0) THEN ! Line breaks at end of array rows
        DO J = 1,NV
          JJ = J; IF(KLT==0) JJ = NV
          WRITE(13,FMT) (NINT(KP*CV(LOC(I,J))),I=1,JJ)
        END DO
      END IF
      WRITE(13,'(/"Names of the variables, letter-indexing their sal",
     +  "ient factors:")')
      N = LLEN/(1+MAXL)
      WORD = '(80('//CF(:JJF(N))//'(1X,A),:/))'
      WRITE(13,WORD) (IDENT(I)(:LAST(IDENT(I))),I=1,NV)  ! Fix length for transfer read
      CLOSE(13)

C File population information in ASCII file *.LOG.
C **** QSCOR=t iff scores;
      OPEN(12,FILE=F2)  ! F2 is LOG-file
      WRITE(12,'(22X,"GENSCOR DATA CREATION")')
      CALL DAY(12)
      WRITE(12,'(/" Production of simulation ",A," for ",A," data var",
     +  "iables Y with ",A," common factors F.  The")') F3(:LAST(F1)-4),
     +  CF(:JJF(NV)), CF(:JJF(NF))
C The simulation model is Y = WF + E, Cyy = WCffW' + De  (diagonal Cee = De, Cfe = 0)
      WRITE(12,'(" simulation model is Y = WF + E, Cyy = WCffW'' +",
     +  " De  (Cfe = 0, De is diagonal Cee).")')
      REWIND 15; REWIND 16
430   READ(15,'(A)',END=435) WORD
      WRITE(12,'(A)') WORD(:LAS(WORD))
      GOTO 430      !        ^No typo; it suppresses L-justification
435   READ(16,'(A)',END=440) WORD
      IF(WORD(:3)=='###') THEN
        WRITE(12,'(/"  A size-",A," population of joint scores on <Y,",
     +    "F> having this stipulated structure"/"  has been saved in",
     +    " binary file ",A,".  The joint distribution")')
     +    CF(:JJF(NS)), F3
        GOTO 435
      END IF
      WRITE(12,'(A)') WORD(:LAS(WORD))   ! Copying rows of file 16
      GOTO 435      !       ^No typo; it suppresses L-justification
440   IF(QSCOR) WRITE(12,'(/" >>>> REMINDER: Binary scorefile ",A,
     +  " cannot be read in a text editor."/16X,"To transcribe this",
     +  " or extractions from it into an ASCII file"/16X,"with HYDA",
     +  "TA-standard format, run it through program MORSCOR."/)')
     +  F3(:LAST(F3))
      WRITE(6,'(/" >>>> Production of this simulation material is com",
     +  "plete."/6X,"See ASCII file ",A," for product information.")')
     +  F2(:LAST(F2))
      CLOSE(15,STATUS='DELETE'); CLOSE(16,STATUS='DELETE')
      STOP
      END
C
      SUBROUTINE CAP(WORD,L)
C This makes the first L letters in WORD all upper-case.
      CHARACTER WORD*(*)
      DO I = 1,L
        N = ICHAR(WORD(I:I))
        IF(N>=97 .AND. N<=122) WORD(I:I) = CHAR(N-32)
      END DO
      END SUBROUTINE
C
      FUNCTION JJF(N)  ! Standard name JF() pre-empted by pattern coding
C Return character expression of integer N left-justified in field CF; then
C CF(:JJF(N)) writes N with exactly the right length in format specifier A.
C *** Haven't found any way to avoid requiring N to be INTEGER(4).
      CHARACTER(12) CF
      INTEGER(4) K
      COMMON /CF/ CF
      K = ABS(N)
      CF = '            '
      J = 13
10    J = J-1
      CF(J:J) = CHAR(48+MOD(K,10))
      K = K/10
      IF(K>0) GOTO 10
      IF(N<0) CF(J-1:J-1) = '-'
      CF = ADJUSTL(CF)
      JJF = LEN_TRIM(CF)
      END FUNCTION
C
      FUNCTION CLN(X,NFF,ND)
C Express real number X as a character string in fieldwidth NF with LD decimals
C when room, where LD = ABS(ND).  ND<0 tries to precede all positive numbers
C with a blank. LD > 9 displays zero decimals and no decimal point while
C returning X = 0 as ND-10 zeros right-justified in selected field.
C ***** WARNING: If a call of CLN prints garbage, you have forgotten to
C       declare CLN as CHARACTER*8 in the calling routine.
      PARAMETER (KW=16)
      CHARACTER CLN*8, WK(KW)
      CLN = '        '
      NF = MIN(8,NFF)  ! Limit fieldwidth to 8 chars
      LD = ABS(ND)
      IF(ABS(X)<1.0E-12) THEN !  Special for vanishingly small X
        N = MIN(NF-1,MOD(LD,10))
        IF(LD<10) CLN(NF-N:NF+1-N) = '.0'
        IF(LD<10) RETURN
        DO I = 1,N
          CLN(NF+1-I:NF+1-I) = '0'
        END DO
        RETURN
      END IF
      M = NF; IF(ABS(X)>1.) M = NF-1-INT(LOG10(ABS(X))) !  M is space free for decimal (or M-1 if X<0) )
      IF(M<0 .OR. M<1.AND.X<0.) GOTO 55
      IF(ND>=10) LD = 0; LD = MIN(LD,M)
      DO I = 1,KW
        WK(I) = ' '
      END DO
      IF(LD==0) N = NINT(ABS(X))         ! Shouldn't be needed, but is
      IF(LD/=0) N = NINT(ABS(X)*10**LD)  ! This can overflow if LD is large
      DO I = KW,KW-LD+1,-1
        WK(I) = CHAR(48+MOD(N,10))
        N = N/10
      END DO
      WK(KW-LD) = '.'
      IF(N==0 .AND. LD==0) WK(KW-LD-1) = '0'
      IF(N==0 .AND. X<0. .AND. LD>0) WK(KW-LD-1) = '-'
      DO I = KW-LD-1,2,-1
        IF(N>0) WK(I) = CHAR(48+MOD(N,10))
        IF(N>0 .AND. X<0.) WK(I-1) = '-'
        N = N/10
      END DO
      KS = KW-NF+1
25    KS = KS-1
C       Field returned will start at WK(KS+1) for initial KS
      IF(KS<=0) GOTO 50
      IF(ND>=0) THEN
        IF(WK(KS)/=' ' .AND. WK(KS+NF)/='.') GOTO 25
      ELSE
        IF(WK(KS+1)/=' '. AND. WK(KS+1)/='-' .AND. WK(KS+NF)/='.')
     +    GOTO 25
      END IF
      IF(ND>=10) KS = MAX(1,KS-1)
      DO I = 1,NF
        CLN(I:I) = WK(KS+I)
      END DO
50    IF(WK(KS)==' ') RETURN
55    CLN = ' *******'
      END FUNCTION
C
      SUBROUTINE DAY(N)
C This date-stamps output file N, with K spaces at start of printline
      CHARACTER ZZZ*8, WORD*40, CF*12, LST(12)*10
      COMMON /CF/ CF
      DATA LST/'January  7','February 8','March    5','April    5',
     +         'May      3','June     4','July     4','August   6',
     +         'September9','October  7','November 8','December 8'/
      CALL DATE_AND_TIME(ZZZ)  ! Last three outputs not used here
      M = 10*(ICHAR(ZZZ(7:7))-48) + ICHAR(ZZZ(8:8))-48  ! M gets day of month
      READ(ZZZ(5:6),*) L; K = ICHAR(LST(L)(10:10))-48
      WORD = CF(:JJF(M))//' '//LST(L)(:K)//' '//ZZZ(:4)
      WRITE(N,'(/16X,"Date of this run:  ",A)') WORD
      END SUBROUTINE
C
      SUBROUTINE DEKOD(CH,KRP,NV,NF,JV,JF,JG,LST,KC,MB)
C CH(1:2) is letter code for <JF,NFB,NVB>; CH(3:4) is number code for
C complexity list; KRP is complexity-1 recurrence count in master block;
C JG is number of general factors in master block.  Unpack these and report
C to screen. Complexity list is returned as a character string in LST and
c as an integer in KC.
C *** LST should get at most 15 characters, but tolerates spillover.
      CHARACTER CH*(*), LST*(*), CF*12
      COMMON /CF/ CF
      KD = 26*(ICHAR(CH(1:1))-65) + (ICHAR(CH(2:2))-65)
      JF = 1 + KD/100
      NFB = 1 + MOD(KD/10,10)
      NVB = 1 + MOD(KD,10)
      K = 10*(ICHAR(CH(3:3))-48) + (ICHAR(CH(4:4))-48)
C       K is the 2-digit code for complexity levels  2.
      K = 2*K + MIN(1,KRP)  ! Expand to include complexity-1 info
      KC = 0; LL = 0
      DO L = 1,MB
        IF(MOD(K,2)>0) THEN
           KC = 10*KC + L
           LL = LL+2
           LST(LL:LL) = CHAR(48+L)
           IF(L==1 .AND. KRP>1) THEN
             J = JJF(KRP); LST(LL+1:LL+J+1) = '*'//CF(:J); LL = LL+J+1
           END IF
C            LST lists the realized complexity levels (spaced numeric characters)
        END IF
        K = K/2
      END DO
      IF(JG>0) LST(LL+2:LL+3) = 'G'//CHAR(48+JG)
      JV = MAX(0,KRP-1)*(JF-JG)
      L = KC
20    K = MOD(L,10)
      JV = JV + KOMB(JF-JG,K)
      L = L/10
      IF(L>0) GOTO 20
      NV = JV*NVB
      NF = JF*NFB
      RETURN
      END

      SUBROUTINE EZCOV(NF,NFF,CFF,NC,BL,BH,IER)
C The random selection of covs returned in CFF requires CFF to be sym-storage
      CHARACTER CLN*8, CF*12, C4*4, CH1
      INTEGER KOV(NFF)
      REAL CFF(NFF), R(NF), X(2)
      COMMON /CF/ CF
      LO(I,J) = J*(J-1)/2 + I
      LOC(I,J) = LO(MIN(I,J),MAX(I,J))

C  Ŀ
c    Creation here of a semi-random Cff for xx factors requires you to enter 
c                                                                            
c    1) an integer NC and if NC>0 an interval <BL,BH> within limits -1 to +1 1
c       such that NC of the xxx distinct proper correlations are to receive  2
c       random values provisionally between BL and BH while the rest are     5
c       zero.  And unless you make Cff fully orthogonal by stipulating       7
c       NC = 0, you will also be invited to choose                           
c                                                                            
c    2) a value LOW between 0 and 1 to be the smallest eigenvalue of the     
c       created correlation matrix.                                          
c                                                                            
c       Given these parameters, SEMCOV first generates an order-NF symmetric 1
c    matrix Q with unit diagonal and zeros or random values between BL and   3
c    BH elsewhere.  It then tweaks Q into a correlation matrix Cff having    4
c    the same off-diagonal correlation ratios and zeros (orthogonalities)    4
c    as Q while making LOW its smallest eigenvalue.                          26
C  

      CALL SYSTEM('cls');  NT = NFF-NF; NC = 0
      WRITE(6,'("  ",74(""),"")'); JJ = JJF(NF); C4 = '    '
      WRITE(6,'("    Creation here of a semi-random Cff for ",A,
     +  " factors requires you to enter",A,"")') CF(:JJ), C4(:3-JJ)
      WRITE(6,'("  ",74(" "),"")'); JJ = JJF(NT)
      WRITE(6,'("    1) an integer NC and if NC>0 an interval <BL,B",
     +  "H> within limits -1 to +1 "/2X,"",5X,"such that NC of th",
     +  "e ",A," distinct proper correlations are to receive",A,""/
     +  2X,"",5X,"random values provisionally between BL and BH whi",
     +  "le the rest are",5X,""/2X,"",5X,"zero.  And unless you ma",
     +  "ke Cff fully orthogonal by stipulating",7X,""/2X,"",5X,
     +  "NC = 0, you will also be invited to choose",27X,"")')
     +  CF(:JJ), C4(:5-JJ )
      WRITE(6,'("  ",74(" "),"")')
      WRITE(6,'("    2) a value LOW between 0 and 1 to be the small",
     +  "est eigenvalue of the",5X,""/2X,"",5X,"created correlation",
     +  " matrix.",42X,"")')
      WRITE(6,'("  ",74(" "),"")')
      WRITE(6,'("  ",5X,"Given these parameters, SEMCOV first gen",
     +  "erates an order-NF symmetric "/2X,"  matrix Q with unit ",
     +  "diagonal and zeros or random values between BL and   "/2X,
     +  "  BH elsewhere.  It then tweaks Q into a correlation matri",
     +  "x Cff having    "/2X,"  the same off-diagonal correlation",
     +  " ratios and zeros (orthogonalities)    "/2X,"  as Q while",
     +  " making LOW its smallest eigenvalue.",26X,"")')
      WRITE(6,'("  ",74(""),"")'); CALL WAIT(0)
      CFF = 0.; DO J = 1,NF; CFF(LO(J,J)) = 1.; END DO   ! Orthnorm Cff default
11     WRITE(6,'(/3X,"The count NC of distinct factor correlations to ",
     +  "make oblique (nonzero)"/3X,"is now set at ",A," out of a maxi",
     +  "mum ",A,".  Hit RETURN if OK, or enter the"/3X,"number of obl",
     +  "iquities you prefer in Cff.)"/)') CF(:JJF(NC)), CF(:JJF(NT))
      CALL SCANB(J,1,'I',5,CH1,0); IF(J<0) GOTO 11
      IF(J>0) THEN; READ(2,*) M; NC=MIN(ABS(M),NT); GOTO 11; END IF
      IF(NC*(NT-NC)==0) THEN
        IF(NC==0) WRITE(6,'(" Hit RETURN to confirm that you want ",
     +    "only orthogonal factors.")')
        IF(NC==NT) WRITE(6,'(" Hit RETURN to confirm that you don''t",
     +    " require any factor orthogonalities.")')
        WRITE(6,'(" Otherwise, enter anything to reconsider your NC ",
     +     "setting.")')
        CALL SCN(J); IF(J/=0) GOTO 11
        IF(NC==0) RETURN     ! Orthogonal Cff already set as default
      END IF
      KOV = (/(I,I=1,NFF)/)
      IF(NC==NT) GOTO 30  ! In this case exend NC to NFF and later fix diagonal
      DO I = 1,NF; KOV(LO(I,I)) = 0; END DO
      DO I = 0,NC-1
        I1 = I+1
21      K = MIN(NFF,I1+INT((NFF-I)*RANDY()))
        IF(KOV(K)==0) GOTO 21
        L = KOV(K); KOV(K) = KOV(I1); KOV(I1) = L
      END DO ! ^KOV now lists NC random off-diagonal sym-storage coordinates
30    WRITE(6,'(/" Ready to randomize oblique factor correlations be",
     +  "tween limits now set at"/" <",A3,",",A4,">.  Hit RETURN if ",
     +  "these limits are OK, or enter new pair."/)') CLN(BL,3,2),
     +  CLN(BH,4,2)
      CALL SCANB(J,0,'R',5,CH1,0)
      IF(J==0) GOTO 40
      IF(J<0) GOTO 30
      READ(2,*) (X(I),I=1,MIN(J,2)); IF(J==1) X(2) = X(1)
      IF( ABS(X(1)) >1. .OR. ABS(X(1)) >1. ) THEN
        DO I = 1,2; IF( ABS(X(I)) >1.) X(I) = X(I)/10; END DO
      END IF
      BL = MAX(-1.,MIN(1.,X(1),X(2)))
      BH = MIN(1.,MAX(-1.,X(1),X(2)))
      GOTO 30
40     IF(ABS(BH)<.01) THEN
        WRITE(6,'(" Hit RETURN to confirm your belated call for all-o",
     +    "rthogonal factors."/" Otherwise, enter anything to reconsi",
     +    "der.")')
        CALL SCN(J); IF(J/=0) GOTO 30
        CFF = 0.; DO I = 1,NF; CFF(LO(I,I)) = 1.; END DO
        ZLO = 1.;  RETURN
      END IF
C Generate raw Cuu
      BD = BH-BL; NN = 0; MM = 20
50    DO I = 1,NC
        CFF(KOV(I)) = BL + BD*RANDY()
      END DO
      DO I = 1,NF; CFF(LO(I,I)) = 1.; END DO
      CALL EIGS(NF,NF,CFF,NFF,R,X,NV,0,IER,6); ZLO = R(NF)
      IF(ZLO<.001.AND.NN<=MM) GOTO 50    ! Precaution surely never needed
      ZLO = .1; CALL TWEAK(NF,NFF,CFF,R,ZLO,ZLB,ZZM,'Cuu',IER,0)
      END SUBROUTINE ! ^ wants the covs to be in sym-storage

      SUBROUTINE GETL(KFL,CH,L,N,K,IER)
C Find the Nth line in formatted file KFL starting with char-string CH(:L)
C left-justification; and position KFL for SCAN to read that line if K = 0
C or the next line if K>0. (Repositioning can easily be generalized.)
C **** Can easily extend this for more diverse searches
      CHARACTER CH*(*), WORD*200  !, CH2*2
CC      IF(N==1) CH2='st'; IF(N==2) CH2='nd'
CC      IF(N==3) CH2='rd';IF(N>3) CH2='th'
      REWIND KFL; IER = 0; NN = 0
10    READ(KFL,'(A)',END=50,ERR=50) WORD
      J = LAST(WORD); IF(J<L) GOTO 10
      IF(CH(:L)/=WORD(:L)) GOTO 10
      NN = NN+1; IF(NN<N) GOTO 10
      IF(K==0) BACKSPACE KFL
      RETURN
50    IER = 1 ! Failure to find CH may set informed continuation
      END SUBROUTINE

      SUBROUTINE INVF(W,NV)
C This triangularly Gram-factors an order-NV symmetric matrix W in
C sym-storage and returns the inverse of this triangular pattern in W.
Conceptually, both the gram-factor and its inverse are lower-triangles.
      REAL(8) W(*)
      LO(I,J) = J*(J-1)/2 + I
C  Replace W by its lower-triangular Gram-factor
      DO I = 1,NV
        I0 = I-1; I1 = I+1; II = LO(I,I)
        DO K = 1,I0   ! Not executed if I=1
          W(II) = W(II) - W(LO(K,I))**2
        END DO
        IF(W(II)<1.0E-35) THEN
          WRITE(6,'(" *** Subroutine INVF cannot invert ill-",
     +      "conditioned matrix"/)'); STOP
        END IF
        W(II) = SQRT(W(II))
        DO J = I1,NV  ! Not executed if I=NV
          IJ = LO(I,J)
          DO K = 1,I0  ! Not executed if I=1
            W(IJ) = W(IJ) - W(LO(K,J))*W(LO(K,I))
          END DO
          W(IJ) = W(IJ)/W(II)
        END DO
      END DO
C Invert triangular Gram-factor
      W(LO(1,1)) = 1./W(LO(1,1))
      IF(NV==1) RETURN
      DO I = 2,NV
        I0 = I-1; II = LO(I,I)
        DO J = 1,I0; X = 0.
          DO K = J,I0
            X = X - W(LO(K,I))*W(LO(J,K))
          END DO
          W(LO(J,I)) = X/W(II)
        END DO
        W(II) = 1./W(II)
      END DO
      END SUBROUTINE

      SUBROUTINE ISORT(N,LST)
C Sort LST integers into descending or ascending order
      INTEGER LST(*)
10    DO J = 2,N
        L = LST(J)
        DO 11 I = J-1,1,-1
         IF(LST(I)<=L) GOTO 12    ! Increasing order
C         IF(LST(I)>=L) GOTO 12    ! Decreasing order
11       LST(I+1) = LST(I)
        I = 0
12      LST(I+1) = L
      END DO
      END SUBROUTINE
C
      FUNCTION LAST(WORD)
C This left-justifies leading substring WORD(:M) of WORD, and returns its
C length as L. If the string is empty, LAST is returned as 0.
      CHARACTER WORD*(*)
      WORD = ADJUSTL(WORD)
      LAST = LEN_TRIM(WORD)
      END

      FUNCTION LAS(WORD)
C Same as LAST except for evasion of left-justification
C length as L. If the string is empty, LAST is returned as 0.
      CHARACTER WORD*(*)
CC      WORD = '$'//WORD; WORD = ADJUSTL(WORD)
CC      WORD = WORD(2:)
      LAS = LEN_TRIM(WORD)
      END

      SUBROUTINE LOOK(K,GET,NAME,ML,KNT)
C LOOK calls DOS to delete or report files named in GET, with action set by K.
C K/2 = 0,1 initial-lists/append-lists GET in buffer file ZZZ. MOD(K,2) = 0,1
C stores/shows-to-screen the GET list. So K=0 initial-stores; K=1 initial-
C shows; K=2 append-stores; and K=3 append-shows. The last filename matching
C the GET template is returned in WORD with its length M.  NAME returns the
C list of template matches, the number of which is NL, and prints the numbered
C list to screen.  ML is max NL allowed by array allocation.
C ***** This variant uses negative input on KNT to suppress screen write
      LOGICAL PRM, QY
      CHARACTER*12 GET*(*), NAME(*), WORD*40, WD*40, TMP
      CHARACTER E   ! <<<  Special characters as needed
      SAVE NL  ! NL must NOT be disturbed by KNT if K > 1
      NOSHO = KNT; IF(K<2) NL = 0  ! Shd always have K  1 on 1st call
      WD = GET; M = LAST(WD); LL = M
CC      LL = 1    ! Why set LL this old way, or distinguish it from M ???
CC5     LL = LL+1; IF(LL<M) GOTO 5   ! Some old code glitch ??
      WD(M+2:M+LL+1) = WD(:LL); WD(40:40) = ' '  ! Shd now always have LL = M
      DO I = M+2,M+LL+1
        IF(WD(I:I)=='*' .OR. WD(I:I)=='?' ) THEN
          WD(40:40) = '!'; WD(I:I) = '!'
        END IF
      END DO
      INQUIRE(FILE=WD(M+2:M+LL+1),EXIST=QY)
      IF(.NOT.QY) OPEN(19,FILE=WD(M+2:M+LL+1)) ! Precludes no-match error message
      IF(K/2==0) CALL SYSTEM('dir '//WD(:M)//' >ZZZ')
      IF(K/2>0) CALL SYSTEM('dir '//WD(:M)//'>>ZZZ')  ! Append
      IF(.NOT.QY) CLOSE(19,STATUS='DELETE')
      IF(MOD(K,2)==0) RETURN
      OPEN(4,FILE='ZZZ')
10    READ(4,'(A)',END=50) WORD
      IF(WORD(25:26)==' 0' .OR. WORD(16:16)=='<') GOTO 10  ! No directory names
C Special filter for schemata
      E = WORD(:1); IF(E==' ' .OR. E=='.') GOTO 10
      IF(ABS(NOSHO)<ML .AND. WORD(:3)=='PAT') GOTO 10  ! GENPAT-call exclusion
      IF(WORD(:3)=='COV' .AND. LGT(WORD(4:4),'9')) GOTO 10
      IF(WORD(:3)=='KOV' .AND. LLE(WORD(4:4),'9')) GOTO 15
      IF(WORD(:3)=='PAT' .AND. LGT(WORD(4:4),'9')) GOTO 10
      IF(WORD(:4)=='APAT' .AND. LGT(WORD(6:6),'9')) GOTO 10
      IF(LGT(WORD(10:10),'9')) GOTO 10 ! KOV allows letter extensions
15    L = LAST(WORD(:14))      ! LGE is Lexically Greater Than
      DO I = 1,L; IF(WORD(I:I)=='!') GOTO 10; END DO
      L = 0
20    L = L+1; IF(WORD(L+1:L+1)/=' ' .AND. L<8) GOTO 20
      IF(WORD(10:10)/=' ') WORD(L+1:L+4) = '.'//WORD(10:12)
      DO I = 1,NL  !  Eliminate possible duplications
        IF(NAME(I)(:L+4)==WORD(:L+4) .AND. NAME(I)(L+5:L+5)==' ')
     +    GOTO 10
      END DO
      NL = NL+1
      NAME(NL) = WORD(:L+4)//'     '
      IF(NL<ML) GOTO 10
50    CLOSE(4,STATUS='DELETE')
      DO J = 2,NL
        TMP = NAME(J)
        DO I = J-1,1,-1
          IF(.NOT.PRM(NAME(I),TMP)) GOTO 55    ! Increasing order
          NAME(I+1) = NAME(I)
        END DO
        I = 0
55      NAME(I+1) = TMP
      END DO
      KNT = NL
      IF(NOSHO<0) RETURN
      IF(NL>0) WRITE(6,'(20(:/4(I5,". ",A,:)))') (I,NAME(I),I=1,NL)
      IF(NL>0) WORD(:12) = NAME(NL)  ! Why???
      RETURN
      END

      FUNCTION PRM(NAM1,NAM2)
C Consider both base and extension when flagging which filename comes first
C in alphanumeric listing.  PRM=t says that NAM2 comes before NAM1.
      LOGICAL PRM
      CHARACTER NAM1*(*), NAM2*(*)
      PRM = .FALSE.
      L1 = LAST(NAM1); L2 = LAST(NAM2)
      N1 = L1+1; N2 = L2+1  ! Find dot positions, if any
10    N1 = N1-1; IF(NAM1(N1:N1)/='.' .AND. N1>1) GOTO 10
20    N2 = N2-1; IF(NAM2(N2:N2)/='.' .AND. N2>1) GOTO 20
      IF(N1==1) N1=L1+1; IF(N2==1) N2=L2+1  ! Where dot would be were there one
      JJ = KPRM(NAM1(:N1-1),NAM2(:N2-1))
      IF(JJ==-1) PRM = .TRUE.; IF(JJ/=0) RETURN  ! JJ=0 if basenames are same
      IF(JJ==0 .AND. (N1>L1 .OR. N2>L2)) THEN  ! An extension is blank
        IF(N2>L2) PRM = .TRUE.; RETURN
      END IF
      JJ = KPRM(NAM1(N1:L1),NAM2(N2:L2))  ! Include extension dot
      PRM = .FALSE.; IF(JJ==-1) PRM = .TRUE.
      END FUNCTION

      FUNCTION KPRM(WRD1,WRD2)
C Return value 0 if WRD1=WRD2; otherwise  -1 or +1 according to whether WRD2
C comes before or after WRD1 in alphanumeric sequence.
      INTEGER KPRM
      CHARACTER WRD1*(*), WRD2*(*)
      L1 = LAST(WRD1); L2 = LAST(WRD2)
      KPRM = 0; IF(WRD1(:L1)==WRD2(:L2)) RETURN
      N1 = L1+1; N2 = L2+1    ! Find position preceding terminal number string
10    N1 = N1-1; N = ICHAR(WRD1(N1:N1))
         IF(N>47.AND.N<58 .AND. N1>1) GOTO 10
20    N2 = N2-1; N = ICHAR(WRD2(N2:N2))
         IF(N>47.AND.N<58 .AND. N2>1) GOTO 20
      IF(LLT(WRD1(:N1),WRD2(:N2))) KPRM = 1
      IF(LLT(WRD2(:N2),WRD1(:N1))) KPRM = -1
      IF(WRD1(:N1)/=WRD2(:N2)) RETURN
      IF(N1==L1 .OR. N2==L2) THEN  ! A mumber terminus is blank
        KPRM = 1; IF(N2==L2) KPRM = -1; RETURN
      END IF
      READ(WRD1(N1+1:L1),*) K1; READ(WRD2(N2+1:L2),*) K2
      KPRM = 1; IF(K2<K1) KPRM = -1
      END FUNCTION
C
      FUNCTION KOMB(K,L0)
C This returns the number of combinations of K things taken L0 at a time.
C If L0 is negative, KOMB returns the number of permutations of K things.
      L = MIN(L0,K-L0)
      IF(L0<0) L = K
      KOMB = KMB(K,L)
      IF(L0<0 .OR. K<=0) RETURN
      KOMB = KOMB/KMB(L,L)
      RETURN
      END
C
      FUNCTION KMB(K0,M)
C Return the number of ordered M-selections from K0 things.
      KMB = 1
      IF(K0<=0 .OR. M<=0 .OR. M>K0) RETURN
      K = K0
10    KMB = KMB*K
      K = K-1
      IF(K>K0-M) GOTO 10
      RETURN
      END
C
      SUBROUTINE NAMM(F1,EXT,L)
C This receives a left-justified filename in F1, solves for <base> to be the
C part of F1 prior to '.' and preceding numerals, scans the sub-directory for
C the lowest i=1,2,... such that file <base>i.EXT does not already exist, and
C returns <base>i.EXT or maybe <bas>i.EXT in F1(:12) along with char-start length
      LOGICAL QY
      CHARACTER F1*(*), EXT*3, CF*12
      COMMON /CF/ CF
      L = 0
10    L = L+1; IF(L<8 .AND. F1(L:L)/=' ' .AND. F1(L:L)/='.') GOTO 10
CC      F1(L:) = '0.'//EXT(:3)
      I = 0
20    I = I+1; L = JJF(I); IF(I==10) L = L-1
      F1(MIN(L,7):) = CF(:JJF(I))//EXT(:3)
      INQUIRE(FILE=F1,EXIST=QY)
      IF(QY) GOTO 20
      L = L-1  ! L returns length of the base charstring
      END SUBROUTINE

      SUBROUTINE NAMS(NV,NF,IDENT,A,NON,MAXL)
C Finish coding items' names with their pattern saliences: IDENT(J) letter-lists
C item J's salient factors in decreasing loading size up a maximum of 6, to which
C a numerical index is appended if two or more items have the same letter profile.
      CHARACTER IDENT(*)*26, CF*12
      INTEGER LD(NF), LST(NV)
      REAL A(NV,*)
      COMMON /CF/ CF
      DO I = 1,NV          ! Get letter stems from raw pattern
        NK = LAST(IDENT(I))
        DO K = 1,NK
          M = ICHAR(IDENT(I)(K:K))-64  ! Index of Kth factor salient for item I
          LD(K) = 1000*NINT(1000.*(10.-ABS(A(I,M)))) + M
        END DO   ! ^ ISORT is ascending so largest loadings are made smallest
        CALL ISORT(NK,LD)
        DO K = 1,MIN(NK,6)
          IDENT(I)(K:K) = CHAR(64+MOD(LD(K),1000)) ! Recover letter index
        END DO  ! Letters in name are now in decreasing salience order
      END DO; NON = 0
      DO I = 1,NV
        L = LAST(IDENT(I)); IF(ICHAR(IDENT(I)(L:L))<58) CYCLE
        IF(I==NV) THEN; IDENT(NV)(L+1:L+1) = '1'; CYCLE; END IF
        K = 1; M = ICHAR(IDENT(I)(1:1)) - 64; IF(M==0) NON = NON+1
        LST(1) = 1000*NINT(1000.*(10. - ABS(A(I,M)))) + I
        DO J = I+1,NV
          IF(IDENT(I)==IDENT(J)) THEN
            K = K+1; M = ICHAR(IDENT(J)(1:1)) - 64
            LST(K) = 1000*NINT(1000.*(10. - ABS(A(J,M)))) + J
          END IF
        END DO
CC        IF(K==1) CYCLE ! Don't cycle; append digit even if singleton
        CALL ISORT(K,LST); L = LAST(IDENT(I)) ! L shd already be defined
        DO J = 1,K  ! All in LST have same length L
          IDENT(MOD(LST(J),1000))(L+1:) = CF(:JJF(J))
        END DO !   ^    item index
      END DO
      MAXL = 0; DO I = 1,NV; MAXL = MAX(MAXL,LAST(IDENT(I))); END DO
      END SUBROUTINE

      FUNCTION LOOP(NF,LST)  ! ,LPP)
C Determine if sparce NFNF matrix compressed in LST is recursive.  Each of NL
C integers in LST is 10000*i + 100*j + L for nonzero entry in KTL cell <i,j>.
C Function value returned is count LOOP of looped nodes; list of indices that
C are in loops are returned in LPP
      INTEGER LST(0:*), KTL(NF,NF) !, LPP(*)
      KTL = 0; LOOP = 0; LK = 1; NL = LST(0)
      DO I = 1,NL
        KTL( LST(I)/10000, MOD(LST(I)/100,100) ) = 1
      END DO
      DO WHILE(LK<NF)
        DO I = 1,NF; DO J = 1,NF; DO K = 1,NF
           IF(KTL(I,K)*KTL(K,J)==1) KTL(I,J) = 1
        END DO; END DO; END DO
        LK = LK*2
      END DO
      DO I = 1,NF
        IF(KTL(I,I)>0) THEN
          LOOP = LOOP+1  !; LPP(LOOP) = I
        END IF
      END DO
      END FUNCTION

      SUBROUTINE SKEME(NY,NF,F1,MM,B,LDB,FUZ,CUT,MSG)
C F1 is the uploaded schema, MM the largest index of stored files of F1's kind.
C B returns a full instantiation of schema F1. MSG: At input, >0 flags interactive
C show, <0 flags write to LOG buffer.  At output, MSG flags error condition.
C >>>>> In this variant, unless SKEME aborts schema instantiation it ends by
C       writing the schema and its current parameter specs to LOG-output buffer.
      CHARACTER*12  CF, F1, CLN*8, CH1, PL*2, CH4, WORD*160
      INTEGER  LKOD(0:3,26), LKBK(0:3,26), LST(0:NY*NF), LTRS(0:NY*NF)
      REAL  X(20), B(LDB,NF) ! Declare full B-size to allow whole-array ops
      NDIG(N) = (MAX(1,N)/48)*(57/MAX(1,N))  ! 1 if N is ICHAR for digit; otherwise, 0
      NLET(N) = (MAX(1,N)/64)*(90/MAX(1,N))  ! 1 if N is ICHAR for cap letter or @; otherwise, 0
      COMMON /CF/ CF
      SAVE LSHO  ! Flag for info display; unrelated to subroutine SHO
      PL = 's '  ! PL(K:); want K=2 if singular, otherwise K=1
      LSHO = 1; MSG = 0; CUT = .01
CC      KIND = 3; IF(F1(5:5)=='H') KIND = 2; IF(F1(5:5)=='R') KIND = 4
      KIND = 1; IF(F1(:1)=='C') KIND = 3  ! GENSCOR uses only PAT and COV schemata
      GOTO 30
23    WRITE(6,'(" Apologies: File ",A," is corrupt.  Try another.")')
     +  F1(:LAST(F1)); CLOSE(7); MSG = -1; RETURN
30    OPEN(7,FILE=F1,FORM='UNFORMATTED')
      READ(7,ERR=23) NL, (LST(I),I=1,NL); LST(0) = NL
      READ(7,ERR=23) LLET, ((LKOD(I,J),I=0,3),J=1,LLET)
31    CLOSE(7)
CCC      IF(KIND==2) THEN; IE = 0  !>>> Path schemata not called in GENSCOR

C ========= Start of schema display ==========   JOB
32    CALL SHO(NY,NF,KIND,F1,LST,LKOD,LLET,FUZ,1,6)
C JOB<=1 SHOs matrix schema; JOB=2 SHOs its letter specs; JOB > 2 SHOs both
CCC      IF(KIND==2) THEN    !>>> Path schemata not called in GENSCOR
      IF(LSHO>0.OR.NF>5) CALL WAIT(0) ! Flag LSHO is unrelated to sub SHO

C =============== Start of letter specifications ==================
41    DO I = 1,NL
        LTRS(I) = MOD(LST(I),100)     ! LTRS collects letter entries with repetitions
      END DO
      LLET = 0
      OUTER: DO L = 65,90    ! Alphabetize LTRS without repetitions
        DO N = LLET+1,NL
          IF(LTRS(N)==L) THEN
            LLET = LLET+1; LTRS(N) = LTRS(LLET); LTRS(LLET) = L
            CYCLE OUTER
          END IF
        END DO; IF(LLET==NL) EXIT
      END DO OUTER
      DO I = 1,LLET           ! Count occurrences of each letter (needed?)
        M = LTRS(I); KL = 0   ! and store with letter in LKOD(0,-)
        DO J = 1,NL
          IF(LTRS(J)==M) KL = KL + 1
        END DO
        LKOD(0,I) = M + KL*100
      END DO

C  Reminder: Each letter in this schema goes proxy for a triple of adjustable
C  parameters; Hi,Lo,Pr. [Lo,Hi] is the size range within which matrix elements
C  coded L will be randomized (or fixed if Lo = Hi), while this value is made
C  negative with probability Pr.  If you elect to revise the present settings,
C  recall that entering less than three numbers when editing the settings for
C  proxy L will set Pr = 0 and default Lo to Hi if you enter just one.

      IF(LSHO==0) GOTO 43
      WRITE(6,'("  Reminder: Each letter in this schema goes proxy f",
     +  "or a triple of adjustable"/"  parameters: Hi,Lo,Pr. [Lo,Hi]",
     +  " is the size range within which matrix elements"/"  coded L",
     +  " will be randomized (or fixed if Lo = Hi), while this value",
     +  " is made"/"  negative with probability Pr.  If you elect to",
     +  " revise the present settings,"/"  recall that entering less",
     +  " than three numbers when editing the settings for"/"  proxy",
     +  " L will set Pr = 0 and default Lo to Hi if you enter just ",
     +  "one.")'); LSHO = 0
      WRITE(6,'("  >> Note 1:  Numeric entries here cannot exceed thr",
     +  "ee digits and, if entered"/7X,"without a decimal point, are ",
     +  "presumed to start with one.  So make the"/7X,"decimal explic",
     +  "it in entries exceeding .999 or less than .010."/)')
      WRITE(6,'("  >> Note 2: Entry of a letter at any point will ju",
     +  "mp to display of what"/7X,"has been entered so far, followed",
     +  " by option to resume/revise these"/7X,"specifications or qu",
     +  "it with provision for finishing later."/)')
      PL = 's '  ! PL(K:); want K=2 if LK = 1, otherwise K=1
C
43    KIK = -1; LKBK = LKOD ! Count changes (none/some would suffice)
45    CALL SHO(NY,NF,KIND,F1,LST,LKOD,LLET,FUZ,2,6)
      IF(KIK<0) THEN
        WRITE(6,'(3X,"To accept these parameter specifications, hit ",
     +    "RETURN.  Otherwise, enter any"/3X,"letter to revise them, ",
     +    "or any numeral to see the schema deploying them again."/)')
        CALL SCN(J); IF(J==0) GOTO 99; IF(J>0) GOTO 32; GOTO 51
      END IF
      DO I = 1,3; DO J = 1,LLET
          IF(LKOD(I,J)/=LKBK(I,J)) KIK = KIK+1
      END DO; END DO
      IF(KIK>=0) THEN
        IF(KIK==0) WRITE(6,'(/"  You have left these settings unchang",
     +    "ed.  Enter something if you want to"/"  make another edit p",
     +    "ass at them.  Otherwise, hit RETURN to accept them.")')
        IF(KIK>0) WRITE(6,'(/7X,"To accept these revised parameter se",
     +    "ttings, hit RETURN.  Otherwise,"/7X,"enter anything to cor",
     +    "rect them.")')
        CALL SCN(J); IF(J/=0) GOTO 51
      END IF
47    LDT=7; IF(KIND==3) LDT=5; IF(F1(LDT+1:LDT+1)=='.') LDT=LDT+1
      WRITE(6,'("  If you want this schema stored with these altered",
     +  " parameter settings"/"  enter any letter to create a new ",
     +  A," file, or any integer to overwrite"/2X,A,". Otherwise, ",
     +  "hit RETURN to waive archiving these changes.")')
     +  F1(:LDT-1), F1(:LAST(F1))
      CALL SCN(J); IF(J==0) GOTO 99
      IF(J>0) THEN ! Integer calls rewrite
        WRITE(6,'("  Hit RETURN to confirm replacing the parameters ",
     +    "now saved in ",A," with"/"  their present values.  Otherw",
     +    "ise, enter anything to reconsider.")')
        CALL SCN(J); IF(J==0) GOTO 49; GOTO 47
      END IF
      F1 = F1(:LDT)//CF(:JJF(MM+1))
49    OPEN(7,FILE=F1,FORM='UNFORMATTED')
      WRITE(7) LST(0),  (LST(I),I=1,LST(0))
      WRITE(7) LLET, ((LKOD(I,J),I=0,3),J=1,LLET)
      CLOSE(7)
      WRITE(6,'("  Changes saved as instructed.")')
      GOTO 99
51    DO L = 1,LLET  ! Start editing
        K = 1; IF(LKOD(0,L)/100==1) K = 2  ! PL argument
54      WRITE(6,'(4X,"Your parameter settings for generating ",A,1X,
     +    "element",A,"coded  ",A,"  are now"/16X,"<Hi,Lo,Pr> = <",
     +    A4,2(1X,A4),">"/4X,"Hit RETURN if OK, or enter preferred ",
     +    "triple with Pr last."/)') CF(:JJF(LKOD(0,L)/100)), PL(K:),
     +    CHAR(MOD(LKOD(0,L),100)), (CLN(.001*LKOD(J,L),4,3),J=1,3)
        CALL SCANB(J,0,'R',5,WORD,100)   ! ***** Special return in WORD
        IF(J==0) CYCLE; IF(J<0) GOTO 32
        JJ = MIN(3,J)
        READ(2,*) (X(I),I=1,JJ)
        DO I = 1,JJ
          IF(X(I)/=NINT(X(I)) .OR. WORD(I:I)=='R')  CYCLE
57        IF(ABS(X(I))>1.) THEN; X(I) = .1*X(I); GOTO 57; END IF
        END DO
        IF(JJ==1) THEN
          LKOD(1,L) = NINT(1000*X(1)); LKOD(2,L) = LKOD(1,L)
        ELSE IF(JJ>1) THEN
          K = 1; IF(ABS(X(2))>ABS(X(1))) K = 2; M = NINT(SIGN(1.,X(K)))
          LKOD(1,L) = NINT(M*1000*MAX(ABS(X(1)),ABS(X(2))))
          LKOD(2,L) = NINT(M*1000*MIN(ABS(X(1)),ABS(X(2))))
        END IF
        IF(JJ==3) LKOD(3,L) = 1000*ABS(X(3)); IF(JJ<3) LKOD(3,L) = 0
        GOTO 54
      END DO                     !
      GOTO 45
C =============== End of letter specifications ==================
99    WRITE(6,'(" Hit RETURN to instantiate this schema in the data ",
     +  "you are creating."/" Otherwise, enter anything for return ",
     +  "to available alternatives.")')
      CALL SCN(J); Z = 1.
      IF(J/=0) THEN
        WRITE(6,'(" >>> Hit RETURN to confirm that you don''t want to",
     +    " use this schema at present,"/5X,"or enter anything to cor",
     +    "rect what may have been an entry error.")')
        CALL SCN(J); IF(J/=0) GOTO 99
        MSG = 1; RETURN
      END IF
C Start instantiation
      DO L = 1,LLET; Z = MIN(Z,ABS(.001*LKOD(2,L))); END DO  ! Z is smallest Lo
      WRITE(6,'(" This schema''s ""background"" consists of its nons",
     +  "alient (dot-marked) elements."/" If you want those to remai",
     +  "n strictly zero in this instantiation, hit RETURN."/" Other",
     +  "wise, enter a letter to fuzz your schema with background no",
     +  "ise.")')
      CALL SCN(J); IF(J==0) GOTO 120
110   WRITE(6,'(" This schema''s FUZZ parameter, which will randomize",
     +  " its background elements"/" within FUZZ of zero, is now",A5,
     +  ".  Hit RETURN if OK; otherwise, enter your"/" preferred FUZZ",
     +  ". (Usually this should be considerably less than your lowest"/
     +  " raw salience bound, which at present is",A5," here.)"/)')
     +  CLN(FUZ,5,3), CLN(Z,5,3)
      CALL SCANB(J,0,'R',5,CH1,0); IF(J==0) GOTO 114; IF(J<0) GOTO 110
      READ(2,*) FUZ; FUZ = MIN(ABS(FUZ),Z-.01)
112   IF(FUZ>=1.) THEN; FUZ = FUZ/10; GOTO 112; END IF
      GOTO 110
114   IF(FUZ>.9*Z .AND. KIND/=3) THEN
        WRITE(6,'(" Do you really want to blur the contrast between ",
     +    "salient and nonsalient"/" loadings here with FUZZ this la",
     +    "rge (",A5,")?  If so, enter anything to"/" confirm.  Othe",
     +    "rwise, hit RETURN to reconsider.")') CLN(FUZ,5,3)
        CALL SCN(J); IF(J==0) GOTO 110
      END IF
120   B=0.; IF(KIND==3) THEN; DO I=1,NF; B(I,I)=1.; END DO; END IF  ! Unities on COV diagonal
      IF(FUZ>=.005) THEN; S = FUZ/.5   ! Easiest to fuzz all, then overwrite with salients
        DO I = 1,NY; L = NF; IF(KIND==3) L = I-1
          DO J = 1,L
            B(I,J) = S*(RANDY()-.5)
          END DO
        END DO
        CUT = (Z + FUZ)/2 ! For picking out namestem salients
      END IF
      DO K = 1,LST(0)
        I = LST(K)/10000; J = MOD(LST(K)/100,100); L = 0
cccccc        B(I,NF+1) = B(I,NF+1) + 1  ! Count factors salient for item i
125     L = L+1
        IF(MOD(LKOD(0,L),100)/=MOD(LST(K),100).AND. L<LLET) GOTO 125
        HO = .001*LKOD(2,L); HI = .001*LKOD(1,L); KP = LKOD(3,L)
        Z = ABS(HO) + ABS(HI-HO)*RANDY(); IF(KP==0) Z = SIGN(Z,HI)
        IF( KP>0 .AND. RANDY()<.001*KP ) Z = -Z
        B(I,J) = Z
      END DO
      IF(KIND==3) THEN ! Return full matrix even if symmetric
        DO J = 2,NF; DO I = 1,J-1; B(I,J) = B(J,I); END DO; END DO
      END IF
      KF=15; CH4 ='WXYZ'; IF(KIND==3) THEN; KF=16; CH4='ZYXW'; END IF
      CLOSE(KF); OPEN(KF,FILE=CH4)
      IF(KIND==1) WRITE(KF,'(/" The raw pattern schema (",A,") was")')
     +  F1(:LAST(F1))
      IF(KIND==3) WRITE(KF,'(//" The raw Cff schema (",A,") was")')
     +  F1(:LAST(F1))
      CALL SHO(NY,NF,KIND,F1,LST,LKOD,LLET,FUZ,-3,KF)
      END SUBROUTINE
C
      SUBROUTINE SCORS(NS,NF,NV,NVF,A,B,U,MTW,ORT,TW,CV,NVFF) ! TW is TWIST
C This modification of ORTH2 first generates a random NS-by-NF+NV score matrix
C saved in scratchfile 10. Next, it orthonormalizes the 1st NF columns of the
C score file, partials them out of the remaining NV columns iff ORT = 1, and if
C so also orthogonalizes these uniqueness columns.  All columns are centered
C and standardized to unit variance. These principal-factor scores G (the 1st
C NF orth'ed columns) are then multiplied by the SS-factors' NFNF pattern B on
C prin-factors G to get SS-factor scores F, while G is pre-multiplied by the
C NVNF true-parts' (H) pattern A on the prin-factors to obtain H-scores. The
C unique-factor scores are then rescaled to get SDs equal to the unique SDs
C in U, and are added to the true-part scores to yield the observed-data scores
C which are saved in scratchfile 10.  A,B,U are provided as input
C           [ ORT<0 rescaling currently suspended ]
CC followed if ORT<0 by the SS-factor scores after a final rescaling to normalize
CC the observed-data variances if uniqueness orthogonalities are not exact (ORT0).

C Datascore covars [ maybe needing final norming ] are returned in CV.
C TW contains input/output twist params.  Score records -- datascores followed
C by SS-factor scores -- are stored in direct-access scratchfile 10.
C
      LOGICAL QN(NVF)                ! TW now dimensionalized as NVF-by-7: 1-3 for
      INTEGER ORT  ! Single integer  ! controls, 4-5 for skew/kurtosis of finished
      CHARACTER CF*12  ! , CLN*8      ! common/unique factors, 6-7 for raw generator axes.
      REAL A(NV,*),B(NV,*), CV(*), TW(NVF,7), U(*), DV(NVF,2), WK(NVFF),
     +     Q(NVF,NVF), RSIG(-50:50), VF(NF), VH(NV), VT(NVF), VU(NV)
      REAL(8) DWK(NVFF), SM(NVF,4)
      COMMON/RS/ RSIG
      COMMON/CF/ CF
      LO(I,J) = J*(J-1)/2 + I
      FL(Z,R) = SIGN((ABS(Z)+1.)**R-1.,Z)  ! Twist by power R
      OPEN(10,STATUS='SCRATCH',FORM='UNFORMATTED',ACCESS='DIRECT',
     +  RECL=4*(NVF+1))
      WK = 0.; SM = 0.D0; QN = .FALSE.
      IF(MTW==0) GOTO 10   !  R = .1(K+10) = 1.1, 1.2, ..., 6.0
C  If dists are twsted, get their approx. mean-sqs of scores on neg/pos sides
      DO J = 1,NVF
        R1 = TW(J,1); T1 = 10*MAX(R1,1./R1)-10.; J1=NINT(SIGN(1.,R1-1))
        R2 = TW(J,2); T2 = 10*MAX(R2,1./R2)-10.; J2=NINT(SIGN(1.,R2-1))
        K1 = INT(T1); F1 = T1-K1; K2 = INT(T2); F2 = T2-K2
        L1 = J1*MIN(K1+1,50); L2 = J2*MIN(K2+1,50); K1=J1*K1; K2=J2*K2
        DV(J,1) = RSIG(K1) + F1*(RSIG(L1)-RSIG(K1)) ! Lefthand norming divisor
        DV(J,2) = RSIG(K2) + F2*(RSIG(L2)-RSIG(K2)) ! Righthand norming divisor
        IF(NINT(TW(J,3))==1) QN = .TRUE.  ! Do central smoothing iff QN=t
      END DO    !    ^ TW(-,3) is set in subroutine TWST; TW is zeroed there

10    J = JJF(NS); KK = MAX(100,MIN(100000,10**(J-1)))  ! WRITE(6,'()')
      DO NN = 1,NS   ! Generate random scores with twist as stipulated
        DO J = 1,NVF
          S = ZRAND()
          IF(MTW==1) THEN  ! Larger R received on the right in TW with assigned P
            IF(S>0) S = FL(S,TW(J,1))/DV(J,1)
            IF(S<0) S = FL(S,TW(J,2))/DV(J,2) ! DV is free at end of loop
            IF(QN(J) .AND. 2*RANDY() < ABS(S)+1.) S = -S  ! Central smoothing
          END IF
          VT(J) = S; SM(J,1) = SM(J,1) + S; SM(J,2) = SM(J,2) + S*S
C               Out-of-Bounds in SM or VT
        END DO   ! Raw scores for record NN completed
        WRITE(10,REC=NN) (VT(J),J=1,NVF)
        IF(MOD(NN,KK)==0) WRITE(6,'(4X,"Generating crude score rec",
     +    "ord ",A)') CF(:JJF(NN))
      END DO     ! End of unprocessed score production when loop completed
C  Center and norm scores on all initial factors.
      DO J = 1,NVF
        SM(J,1) = SM(J,1)/NS
        SM(J,2) = SQRT(SM(J,2)/NS - SM(J,1)**2)
      END DO
      DO NN = 1,NS
        READ(10,REC=NN) (VT(J),J=1,NVF)
        DO J = 1,NVF
          VT(J) = (VT(J)-SM(J,1))/SM(J,2)   ! Z-scores on common and uniq facs
          DO I = 1,J  ! Accumulate raw products over all c/u factors
            DWK(LO(I,J)) = DWK(LO(I,J)) + VT(I)*VT(J)
          END DO
        END DO
        IF(NN==1) WRITE(6,'()')
        WRITE(10,REC=NN) (VT(J),J=1,NVF)
        IF(MOD(NN,KK)==0) WRITE(6,'(4X,"Standardizing scores in rec",
     +    "ord ",A)') CF(:JJF(NN))
      END DO
      DWK = DWK/NS   ! 2nd moments of all initial common/unique factor Z-scores

C==================== Old version ===============================
CCC ( Score matrices S in notes here are either items-by-records or records-by- )
CCC ( items in accord with context. Premultiplication by matrix is Item-by-Rec. )
CCC To orthnorm random-generator axes S* (Sf if ORT=0 or [Sf Se] if ORT=1), we
CCC start with triangular factor solution S* = QG where G is orthonormal and
CCC Q is triangular; then G = QS* where Q = Inv[Q].  (Subroutine INVF solves
CCC Cov(S*,S*) in WRK for Q.)  Next define Qff to be the leading NFxNF submatrix
CCC of Q (the whole Q if ORT=1); and compute A = PQff, B = P1Qff, where P,P1
CCC are the patterns of the items' common-parts H and SS factors F, respectively,
CCC on orthonormal axes that we can take for the common-parts' leading NF
CCC principal axes.  (P/P1 are stored in the head/tail of array P.)  Then from
CCC the normalized score matrix S = [Sf Se], we compute H = ASf and F = BSf.
CCC And if ORT=0, we have simply U = DuSe for the unique scores.  But getting
CCC U under ORT=1 is more troublesome: Start in this case with full orthnorming
CCC finished scores
CC
CCC                                                  
CCC                Qff   0    Sf         QffSf      
CCC       QS* =                =                      (Qff and Qee
CCC                Qfe  Qee   Se     QfeSf + QeeSe    lower triangular)
CCC                                                  
CC
CCC Then U = Du(QfeSf + QeeSe) = (DuQfe)Sf + (DuQee)Se = DuQfeeeS
CCC where Qfeee = Qfe  Qee and S = Sf'  Se''.  (Score matrices in the last
CCC few documentation lines are all items-by-records.)
CC
CC      NN = NF; IF(ORT==1) NN = NVF  ! Raw unique factors excluded if ORT < 1
CC      CALL INVF(WK,NN)    ! Returned up-tri WK is transpose of Q
CC      A = 0.; B = 0
CC      DO I = 1,NV; DO J = 1,NF; DO K = J,NF
CCC             k P(i,k)Qff(k,j) [kj]  =  k P(i,k)WRK(LO(j,k)); kj
CC            A(I,J) = A(I,J) + P(I,K)*WRK(LO(J,K))
CC      END DO; END DO; END DO
CC      DO I = 1,NF; DO J = 1,NF; DO K = J,NF
CCC             k P1(i,k)Qff(k,j) [kj]  =  k P1(i,k)WRK(LO(j,k)); kj
CC            B(I,J) = B(I,J) + W(I,K)*WRK(LO(J,K))
CC      END DO; END DO; END DO

C==================== New version ===============================
C Partition raw random axes as S = [Sf Se].  If ORT = 1, we want NVFxNVF
C rotation matrix Qfe such that QfeS is orthonormal.  Otherwise, we want just
C NFxNF rotation matrix Qf such that QfSf is orthonormal.
      NZ = NVF; IF(ORT<1) NZ = NF
      NZZ = LO(NZ,NZ); WK = SNGL(DWK)
      IF(ORT<1) THEN
        CALL EIGS(NZ,NZ,WK,NZZ,VT,Q,NVF,1,IER,6)
        DO J = 1,NZ; S = SQRT(VT(J)); DO I = 1,NZ
            Q(I,J) = Q(I,J)/S
        END DO; END DO       ! EIGS not set for DP-input
      ELSE; Q = 0.           ! Want orthnormed X = WZ where WCzzW' = I
        CALL INVF(DWK,NZ)    ! INVF returns W for lo-tri W doing WCzzW'=I, so
        DO J = 1,NZ; DO I = 1,J  ! DWK comprises lower triangle of up-zero Q.
            Q(I,J) = SNGL(DWK(LO(I,J)))  ! But orthogonalization below is
        END DO; END DO                   ! computed Q'CzzQ=I, so put WK
      END IF                             ! in upper-triangle of Q.

      DO I = 1,NVF; DO J = 4,7; TW(I,J) = 0.; END DO; END DO ! TW(i,4-7) collects moments 3,4
C To orthogonalize Czz, do Q'CzzQ (note transposition of Q !!)
      DWK = 0.; SM = 0.   ! Collect data covs CV in real*8 DWK
      WRITE(6,'()')
      DO NN = 1,NS        ! <<<<<<< Start of file-scores loop
        DV = 0.; READ(10,REC=NN) (DV(I,2),I=1,NVF)   ! S <= QS;  Q is Qfe if ORT=1 or is Qf otherwise
        DO I = 1,NVF; DO J = 3,4
          TW(I,3+J) = TW(I,3+J) + DV(I,2)**J  ! Cols 6,7 of TW
        END DO; END DO  ! ^ Get skew and kurtosis of generator axes
        DO I = 1,NZ; DO J = 1,NZ
            DV(I,1) = DV(I,1) + Q(J,I)*DV(J,2)  ! Orthnorm G-factors and, if
        END DO; END DO                          ! ORT=1, also u-variables
        IF(ORT<1) THEN
          DO I = NF+1,NVF; DV(I,1) = DV(I,2); END DO
        END IF
C Multiplying by A maps scores on the 1st NF orthogonalized G-axes into
C common-part scores H
        VH = 0.; VF = 0.; VU = 0.
        DO I = 1,NV  ! Range over NV data variables, not unique factors
          DO J = 1,NF
            VH(I) = VH(I) + A(I,J)*DV(J,1)
          END DO          ! ^ Only use of A
        END DO
C Multiplying by B maps scores on the 1st NF orthogonalized G-axes into
C SS-factor scores F
        DO I = 1,NF
          DO J = 1,NF
            VF(I) = VF(I) + B(I,J)*DV(J,1)
          END DO          ! ^ Only use of B
        END DO
C  Compute this record's U-scores
        DO J = 1,NV  ! Reminder: U contains SDs, not variances
          VU(J) = U(J)*DV(NF+J,1)  ! Can do this on VT = ... line
        END DO
C Finish data-scores (Y = H+E), accumulate data moments, and file
        DO J = 1,NV
          VT(J) = VH(J) + VU(J)  ! VT now holds data scores
          DO I = 1,J
            DWK(LO(I,J)) = DWK(LO(I,J)) + VT(I)*VT(J)
          END DO
        END DO
        WRITE(10,REC=NN) (VT(I),I=1,NV), (VF(I),I=1,NF)
        IF(MOD(NN,KK)==0) WRITE(6,'(4X,"Buffering finished scores ",
     +    "for record ",A)') CF(:JJF(NN))

C  Check orthogonalities in case ORT=1; finish standardizing in case ORT=-1
        DO I = 1,NVF                            ! Not currently operative ^
          IF(I<=NF) S = VF(I); IF(I>NF) S = VU(I-NF)
          SS = S*S
          SM(I,1) = SM(I,1) + S   ! Don't seem to use this at present
          SM(I,2) = SM(I,2) + SS
          TW(I,4) = TW(I,4) + S*SS
          TW(I,5) = TW(I,5) + SS*SS
        END DO
      END DO   ! <<<<<<< End of file-scores loop

CC TEST LINES: Independent computation of unrotated moments
C      TST = TST/NS     !
C      WRITE(21,'(" RAW MOMENTS:")')
C      DO J = 1,4
C        WRITE(21,'(I2,":",8F5.2,2X,30F5.2)') J, (TST(I,J),I=1,NVF)
C      END DO

C  Raw skew/kurtosis for SS-factors and U-residuals accumulated in TW(i:4-5)
C  Raw skew/kurtosis for corresponding generator axes accumulated in TW(i:6-7)
ccc ******* Neither the SS-factors nor U-resids have unit vars, so
ccc ******* mean powered scores need division by powered SD
      DO I = 1,NVF
C            SM(I,1) = SM(I,1)/NS  ! TEST LINE
        S2 = SM(I,2)/NS  !; S1 = SM(I,1)/NS  ! S1 shd be all zero; but can check
        S3 = S2*SQRT(S2); S4 = S2*S2         !   ^ YES !!!
        TW(I,4) = (TW(I,4)/NS)/S3; TW(I,5) = (TW(I,5)/NS)/S4 ! sk/kt for finished factors
        TW(I,6) = TW(I,6)/NS; TW(I,7) = TW(I,7)/NS  ! skew/kurtosis for raw generator axes
      END DO

CC TEST LINES  Check JUNK match with *.SEE output from MORSCOR
C      WRITE(21,'(/" Final 1st moment:"/8F5.2,2X,30F5.2)') (SNGL(SM(I,
C     +  1)),I=1,NVF)
C      WRITE(21,'(/" Final 2nd moment:"/8F5.2,2X,30F5.2)') (SNGL(SM(I,
C     +  2))/NS,I=1,NVF)
C      WRITE(21,'(/" Raw Skew:"/8A5,2X,30A5)')(CLN(TW(I,6),5,2),I=1,NVF)
C      WRITE(21,'(/" Raw Kurtosis:"/8A5,2X,30A5)') (CLN(TW(I,7),5,2),
C     +  I=1,NVF)

CC      Do ORT = -1 final variance norming in Main [ currently suspended ]
      DO I = 1,LO(NV,NV); CV(I) = SNGL(DWK(I)/NS); END DO
      END           ! Data means are already zero (or should be)

      SUBROUTINE SCN(NL)
C This is a baby SCAN that responds to keyboard entry KB by returning NL = 0
C if KB is RETURN, NL = 1 if KB is a number, and NL = -1 KB is a letter.
      CHARACTER CH
      READ(6,'(A)') CH; N = ICHAR(CH)
      NL = 0; IF(N<=32) RETURN
      IF(N<58) NL = 1; IF(N>64) NL = -1
      END SUBROUTINE
C
      SUBROUTINE SCANB(NL,NS,SEQ,KFILE,CH,KK)
C     Copyright (c) 1989 by W. W. Rozeboom.   All rights reserved.
C This reads a string in I/O unit KFILE, cleans it for list-directed reading
C of the numbers therein, and checks whether it contains NS integers/reals in
C the sequence of Is and Rs received in SEQ if NS>0, or, if NS<1, whether
C all its numbers are of the first I/R kind listed in SEQ. (Integers are
C accepted also as reals.)  NL returns 0 if the input string is blank,
C -1 if this contains only non-numeric characters, -2 if the cleaned number
C string returned in File 2 is non-null but does not match SEQ, and gives
C the total count of numbers in the returned string otherwise.
C ***** If SEQ is "B", NL returns 0 if the input line is blank, and
C       returns -1 otherwise.
C ***** In SCANB, CH returns the KFILE string's first ABS(KK)-term sequence
C       while removing this from the string scanned for numbers if KK < 0.
C         In this special modification, KK > 99 returns in CH the I/R sequence
C       No continuation lines are allowed.                 !!  of input numbers
      CHARACTER AA, SEQ*(*), WA*240, WB*240, CH*(*)
      READ(KFILE,'(A)',END=999) WA
      NL = LAST(WA)
      IF(NL==0) RETURN
      IF(SEQ(1:1)=='B') NL = -1
      IF(SEQ(1:1)=='B') RETURN
      WB(NL+1:NL+1) = ' '
      IF(KK==0) GOTO 15
cc      CH = ' ' ! Blanking CH probably pointless
      K = IABS(KK); IF(K<100) CH = WA(:K)
      IF(KK>0) GOTO 15    ! Positive K > 99 is free to carry another signal
      DO I = 1,K; WA(I:I) = ' '; END DO
15    DO I = 1,NL
        WB(I:I) = ' '
        IF(WA(I:I)<CHAR(48) .OR. WA(I:I)>CHAR(57)) CYCLE
        WB(I:I) = WA(I:I)
        WA(I:I) = '0'
      END DO
      IP = 0
      IF(WA(:3)=='-.0') WB(:1) = '-'
      IF(WA(:2)=='-0') WB(:1) = '-'
      IF(WA(:2)=='.0') WB(:1) = '.'
      IF(WA(:2)=='.0') IP = 1
      DO I = 2,NL
        IF(WA(I:I)=='-') THEN
          IF(WB(I-1:I-1)==' ' .AND. (WA(I+1:I+1)=='.'.OR.WA(I+1:I+1)
     +      =='0')) WB(I:I) = '-'
          IP = 0
        ELSE IF (WA(I:I)=='.') THEN
          IF((WA(I-1:I-1)=='0'.OR.WA(I+1:I+1)=='0') .AND. IP==0)
     +      WB(I:I) = '.'
          IF(WB(I:I) == '.') IP = 1
        ELSE IF (WA(I:I) /= '0') THEN
          IP = 0
        END IF
      END DO
C Identify Integer/Real sequence in WB and put in start of WA
      AA = 'I'
      IF(WB(1:1)=='.') AA = 'R'
      NN = 0
      DO I = 2,NL+1
        IF(WB(I:I)==' ' .AND. WB(I-1:I-1)/=' ') THEN
          NN = NN + 1
          WA(NN:NN) = AA
          AA = 'I'
        ELSE IF (WB(I:I)=='.') THEN
          AA = 'R'
        END IF
      END DO
      IF(NN==0) NL = -1
      IF(NN==0) RETURN

      IF(KK>99) CH(:NN) = WA(:NN)  !  Return list of I,R kinds

      AA = '+'
      IF(NS<=0 .AND. SEQ(1:1)=='R') GOTO 60
      IF(NS<=0) GOTO 50
      IF(NN<NS) GOTO 57
      DO I = 1,NS
        IF(SEQ(I:I)=='I' .AND. WA(I:I)/='I') AA = '0'
      END DO
      GOTO 55
50    DO I = 1,NN
        IF(WA(I:I)/=SEQ(1:1)) AA = '0'
      END DO
55    IF(AA=='+') GOTO 60
57    NL = -2
      IF(NL<=-2) WRITE(6,'(/" Your input does not contain the ",
     +  "number sequence requested. Try again.")')
      RETURN
60    REWIND 2
      WRITE(2,'(A)') WB(1:NL)
      NL = NN
      REWIND 2
      RETURN
999   NL = 999
      RETURN
      END
C
      SUBROUTINE SCHEM(NV,NF,JV,JF,CUT,A,LDA)
C Display schematized factor pattern in A with blanks for loadings smaller
C than CUT and others rounded to nearest lst decimal.
      CHARACTER*3 LN(:), FMT*40, C2*2, C1*1, CF*12
      ALLOCATABLE :: LN
      REAL A(LDA,*)
      COMMON /CF/ CF
      ALLOCATE ( LN(NF) )
5     IF(CUT>1.5) CUT = CUT/10
      IF(CUT>1.5) GOTO 5
      C2 = CF(:JJF(JF)); C1 = CF(:JJF(75/(2+3*JF)))
      FMT = '(I4,".",'//C1//'(2X,'//C2//'A),5(:/6X,'//C1//'(2X,'//
     +       C2//'A)))'
      DO I = 1,NV
        IF(MOD(I-1,JV)==0 .AND. I<NV) WRITE(6,'()')
        DO J = 1,NF
          LN(J) = '  .'
          X = ABS(A(I,J)); IF(X<=CUT) CYCLE
          IF(A(I,J)<.01) LN(J)(2:2) = '-'
          LN(J)(3:3) = CHAR(48+NINT(MIN(9.,10*X)))
        END DO
        WRITE(6,FMT) I, (LN(J), J=1,NF)
        IF(((MOD(I,10)==0.AND.NV-I>5).OR.I==NV)) CALL WAIT(1)
      END DO
      END SUBROUTINE
C
      SUBROUTINE SHO(NY,NF,KIND,F1,LST,LKOD,LLET,FUZ,JOB,KF)
C JOB<=1 shows matrix schema; JOB=2 shows its letter specs; JOB > 2 shows both
C Only negative SHO job in Gendata package so far is -3.
      CHARACTER*4  C0,C1,C2,C3,C4, CH3*3, CLN*8, FMT*50
      CHARACTER*12  CF, F1, WORD*80, WRD(25)*3, CC*2
      INTEGER GRID(NY,NF), LST(0:NY*NF), LKOD(0:3,26)
      COMMON /CF/ CF
C ========= Start of schema display ==========
      MM = 24; IF(KF/=6) MM = 50
      IF(ABS(JOB)==2) GOTO 11
      LS = 3; WORD(:3) = 'Cff'
      IF(F1(:1)=='K') WORD(:3) = 'Cuu'
      IF(KIND==2) THEN; WORD(:18) = 'Pattern'; LS = 7; END IF
cc      IF(KIND==4) WORD(:18) = 'Recursive pattern'; LS = 17 ! Not GENSCOR input
      CC = '24'; IF(KF/=6) CC = '50'   ! "//CC//"
      NL = LST(0); IE = 0; GRID = 249    ! Show grid zeros by 
      DO K = 1,NL  ! Unpack grid saved in LST
        I = LST(K)/10000; J = MOD(LST(K)/100,100)
        GRID(I,J) = MOD(LST(K),100); IF(KIND/=2) CYCLE
        IF(I<=J) IE = IE+1  ! IE>0 flags not subdiagonal
      END DO
      IF(KF==6) WRITE(KF,'(4X,A," schema ",A," is")') WORD(:LS),
     +  F1(:LAST(F1))
CCC      IF(KF/=6) WRITE(KF,'(/4X,A," by ",A," raw pattern schema ",A,
CCC     +  " is")') CF(:JJF(NY)), CF(:JJF(NF)), F1(:)LAST(F1)  ! Already written in SKEME
      WRITE(KF,'(4X,'//CC//'I3,3(:/6X,'//CC//'I3))') (I,I=1,NF)  ! Indices on top line
      CH3 = 'Ŀ '; IF(NF>MM) CH3 = '   '  ! IF(NF>MM.OR.KIND==3) CH3 = '   '
      WRD=' '; N1 = NF+1
      IF(KIND==3 .AND. NF<=24) THEN
        WRD = ' '; N1 = NF+1; WRD(NF:NF) = 'sym'; WRD(N1:N1) = '  '
      END IF
      WRITE(KF,'(3X,"",'//CC//'A3,2A3)') ('',I=1,MIN(NF,MM)), CH3
      CH3 = '  '; IF(KIND==3) CH3 = '  1'  ! Rem: MaxEntries^ in line
      L = 1+(NF-1)/MM; LSHN = 3; LSHB = 0       ! L is lines per table row
      FMT = '(I3,"",'//CC//'A3,2A3)'
      IF(NF>MM) FMT(13:) = '/3(:3X,"",'//CC//'A3))'
      DO K = 1,NY; JJ = NF; IF(KIND==3) JJ = K-1
        IF(NF<=MM) WRITE(KF,FMT) K, ('  '//CHAR(GRID(K,I)),I=1,JJ),
     +     CH3, WRD(JJ+1:N1)
        IF(NF>MM) WRITE(KF,FMT) K, ('  '//CHAR(GRID(K,I)),I=1,JJ) !, CH3
        IF(K==1 .AND. KIND==3) WRD(:NF) = '   '
        IF(KF/=6) CYCLE; LSHN = LSHN+L
        IF(LSHN-LSHB>20.AND.(NY-K)*L>4) THEN
          CALL WAIT(0); LSHB = LSHN
        END IF
      END DO
      WRITE(KF,'(3X,"",'//CC//'A3,A3)') ('',I=1,MIN(NF,MM))," "
      IF(ABS(JOB)<2) RETURN
C ==================== End of schema display ======================
11    IF(JOB>1) WRITE(KF,'(3X,A,"''s current parameter assignments ",
     +  "are")') F1(:LAST(F1))
      IF(JOB<0) WRITE(KF,'(3X,"with parameter specifications")')
      MM = 17; IF(KF/=6) MM = LLET
      C1 = 'Ŀ  '; C2 = '   '; C3 = 'Ĵ  '; C4 = '  '; C0 = '   '
      IF(LLET>MM) THEN; C1 = ' '; C2 = ' '; C3 = ' '; C4 = ' '; END IF
      WRITE(KF,'(6X,"",20A)') ((""),I=1,MIN(LLET,MM)), C1
      WRITE(KF,'(6X,"",40A)') ('   ',CHAR(MOD(LKOD(0,I),100)),I=1,
     +  MIN(LLET,MM)), C2
      WRITE(KF,'(" ",60A)') ((""),I=1,MIN(LLET,MM)), C3
      WRITE(KF,'("  Hi ",60(1X,A3))') (CLN(.001*LKOD(1,I),3,2),I=1,
     +  MIN(LLET,MM)), C0
      WRITE(KF,'("  Lo ",60(1X,A3))') (CLN(.001*LKOD(2,I),3,2),I=1,
     +  MIN(LLET,MM)), C0
      WRITE(KF,'("  Pr ",60(1X,A3))') (CLN(.001*LKOD(3,I),3,2),I=1,
     +  MIN(LLET,MM)), C0
      WRITE(KF,'(" ",60A)') ((""),I=1,MIN(LLET,MM)), C4
      IF(LLET>MM) THEN; K = 34-LLET    ! K + LLET-17 = 17
        C1 = ''; C2 = ''; C3 = ''; C4 = ''
        WRITE(KF,'(7X,20A)') ('    ',I=1,34-LLET), ('',I=18,LLET),C1
        WRITE(KF,'(7X,20A4)') ('    ',I=1,34-LLET),  ('   ',CHAR(MOD(
     +    LKOD(0,I),100)),I=18,LLET), C2
        WRITE(KF,'(7X,20A)') ('    ',I=1,34-LLET), ('',I=18,LLET),C3
        WRITE(KF,'(7X,20(1X,A3))') ('    ',I=1,34-LLET),(CLN(.001*LKOD(
     +    1,I),3,2),I=18,LLET), C0
        WRITE(KF,'(7X,20(1X,A3))') ('    ',I=1,34-LLET),(CLN(.001*LKOD(
     +    2,I),3,2),I=18,LLET), C0
        WRITE(KF,'(7X,20(1X,A3))') ('    ',I=1,34-LLET),(CLN(.001*LKOD(
     +    3,I),3,2),I=18,LLET), C0
        WRITE(KF,'(7X,20A)') ('    ',I=1,34-LLET), ('',I=18,LLET),C4
      END IF
      IF(KF/=6 .AND. FUZ>.005) WRITE(KF,'(3X,"and level ",A3," ba",
     +  "ckground fuzz.")') CLN(FUZ,3,2)
      END SUBROUTINE
C
      SUBROUTINE SHOWP(NF,JF,B,LDB,KF)
C Write frame-parameter specifications to file KF
      CHARACTER FMT*(61), CLN*8, CH1*1
      REAL B(LDB,*)
      NFB = NF/JF
      CH1 = CHAR(48+NFB)
      FMT = '(4X,"",'//CH1//'(""),"")'
      WRITE(KF,FMT)
      FMT = '(4X,"    Factors  :",'//CH1//'(I3,"",I2,1X),"")'
      WRITE(KF,FMT) ((J-1)*JF+1,J*JF,J=1,NFB)
      FMT = '(4X,"",'//CH1//'(""),"")'
      WRITE(KF,FMT)
      FMT = '(4X," Hi Salient  ",'//CH1//'(A5,2X),"")'
      WRITE(KF,FMT) (CLN(B(1,J),5,-2),J=1,NFB)
      FMT = '(4X," Lo Salient  ",'//CH1//'(A5,2X),"")'
      WRITE(KF,FMT) (CLN(B(2,J),5,-2),J=1,NFB)
      FMT = '(4X," PrNeg Sal.  ",'//CH1//'(A5,2X),"")'
      WRITE(KF,FMT) (CLN(B(3,J),5,-2),J=1,NFB)
      FMT = '(4X," Hi nonSal.  ",'//CH1//'(A5,2X),"")'
      WRITE(KF,FMT) (CLN(B(4,J),5,-2),J=1,NFB)
      FMT = '(4X,"",'//CH1//'(""),"")'
      WRITE(KF,FMT)
      END SUBROUTINE
C
      SUBROUTINE START(J,F1)
C This opens formatted file F1 with unit-number J, and finds its first line
C beginning with a number possibly real or negative.
      CHARACTER F1*12, CH*80
      OPEN(J,FILE=F1)
10    READ(J,'(A)',END=50) CH
      K = 0
12    K = K+1
      L = ICHAR(CH(K:K))
      IF((L==32.OR.L==0.OR.L==45.OR.L==46).AND.K<80) GOTO 12
      IF(L<48 .OR. L>57) GOTO 10
      BACKSPACE J
      RETURN
50    WRITE(6,'(/" File ",A," is defective.")') F1
      STOP
      END
C
      SUBROUTINE TWST(NF,NV,TW,TP,QT)   !  TP will be written to output
C Set TWIST parameters  for common-factor and unique-factor score production
C TW is local name for array created under name TWIST.
      LOGICAL QT
      CHARACTER CH6*6, CLN*8, CH, WORD*54
      REAL TW(NF+NV,7), TP(2,4), TMP(3)  ! TW now has 7 cols
      INQUIRE(FILE='OLDTWST',EXIST=QT)
      TP=1.; TP(1,3)=.5; TP(2,3)=.5; P=.5; RA = 1.; RB = 1.
      IF(QT) THEN
         OPEN(11,FILE='OLDTWST')
         READ(11,*) ((TP(I,J),J=1,4),I=1,2); CLOSE(11); QT = .FALSE.
      END IF

C NonNormal random-score distributions here are shaped by parameters RA/RB/P.
C Each raw score S derives from the output Z of a Normal random generator by
C giving S the sign of Z while the size (magnitude) of S determines the size
C of S under a transformation of form
C
C                 S = (Z) =def (Z+1)^r - 1
C
C wherein parameter r is a positive real exponent.  With r fixed, function  is
C a monotone increasing function on non-negative real numbers that is concave
C (positively accelerated) when r>1, an Identity transform ((x) = x) when r=1,
C and convex (negatively accelerated) when r<1.  Accordingly, the distribution
C of scores generated under fixed r is Normal if r=1, and becomes increasingly
C leptokurtic (long slender tails, kurtosis > 3), or platykurtic (stubby tails,
C kurtosis < 3), with r taken increasingly larger, or smaller, than 1.
C
C Further, GENSCOR allows a created distribution's initial Zs to be shaped by 
C under a different r when Z is positive than when Z is negative.  Difference
C in r between the two distribution sides creates skew which can be quite large
C if one side is leptokurtic while the other is platykurtic.
C
C   Production of nonNormal scores is controlled by your choice of RA/RB/P
C   separately for the two groups of factors, common and unique.  For each
C   factor f in each group, exponent r is set at a value r1 for one side
C   of f's distribution and at r2 for the other. (r1 = r2 is symmetric.)
C   RA and RB are allowed to vary from leptokurtic 6.0 through Normal 1.0
C   down to platykurtic .167 (1/6) in magnitude; however, you may also enter
C   RA and/or RB with negative sign to randomize as follows:
C
C       Setting                 Action in factor group
C 1. pos RA, pos RB    Fix r1 = RA and r2 = RB (symmetric if RA = R2)
C 2. pos RA, neg RB    Fix r1 = RA, randomize r2 between RA and RB (asymmetric)
C 3. neg RA, pos RB    Randomize r1 = r2 between RA and RB (symmetric)
C 4. neg RA, neg RB    Randomize both r1 and r2 between RA and RB
C
C  Finally you can choose P between 0 and 1 to be the probability that skewed
C  distributions in this group are skewed positively.  (This option has no
C  effect on symmetric distributions.)  And you will be allowed to waive
C  "central smoothing", which is gradedly random sign reversal of scores
C  near the median to mitigate density discontinuities at the median created
C  by appreciable difference between r1 and r2.

10    WRITE(6,'(" NonNormal random-score distributions here are shape",
     +  "d by parameters RA/RB/P."/" Each raw score S derives from th",
     +  "e output Z of a Normal random generator by"/" giving S the s",
     +  "ign of Z while the size (magnitude) of S determines the size"/
     +  " of S under a transformation of form"//18X,"S = (Z) =",
     +  "def (Z+1)^r - 1"//" wherein parameter r is a positive real",
     +  " exponent.  With r fixed, function  is"/" a monotone incre",
     +  "asing function on non-negative real numbers that is concave"/
     +  " (positively accelerated) when r>1, an Identity transform (",
     +  "(x) = x) when r=1,"/" and convex (negatively accelerated) wh",
     +  "en r<1.  Accordingly, the distribution"/" of scores generate",
     +  "d under fixed r is Normal if r=1, and becomes increasingly"/
     +  " leptokurtic (long slender tails, kurtosis > 3), or platykur",
     +  "tic (stubby tails,"/" kurtosis < 3), with r taken increasing",
     +  "ly larger, or smaller, than 1."//" Further, GENSCOR allows a",
     +  " created distribution''s initial Zs to be shaped by "/" unde",
     +  "r a different r when Z is positive than when Z is negative.  ",
     +  "Difference"/" in r between the two distribution sides creates",
     +  " skew which can be quite large"/" if one side is leptokurtic ",
     +  "while the other is platykurtic."//" Hit RETURN to continue")')
      READ(5,'()')
      WRITE(6,'("   Production of nonNormal scores is controlled by yo",
     +  "ur choice of RA/RB/P"/"   separately for the two groups of fa",
     +  "ctors, common and unique.  For each"/"   factor f in each gro",
     +  "up, exponent r is set at a value r1 for one side"/"   of f''s",
     + " distribution and at r2 for the other. (r1 = r2 is symmetric.)"/
     +  "   RA and RB are allowed to vary from leptokurtic 6.0 through ",
     +  "Normal 1.0"/"   down to platykurtic .167 (1/6) in magnitude; ",
     +  "however, you may also enter"/"   RA and/or RB with negative ",
     +  "sign to randomize as follows:"//7X,"Setting",17X,"Action in ",
     +  "factor group"/" 1. pos RA, pos RB    Fix r1 = RA and r2 = RB",
     +  " (symmetric if RA = R2)"/" 2. pos RA, neg RB    Fix r1 = RA,",
     +  " randomize r2 between RA and RB (asymmetric)"/" 3. neg RA,",
     +  " pos RB    Randomize r1 = r2 between RA and RB (symmetric)"/
     +  " 4. neg RA, neg RB    Randomize both r1 and r2 between RA",
     +  " and RB"/)')
      WRITE(6,'(3X,"Finally you can choose P between 0 and 1 to be th",
     +  "e probability that skewed"/3X,"distributions in this group ",
     +  "are skewed positively.  (This option has no"/3X,"effect on ",
     +  "symmetric distributions.)  And you will be allowed to waive"/
     +  3X,"""central smoothing"", which is gradedly random sign reve",
     +  "rsal of scores"/3X,"near the median to mitigate density disc",
     +  "ontinuities at the median created"/3X,"by appreciable differ",
     +  "ence between r1 and r2.")')
      WRITE(6,'(/" Hit RETURN to continue, or enter anything to leave",
     +  " your distributions Normal ")')
      READ(5,'(A)') CH; IF(CH/=' ') RETURN
      TOP = 6.0; BOT = 1./TOP
      DO K = 1,2
        IF(K==1) CH6 = 'COMMON'; IF(K==2) CH6 = 'UNIQUE'
        WRITE(6,'(/1X,A," TWIST ",52(""))') CH6
cc20      WORD(20:) = ' '  ! Shouldn't need this
20      RA = TP(K,1); RB = TP(K,2)
        IF(RA>0 .AND. RB>0) WORD(:19) = 'r1 = RA and r2 = RB'
        IF(RA>0 .AND. RB<0) WORD = 'r1 = RA, randomize r2 between '//
     +    'RA and RB (asymmetric)'
        IF(RA<0 .AND. RB>0) WORD(:49) = 'randomize r1 = r2 between'//
     +    ' RA and RB (symmetric)'
        IF(RA<0 .AND. RB<0) WORD(:52) = 'separately randomize r1 '//
     +    'and r2 between RA and RB'
        WRITE(6,'(" Twist parameters for generating ",A,"-factor sc",
     +    "ores are now"/5X,"RA     RB     P:     For each ",A," fa",
     +    "ctor:"/2(2X,A5),2X,A5,6X,A)') CH6,CH6, (CLN(TP(K,J),5,-2),
     +    J=1,3),WORD
        IF(RA==1. .AND. RB==1.) WRITE(6,'(" NOTE: These settings yie",
     +    "ld Normal distributions.")')
        WRITE(6,'(" Hit RETURN to approve, or enter new 3-tuple <RA,",
     +    "RB,P> taking care to get"/" its signs and order correct. ",
     +    " (Omitting P will leave P unchanged.)"/" To repeat the ex",
     +    "planation of these parameters, enter any letter."/)')
        CALL SCANB(J,0,'R',6,CH,0)
        IF(J==0) GOTO 30
        IF(J<0) GOTO 10
        READ(2,*) (TMP(I),I=1,J)
        TP(K,1) = SIGN(MAX(BOT,MIN(ABS(TMP(1)),TOP)),TMP(1))
        IF(J>1) TP(K,2) = SIGN(MAX(BOT,MIN(ABS(TMP(2)),TOP)),TMP(2))
        IF(J>2) THEN
          P = MAX(0.,TMP(3))
25        IF(P>1.) P = .1*P; IF(P>1.) GOTO 25
          TP(K,3) = P
        END IF
        GOTO 20
30      IF(ABS(ABS(TP(K,1))-ABS(TP(K,2)))>.1) THEN
          IF(NINT(TP(K,4))==1) WRITE(6,'(" To approve central smoothin",
     +      "g of asymmetric ",A,"-factor distributions,"/" hit RETURN",
     +      ".  Otherwise, enter anything to disable this nicety.")')CH6
          IF(NINT(TP(K,4))/=1) WRITE(6,'(" To leave asymmetric ",A6,
     +      "-factor densities discontinuous at the median,"/" hit RE",
     +      "TURN.  Otherwise, enter anything to enable central smoot",
     +      "hing.")') CH6
          CALL SCN(J)
          IF(J/=0) TP(K,4) = 1.- TP(K,4)
          IF(J/=0) GOTO 30
        END IF
      END DO
      OPEN(11,FILE='OLDTWST')
      WRITE(11,'(8A6)') ((CLN(TP(I,J),6,-2),J=1,4),I=1,2)
      CLOSE(11); NQ = 0  ! NQ checks for null twist
      DO K  = 1,2  ! Derive TW for from TP controls   !$$$$$$$$$$$$$$
        J = 1; IF(TP(K,2)<TP(K,1)) J = 2
        RA = TP(K,J); RB = TP(K,3-J)  ! Provisionally put smaller r on left
        FA=SIGN(1.,RA); RA=ABS(RA); FB=SIGN(1.,RB); RB=ABS(RB)
        DD = RB-RA; NQ = NQ + ABS(ABS(RA)-1)+ABS(ABS(RB)-1)
        DO J = 1+(K-1)*NF,NF+(K-1)*NV  ! K=1: J=1,NF; K=2: J=NF+1,NF+NV
          PA = RA; PB = RB
          IF(FA<0.) PA = PA+DD*RANDY(); IF(FB<0.) PB = PB-DD*RANDY()
          IF(RANDY()>TP(K,3)) THEN  ! Smaller r on left with probability P
            X = PA; PA = PB; PB = X
          END IF
          TW(J,1) = PA; TW(J,2) = PB; TW(J,3) = TP(K,4)
        END DO
      END DO
      QT=.TRUE.; IF(NQ<.01) QT=.NOT.QT
      RETURN
      END
C
      FUNCTION ZRAND()
C This returns Normally distributed random Z-scores. [From Press et al, p. 203,
C whose theory is obscure at one crucial point.]
      DATA IFLG/0/
      SAVE G
      IF(IFLG==0) THEN  ! No extra deviate is in store
10      V1 = 2.*RANDY() - 1.   ! Put uniform random number in interval [-1,1]
        V2 = 2.*RANDY() - 1.   ! Do the same for V2
        R = V1**2 + V2**2
        IF(R>=1. .OR. R==0.) GOTO 10 ! If <V1,V2> not in unit circle, try again
        F = SQRT(-2.*LOG(R)/R)
        G = V1*F      ! Save extra random deviate for next time
        ZRAND = V2*F
        IFLG = 1
      ELSE
        ZRAND = G
        IFLG = 0
      END IF
      RETURN
      END
C
      FUNCTION RANDY()
C Use the compiler's random generator for a random number in the unit interval
      DATA NEW/1/
      IF(NEW>0) THEN; NEW = 0; CALL RANDOM_SEED(); END IF
      CALL RANDOM_NUMBER(RANDY)
      END FUNCTION

      SUBROUTINE TWEAK(NF,NFF,CFF,R,ZLO,ZLB,ZZM,CC,IER,JOB)
C Tweak input COV-matrix CFF to have declared low eigenvalue ZLO.  JOB = 0
C suppresses interaction with user. CC is 'Cff' or 'Cuu'.  ZLB is the original
C received R(NF); ZLO is last low eig returned from TWEAK; ZZM is the final
C tweak multiplier from initial R(NF).  The tweaked eivals are returned in R.
c   Returns other than adjusted LO :
c       a) Eigensolution fails:            IER > 0
c       b) No change; Input is orthogonal: IER = -1
c       c) No change; Tweak is obviated:   IER = -2
      CHARACTER CF*12, CC*3, CLN*8, C8*8, CH
      REAL CFF(NFF), R(*)
      LO(I,J) = J*(J-1)/2 + I
      COMMON /CF/ CF        ! ZLO gets stipulated low eig; ZLB is original received low
      SAVE ZMB      ! ZMB is the preceding ZZM
      CALL EIGS(NF,NF,CFF,NFF,R,X,NF,0,IER,6)   ! Eigvecs not needed
      IF(IER>0) THEN
        WRITE(6,'(" Eigenanalysis of ",A," fails; Tweak is abort",
     +    "ed.")') CC; RETURN
      END IF  ! ZLB is negative at start of work on new R
      ZB = R(NF); IF(ZLB<0) THEN; ZLB = ZB; ZZM = 1; END IF
      IF(ZB>.98) THEN        !    ^ Start of work on new R
        IF(JOB/=0) WRITE(6,'(/" >>> This ",A," is orthogonal; it ca",
     +    "n''t be tweaked.")') CC; ZZM = ZMB; IER = -1; RETURN
      END IF
      IF(JOB==0) GOTO 30   ! Only at finish of internal Cuu production
      WRITE(6,'(/1X,A," TWEAK:")') CC
10    C8='factor'; J=6; IF(CC=='Cuu') THEN; C8='exo-root'; J=8; END IF
      WRITE(6,'(3X,"The smallest eigenvalue LOW of your ",A," correlat",
     +  "ion matrix is now ",A4,"."/3X,"By a multiplicative tweak that",
     +  " leaves the correlation ratios unchanged,"/3X,"you can decrea",
     +  "se these correlations by making LOW larger, or increase them"/
     +  3X,"by decreasing LOW.  Admissible limits are 1  LOW  0: The",
     +  1X,A,"s are"/3X,"orthogonalized by LOW = 1, whereas LOW = 0 gi",
     +  "ves them a linear dependency."/)') C8(:J), CLN(ZB,4,3), C8(:J)

C   COV TWEAK:
C   The smallest eigenvalue LOW of your exo-root correlation matrix is now xxxx.
c   By a multiplicative tweak that leaves the correlation ratios unchanged,
C   you can decrease these correlations by making LOW larger, or increase them
C   by decreasing Low.  Admissible limits are 1  LOW  0: All the factors are
C   orthogonalized by LOW = 1, whereas LOW = 0 gives them a linear dependency.

      WRITE(6,'(4X,"If you want to adjust your factor correlations th",
     +  "is way, enter a new"/4X,"choice of LOW.  Otherwise, hit RETU",
     +  "RN to accept ",A," with LOW = ",A4,"."/)') CC, CLN(ZB,4,3)
      CALL SCANB(J,1,'R',5,CH,0)
      IF(J==0) RETURN; IF(J<0) GOTO 10;
      READ(2,*) ZLO; ZLO = ABS(ZLO)
20    IF(ZLO>1.) THEN; ZLO = ZLO/10; GOTO 20; END IF
      WRITE(6,'(4X,"Hit RETURN to confirm choice of ",A4," for ",A,
     +  "''s smallest eigenvalue.")') CLN(ZLO,4,3), CC
      IF(ZLO<.05) WRITE(6,'(4X,"(WARNING: LOW this small is unwise ",
     +  "unless you are testing limits.)")')
      D = ABS(ZLO-ZB); IF(D<.01) IER = -1  ! IER diff not now used
      IF(D<.02) WRITE(6,'(4X,"(Note that this choice is virtually ",
     +  "the same as your initial LOW.)")')
      WRITE(6,'(4X,"Otherwise enter any letter to choose again, or ",
     +  "any number to scrub Tweak.")')
      CALL SCN(J); IF(J<0) GOTO 10
      IF(J>0) THEN; ZLO = ZB; IER = -2; RETURN; END IF
C Theory on 1-parameter tweaking of any unit-diagonal C to fix its jth root:
C For any scalar multiplier a, Q = a(C-I)+I rescales the off-diagonal C-terms
C while leaving unities on its diagonal:  If C = TDT' is C's eigenstructure,
C Q's is Q = T(aD - a+1)T'. So if d is the jth root of C, the jth root d' of
C Q is d' = ad - a+1 = 1+a(d-1) which equals stipulated r iff a = (1-r)/(1-d).
30    IF(JOB==0 .AND. ZB>.1) RETURN
      Z = (1-ZLO)/(1-R(NF))
      DO I = 1,NF-1
        DO J = I+1,NF
          CFF(LO(I,J)) = Z*CFF(LO(I,J))
        END DO
      END DO
      DO I = 1,NF
        R(I) = 1 + Z*(R(I)-1)
      END DO
      IF(JOB/=0) WRITE(6,'(" Eigenvalues of your tweaked ",A," are",
     +  20(:/2(4X,5A6)))') CC, (CLN(R(I),6,2),I=1,NF)
50    ZZM = (1-R(NF))/(1-ZLB); ZMB = ZZM
      END SUBROUTINE  ! ZZM would take the original ZLB into the final R(NF)
C
      SUBROUTINE WAIT(N)
C  N > 0 calls for space before screen display
      DO I = 1,N; WRITE(6,'()'); END DO
      WRITE(6,'(" Hit RETURN to continue")')
      READ(5,'()')
      RETURN
      END

      BLOCK DATA RRSIG
C SDs of TWIST distributions at powers R from 1/6 to 6 in steps of .10.
C Approximate average of two samplings each of size 100,000,000
      REAL RSIG(-50:50)
      COMMON/RS/ RSIG
      DATA  RSIG/  .0920,  .0943,  .0968,  .0993,  .1020,  .1029,
     +     .1037,  .1048,  .1061,  .1077,  .1096,  .1119,  .1146,
     +     .1176,  .1210,  .1247,  .1289,  .1334,  .1382,  .1434,
     +     .1458,  .1478,  .1504,  .1539,  .1583,  .1636,  .1700,
     +     .1773,  .1856,  .1949,  .2049,  .2084,  .2139,  .2216,
     +     .2318,  .2446,  .2601,  .2782,  .2927,  .3034,  .3200,
     +     .3433,  .3735,  .4099,  .4307,  .4670,  .5206,  .5848,
     +     .6400,  .7372,  .8527,  1.000,  1.166,  1.314,  1.523,
     +     1.729,  1.960,  2.255,  2.522,  2.856,  3.266,  3.633,
     +     4.107,  4.666,  5.198,  5.873,  6.648,  7.424,  8.385,
     +     9.478,  10.61,  12.00,  13.55,  15.23,  17.26,  19.47,
     +     21.96,  24.92,  28.14,  31.84,  36.29,  40.93,  46.46,
     +     52.94,  59.94,  68.28,  77.73,  88.43,  101.0,  115.1,
     +     131.3,  150.9,  171.5,  196.3,  224.6,  257.4,  295.4,
     +     337.1,  388.8,  446.6,  513.9 /
C Successive ratio changes in SDs:
C  1.025, 1.026, 1.026, 1.027, 1.009, 1.008, 1.010, 1.013, 1.015, 1.018,
C  1.021, 1.024, 1.026, 1.029, 1.031, 1.033, 1.034, 1.036, 1.038, 1.017,
C  1.013, 1.018, 1.023, 1.029, 1.034, 1.039, 1.043, 1.047, 1.050, 1.051,
C  1.017, 1.026, 1.036, 1.046, 1.055, 1.063, 1.070, 1.052, 1.037, 1.054,
C  1.073, 1.088, 1.097, 1.051, 1.084, 1.114, 1.123, 1.095, 1.152, 1.157,
C  1.159, 1.179, 1.127, 1.160, 1.134, 1.134, 1.151, 1.119, 1.132, 1.143,
C  1.113, 1.131, 1.136, 1.114, 1.130, 1.132, 1.117, 1.130, 1.131, 1.120,
C  1.131, 1.129, 1.123, 1.134, 1.128, 1.128, 1.135, 1.129, 1.132, 1.138,
C  1.130, 1.135, 1.140, 1.132, 1.140, 1.137, 1.138, 1.142, 1.139, 1.142,
C  1.145, 1.141, 1.144, 1.144, 1.145, 1.148, 1.144, 1.151, 1.150, 1.150
      END BLOCK DATA RRSIG

