C  Program KWIKCOVS   (Source code, Fortran-90. EIGS must also be loaded.)
C
C                       Last revised: 2 August 2002
C
C This program, an abbreviation of GENSCOR that omits score production,
C generates a population covariance matrix with a known source structure.
C Its input is a raw simple-structured factor pattern produced by GENPAT, and
C its output is a standardized population covariance matrix.  An arbitrary
C number of simulated sample covariance matrices drawn from this with
C stipulated sample size NS can then be generated by program SAMPLCOV.

C   Input pattern file has name P?ija, where ? is number of complexity-1
C   recurrences in the master pattern block or letter coding thereof if 10,
C    is two-letter code for <JF,NFB,JV> (see GENPAT), ij is two-digit code
C   for complexity levels  2, and a is either a sequential numeric index for
C   repetitions of frame P?ij if JG = 0 or a letter jointly coding JG and
C   number of P?ij repetitions.  (JG is number of general factors in the
C   master pattern block.)

C  Population source-structure information is written to an ASCII file with
C  default name K?ijab.POP for the lowest unused sequential letter index b.
C  Otherwise, the output name is <head>.COV for arbitrary <head>.

C  UPDATE: KWIKCOVS now alternatively allows keyboard entry of a raw factor
C          pattern having arbitrary complexity.  There are no constraints
C          on pattern-file basenames in this case.

      PARAMETER ( MB=7 )
      LOGICAL QY, QP
      CHARACTER*12 CF, F1, F2, F3, LTRS(0:26), WORD*40, CLN*8, EXT*3
      CHARACTER BAR, DBR, PL1*2, PL2*2, CH1, C2
      REAL LOW, B(4,MB)
      CHARACTER,ALLOCATABLE :: IDENT(:)*26, NAME(:)*12
      INTEGER,ALLOCATABLE :: LSET(:), LIST(:), LST(:), PERM(:,:), KOV(:)
      REAL,ALLOCATABLE :: CFF(:), CM(:), CY(:), P(:,:), PP(:,:),
     +     T(:,:), R(:) ! , LL(:)
      COMMON /CF/ CF
      LO(I,J) = J*(J-1)/2 + I
      LOC(I,J) = LO(MIN(I,J),MAX(I,J))
      DATA B0,B1,BL,BH,LOW/.3,.8,-.4,.6,.2/, PC/80./
      DATA BAR,DBR/'',''/
C      KUT = 60
      ML = 40
      ALLOCATE ( NAME(ML) )
      OPEN(2,STATUS='SCRATCH')
      CALL SYSTEM('cls')
C    Ŀ
C     KWIKCOVS generates a population COV-file with a stipulated source 
C     structure either (a) declared in a P... file written by GENPAT or 
C     (b) created internally in KWIKCOVS/GENSCOR from keyboard entry.   
C    
      WRITE(6,'(4X,"",67("","")')
      WRITE(6,'(4X," KWIKCOVS generates a population COV-file with a",
     +  " stipulated source ",4X," structure either (a) declared in",
     +  " a P... file written by GENPAT or ",4X," (b) created inter",
     +  "nally in KWIKCOVS/GENSCOR from keyboard entry.   ")')
      WRITE(6,'(4X,"",67("","")')
6     WRITE(6,'(/4X,"To see the listing of available patterns from GE",
     +  "NPAT, hit RETURN."/4X,"Otherwise, enter anything to initiate",
     +  " pattern entry from keyboard.")')
      CALL SCANB(J,0,'B',5,WORD,1)
      IF(J/=0) GOTO 401
      INQUIRE(FILE='LASTCOV',EXIST=QY)
      IF(.NOT.QY) GOTO 9
      OPEN(3,FILE='LASTCOV')
      READ(3,*,ERR=9) B0, B1, PC, BL, BH, LOW
      CLOSE(3)

C Enter parameters for data generation
9     NN = -1 ! Suppress screen output from LOOK
      CALL LOOK(1,'P*',NAME,ML,NN)
      IF(NN==0) WRITE(6,'(/ " No P... files here. Go generate one.")')
      IF(NN==0) STOP
      ALLOCATE ( LIST(NN) )

C     Ŀ
C        #   P-name    NV    NF   blocksize   complexities  
C     ͵
C       12.  P2AB12C  100    15     25 x 5    1 2 3 4 5 6 7 
C     

11    WRITE(6,'(/" The raw P-patterns available here, with filename",
     +  " information decoded, are:")')
      WRITE(6,'(6X,"",
     +  "Ŀ")')
      WRITE(6,'(6X,"   #   P-name    NV    NF   blocksize",
     +  "   complexities  ")')
      WRITE(6,'(6X,"",
     +  "͵")')
      KKRP = 0
      DO I = 1,NN
        F1 = NAME(I)
        KRP = ICHAR(F1(2:2)) - 48
        IF(KRP>9) KRP = KRP -7   !  alphabetic index + 9
        K = ICHAR(F1(7:7)) - 65  ! <0 if numeral, otherwise alphabet-index less 1
        JG = 0; IF(K>=0) JG = 1+MOD(K,5)
        WORD = ' '; WORD(16:16) = BAR
        CALL DEKOD(NAME(I)(3:7),KRP,NV,NF,JV,JF,JG,WORD,KC,MB)
        LIST(I) = KC
        KKRP = KKRP + KRP-1
        JJG = JG+1
        WRITE(6,'(6X,A,I4,". ",A,1X,A7,1X,A,1X,I3,2X,A,1X,I3,2X,A,2X,
     +    I3," x",I2,2X,A,1X,A)') BAR,I,BAR, NAME(I)(:7), DBR,
     +    NV, BAR, NF, BAR, JV, JF, BAR, WORD(:20)
        IF(MOD(I,18)==0) WRITE(6,'(" Hit RETURN to continue")')
        IF(MOD(I,18)==0) READ(5,'()')
      END DO
      WRITE(6,'(6X,"",
     +   "")')
      IF(KKRP>0) WRITE(6,'(6X,""" 1*k "" signifies k recurrences ",
     +  "of complexity-1 rows in master block.")')
      IF(JJG>1) WRITE(6,'(6X,"""  Gk "" signifies k General factor",
     +  "s in master block.  Each declared"/14X,"complexity J is ",
     +  "actually within-block complexity J+k.")')
      MPAT = 1
14    F1 = NAME(MPAT)
      LF1 = LAST(F1)
16    IF(NN>1) THEN
        WRITE(6,'(/" The raw pattern now chosen is No. ",A," (",A,
     +    ").  Hit RETURN if OK, or"/" enter the index of another ",
     +    "selection from this list."/)') CF(:JJF(MPAT)), F1(:LF1)
      ELSE
        WRITE(6,'(/" This is the only raw pattern here available.  "
     +    "Hit RETURN to continue,"/" or enter anything to exit ",
     +    "and get some more.")')
      END IF
      CALL SCANB(J,1,'I',5,WORD,1)
      IF(J==0) GOTO 19
      IF(NN==1) STOP
      IF(J<0) GOTO 16
      READ(2,*) MPAT
      MPAT = MAX(1,MIN(MPAT,NN))
      GOTO 14
C
19    OPEN(4,FILE=F1)
      KC = LIST(MPAT)
      CALL SCANB(J,0,'I',4,WORD,-20)
      READ(2,*) NV, KV, NF, KF; JV = NV/KV; JF = NF/KF
      IF(F1(:7)/=WORD(13:19)) WRITE(6,'(/" WARNING: This pattern ",
     +  "file was originally named ",A,"."/" Its specifications ",
     +  "may not be correctly reported above.")') WORD(13:19)
      F1(:7) = WORD(13:19)
      NFF = LO(NF,NF); NVV = LO(NV,NV)
      ALLOCATE ( PP(NV,NF), IDENT(NV) )
      CALL GETL(4,' Hi',4,1,IER)
      IF(IER>1) THEN
         WRITE(6,'(" >>> Can''t read this pattern file''s fra",
     +  "me parameters.")'); STOP
      END IF
      DO I = 1,4
        CALL SCANB(J,0,'R',4,WORD,12)
        READ(2,*) (B(I,K),K=1,J)
      END DO
C       B loads frame parameters displayed by SHOWP call.
      IF(J/=NF/JF .OR. WORD(6:9)/='nonS') THEN
        WRITE(6,'(/" Program is not reading the head of input file ",
     +    A," correctly.")') F1(:LF1)
        STOP
      END IF
      CALL SYSTEM('cls'); WRITE(5,'()')
      PL1 = 's,'; PL2 = 's)'
      WRITE(6,'(/1X,A," by ",A," pattern ",A," (",A," item block",
     +  A,1X,A," factor block",A," has complexity"/" levels ",A,
     +  " and frame ","parameters")') CF(:JJF(NV)), CF(:JJF(NF)),
     +  F1(:LF1), CF(:JJF(KV) ),PL1(3-MIN(2,KV):), CF(:JJF(KF)),
     +  PL2(3-MIN(2,KF):), CF(:JJF(KC))
      CALL SHOWP(NF,JF,B,4,6)
      CALL GETL(4,' Item',6,1,IER)
      QP = .TRUE.; IF(IER>0) QP = .FALSE.  ! Probably don't need, but set provisionally
      IF(.NOT.QP) WRITE(6,'(6X,"Subsequent permutation of factor ",
     +  "blocks: None")')   ! No Perm table, no permutations
      IF(QP) THEN
        ALLOCATE ( PERM(KV,KF) )
        DO I = 1,KV
          CALL SCANB(J,0,'R',4,WORD,12); IF(J/=KF+1) IER = I
          IF(IER>0) THEN
            WRITE(6,'(" >>> Failure to read line",I2," of the raw pa",
     +        "ttern permutation table.")') I; STOP
          END IF
          READ(2,*) K, (PERM(I,K),K=1,KF)
        END DO
        WORD(:4) = '   '; IF(KF>9) WORD(:4) = ' '
        WRITE(6,'(4X," Subpattern blocks were shifted thus:     ")')
        WRITE(6,'(4X,"",14(""),"",27(""),"")')
        DO I = 1,KV
          WRITE(6,'(4X," Item block",A2," ",30A3)')
     +      CLN(1.*I,2,10), (CLN(1.*PERM(I,J),3,10),J=1,KF),
     +      ('   ',J=1,MAX(1,9-KF)), WORD(:4)
        END DO
        WRITE(6,'(4X,"",14(""),"",27(""),"")')
        DEALLOCATE ( PERM )
      END IF
      REWIND 4
      CALL START(4,F1)
      DO I = 1,NV
        READ(4,*) (PP(I,J),J=1,NF)
        DO J = 1,NF
          PP(I,J) = PP(I,J)/1000
        END DO
      END DO
C        The raw input pattern is now in PP
      READ(4,*,END=21) CH1; IDENT(1) = '['
      IF(CH1=='N'.OR.CH1=='n') READ(4,*,ERR=21) (IDENT(I),I=1,NV)
      GOTO 22
21    CLOSE(4) ! GENPAT passes the items' name stems
22    WRITE(6,'(/" To see this pattern, enter any letter to display ",
     +  "its 1-digit schema or any"/" number to get 2-digit details.",
     +  "  Otherwise, hit RETURN to move on.")')
      CALL SCANB(J,0,'R',5,WORD,1)
      IF(J>0) THEN
      C2 = CF(:JJF(JF))
      WORD = '(I4,".",2(2X,'//C2//'A),5(:/6X,2(2X,'//C2//'A)))'
        DO I = 1,NV
          WRITE(6,WORD) I, (CLN(PP(I,J),5,-2),J=1,NF)
          IF(MOD(I-1,JV)==0 .AND. I<NV) WRITE(6,'()')
          IF(((MOD(I,10)==0.AND.NV-I>5).OR.I==NV)) CALL WAIT(1)
        END DO
      ELSE IF(J<0) THEN
        CALL SCHEM(NV,NF,JV,JF,.1,PP,NV)
      END IF
      WORD(:13) = 'pick another.'; IF(NN==1) WORD(:13) = 'quit.'
      WRITE(6,'(/" If satisfied that ",A," is the pattern you want, ",
     +  "hit RETURN. Otherwise,"/" enter anything to ",A)') F1(:LF1),
     +  WORD(:13)
      CALL SCANB(J,0,'B',5,WORD,1)
      IF(J/=0) THEN
         IF(NN==1) STOP
         DEALLOCATE ( PP, IDENT )
        GOTO 11
      END IF
      DEALLOCATE ( LIST )
      ALLOCATE ( LSET(NFF), LIST(MAX(NFF,NV+1)), KOV(NFF) )
      ALLOCATE ( CFF(NFF), CM(NV), CY(NVV), P(NV,NF), R(NV), T(NV,NV) )
C
C The to-be-generated population COV-file is named K?ijab.POP, with code
C ?ija from input pattern P?ija, for the lowest free letter b.
      N = 0
31    N = N+1
CC      F2 = 'K'//F1(2:7)//CHAR(N+64)//'.POP'
      F2 = 'K'//F1(2:7)//CF(:JJF(N))//'.POP'
      INQUIRE(FILE=F2,EXIST=QY)
      IF(QY) GOTO 31
      LF2 = 12
C
C Enter control parameters.
      INQUIRE(FILE='LASTGEN',EXIST=QY)
      IF(QY) THEN
        OPEN(15,FILE='LASTGEN')
        READ(15,*) M, N, K, B0, B1, NC, BL, BH, LOW
C         Uniquenesses will be set randomly within limits <B0,B1>
        CLOSE(15)
      END IF
C  Distribute unadjusted random communalities over full unit-interval
      X = 1.; Y = 0.
      DO I = 1,NV
        Z = B0 + BD*RANDY(); CM(I) = Z
        X = MIN(X,Z); Y = MAX(Y,Z)
      END DO
      D = Y-X
      DO I = 1,NV; CM(I) = (CM(I)-X)/D; END DO
      WRITE(6,'(/" Some additional frame specifications are now",
     +  " needed to finish raw pattern ",A,"."/)') CF(:JJF(MPAT))

C ============ Insert pattern entry from keyboard here  ==============


C  You have opted for a keyboard-entry factor pattern. If that was in error,
C  hit RETURN to reconsider.  Otherwise, enter your choice of NY (number
C  of data items; NY  99) and NF (number of common factors; NF  NY).

401   WRITE(6,'(/"  You have opted for a keyboard-entry factor patte",
     +  "ern. If that was in error,"/"  hit RETURN to reconsider.  Ot",
     +  "herwise, enter your choice of NY (number"/"  of data items; ",
     +  "NY  99) and NF (number of common factors; NF  NY)."/)/)
      CALL SCANB(J,0,'I',5,WORD,0); IF(J==0) GOTO 6; IF(J<0) GOTO 401
      ALLOCATE ( LSET(J) ); NV = 0; NF = 0
405   READ(2,*) LSET  ! ??? Will this read correctly ???
      N = ABS(LSET(1)); M = N; IF(J>1) M = ABS(LSET(2))
      NV = MIN(99,MAX(M,N)); NF = MIN(M,N,NV)
406   WRITE(6,'(5X,"Your pattern size is now NY = ",A," item rows wi",
     +  "th NF = ",A," factor"/5X,"columns.  Hit RETURN to accept, or",
     +  " enter corrected NY and NF.")') CF(:JF(NV)), CF(:JF(NF))
      IF(NV==NF) WRITE(6,'(5X,"WARNING: NF = NY is unwise.")')
      WRITE(6,'()')
      CALL SCANB(J,0,'I',5,WORD,0); IF(J<0) GOTO 401
      IF(J>0) GOTO 405
      DEALLOCATE ( LSET )
      ALLOCATE ( LST(0:NV*NF); LST = 0
CC  Check for archived key-entered NV-by-NF patterns to upload into LST

      F3 = 'PATS'//CF(:JF(NV)//CF(:JF(NF));  LF3 = LAST(F3)
      INQUIRE(FILE=F3(:LF3),EXIST=QY)







      CALL GETPAT(NV,NF,LST,LTRS)







C =============== Start of replacement section  =====================

C   Start with the data variables' communalities, which you will recall are
C   their variance proportions accounted for by the common factors in this
C   dataset.  These will be randomized between your pick of limits <Lo,Hi>,
C   which you can then override by stipulations for selected variables.

c Your range <Lo,Hi> of randomized communalities is now set at <xxx, xxx>.
c Hit RETURN if OK, or enter two numbers in the unit interval to revise this.

40    WRITE(6,'(3X,"Start with the data variables'' communalities, w",
     +  "hich you will recall are"/3X,"their variance proportions ac",
     +  "counted for by the common factors in this"/3X,"dataset.  Th",
     +  "ese will be randomized between your pick of limits <Lo,Hi>,"/
     +  3X,"which you can then override by stipulations for selected",
     +  " variables."/)')
41    WRITE(6,'(" Your range <Lo,Hi> of randomized communalities is ",
     +  "now set at <",A3,", ",A3,">."/" Hit RETURN if OK, or enter ",
     +  "two numbers in the unit interval to revise this."/)')
     +  CLN(B0,3,2), CLN(B1,3,2)
      CALL SCANB(J,0,'RR',5,WORD,0)
      IF(J==0) GOTO 47
      IF(J<0) GOTO 41
      READ(2,*) X, Y
45    IF(X>1. .OR. Y>1.) THEN
        IF(ABS(X)>1.) X = X/10; IF(ABS(Y)>1.) Y = Y/10
        GOTO 45
      END IF
      B0 = MAX(0.,MIN(1.,X,Y)); B1 = MIN(1.,MAX(0.,X,Y)); GOTO 41
47    D = B1-B0; DO I = 1,NV; CM(I) = D*CM(I) + B0; END DO  ! <= Final random comms
      WRITE(6,'(" If you don''t want to override any of the communali",
     +  "ties now randomized, hit"/" RETURN.  Otherwise, enter anythi",
     +  "ng to initiate finer communality control.")')
      CALL SCANB(J,0,'B',5,WORD,0); IF(J==0) GOTO 67; KNT = 0

C  Ŀ
C   To assign one or more variables the same communality CM in the unit    
C   interval, enter a line starting with a numeric CM (don't ignore its    
C   decimal), followed either by two integers I,J if you want all items    
C   in index interval [I,J] to have this communality, or by some other     
C   count of indices to assign CM to just the variables explicitly listed. 
c   (To specify CM for just two nonadjacent indices, enter one twice to    
C   change the count.)  You will have opportunity to declare more, as      
C   well as to review and correct errors.                                  
C  
      CALL SYSTEM('cls')
      WRITE(6,'("  ",72(""),"")')
      WRITE(6,'("   To assign one or more variables the same communa",
     +  "lity CM in the unit    "/"   interval, enter a line starti",
     +  "ng with a numeric CM (don''t ignore its    "/"   decimal),",
     +  " followed either by two integers I,J if you want all items",
     +  4X,""/"   in index interval [I,J] to have this communality,",
     +  " or by some other     "/"   count of indices to assign CM ",
     +  "to just the variables explicitly listed. "/"   (To specify",
     +  " CM for just two nonadjacent indices, enter one twice to    ",
     +  ""/"   change the count.)  You will have opportunity to dec",
     +  "lare more, as",6X,""/"   well as to review and correct err",
     +  "ors.",34X,"")')
      WRITE(6,'("  ",72(""),"")')
      WORD = ' '; IF(KNT==0) WORD(:7) = ' empty.'
51    WRITE(6,'(/" Your list of specified communalities is now",A)')
     +  WORD(:7)
      IF(KNT>0) WRITE(6,'(/11(:/1X,9(1X,A,"(",A3,")")))') (CF(:JJF(
     +  LIST(I)/1000)),CLN(.001*MOD(LIST(I),1000),3,2),I=1,KNT)
      WRITE(6,'(/" If accurate and complete, hit RETURN.  Otherwise, ",
     +  "enter another line of item"/" indices preceded by the commun",
     +  "ality assigned to them.  Including an index"/" entered previ",
     +  "ously will replace its earlier specification with the latest"/
     +  " one.  To delete items from this specification list, enter a",
     +  " negative number"/" followed by indices to be dropped."//)')
55    CALL SCANB(J,0,'R',5,WORD,0); JJ = J-1
      IF(J==0) GOTO 67
      IF(J<0 .OR. J==1) WRITE(6,'(4X,"Huh ???  Enter a real number ",
     +  "followed by at least one integer,"/13X,"or hit RETURN to ",
     +  "move on."/)')             ! Working register for declaring comms is
      IF(J<0 .OR. J==1) GOTO 51    ! LIST, whose Jth entry is J*1000 plus 1st
      READ(2,*) Z, (R(I),I=1,JJ)   ! 3 decimals of its comm with point omitted.
      DO J = 1,JJ; IF(R(J)/=ANINT(R(J))) R(J) = 0; END DO
      IF(JJ==2) THEN   ! Interval reading of just two entries
        K = NINT(R(1)); JJ = NINT(R(JJ))
        DO J = K,JJ; R(J-K+1) = J; END DO
      END IF
      IF(Z<0.) THEN  ! Deletions
        DO K = 1,KNT; DO J = 1,JJ
          IF(NINT(R(J))==LIST(K)/1000) LIST(K) = 999999
        END DO; END DO
        CALL ISORT(KNT,LIST); LIST(KNT+1)=999999
57      IF(LIST(KNT)==999999) THEN; KNT = KNT-1; GOTO 57; END IF
        GOTO 51
      END IF
61    IF(Z>1.) THEN; Z = Z/10.; GOTO 61; END IF
      MM = MIN(999,NINT(Z*1000)) ! MM is the mod part of new entries
      OUTR:DO J = 1,JJ  ! First check if index NN is already listed
        NN = NINT(R(J)); IF(NN<1.OR.NN>NV) CYCLE ! Not a valid index
        DO I = 1,KNT
          IF(LIST(I)/1000 == NN) THEN
            LIST(I) = 1000*(LIST(I)/1000) + MM; CYCLE OUTR
          END IF ! Cycling on outer loop insures no index duplications in LIST
        END DO
        KNT = KNT+1; LIST(KNT) = 1000*NN + MM
      END DO OUTR
      CALL ISORT(KNT,LIST)
      GOTO 51
67    DO I = 1,KNT
        Z = MOD(LIST(I),1000)*.001; IF(Z>.998) Z=1.
        CM(LIST(I)/1000) = Z
      END DO

c >>>> Next, you must provide factor correlations in one of three ways:
c      1) Retrieve a stored matrix Cff of correlations for xx factors.
c      2) Create a new Cff randomized under coarse control of obliquity.
c      3) Make Cff orthogonal except for boundedly randomized obliquities
c           at the specific locations in Cff wanted for a SEM simulation.
      WRITE(6,'(" >>>> Next, you must provide factor correlations in",
     +  " one of three ways:")')
      CALL MAKCOV(NF,CFF,KOV,LSET,LIST,T,NV,R,F3,'KWIKCOVS')

C Finish the simple-structure factor pattern, starting with uniquenesses.
201   BD = B1-B0
      DO I = 1,NV; SF = 0.
        DO J = 1,NF; DO K = 1,NF
            SF = SF + PP(I,J)*CFF(LOC(J,K))*PP(I,K)
        END DO; END DO
        S = SQRT(CM(I)/SF)
C         CM(I) is the Ith variable's assigned communality. And SF is the Ith
C         diagonal element of the raw true-part covariance matrix PP*CF*PP'.
        DO J = 1,NF
          P(I,J) = S*PP(I,J)
        END DO
      END DO
      IF(IDENT(1)(:1)/='[') CALL NAMS(NV,NF,IDENT,P)
C ###     P is now the NV-by-NF 1st-level simple-structure factor pattern;
C ###     communalities are in CM; saliences should be coded in item names.
      IF(JF>1) THEN
        OPEN(9,STATUS='SCRATCH',FORM='UNFORMATTED')
        WRITE(9) ((P(I,J),I=1,NV),J=1,NF), (CFF(J),J=1,NFF)
      END IF
      WRITE(6,'(/" The simple-structure factor pattern (communalit",
     +  "ies in parens) is:")')
      DO I = 1,NV
        IF(MOD(I,5)==1) WRITE(6,'()')
        WRITE(6,'(I4,". (",A4,")",2(2X,5A5),20(:/14X,2(2X,5A5)))')
     +    I, CLN(CM(I),4,2), (CLN(P(I,J),5,2),J=1,NF)
        IF(MOD(I,10)==0 .OR. I==NV) THEN
          IF(I<NV) WRITE(6,'(/" Hit RETURN to continue, or enter ",
     +      "anything to omit the pattern''s remainder.")')
          IF(I==NV) WRITE(6,'(/" Hit RETURN to continue.")')
          CALL SCANB(J,0,'B',5,WORD,0)
          IF(J/=0) GOTO 250
        END IF
      END DO
C ==================== Current end of replacement ========================
C
C   Put true-part covars (call them CH) into T to solve for eigenstructure
250   WRITE(6,'(/" Computing common-parts eigenstructure.")')
      DO J = 1,NV
        CY(LO(J,J)) = 1.  ! CY gets the ideal data correlations
        DO I = 1,J; S = 0.
          DO K = 1,NF; DO M = 1,NF
              S = S + P(I,K)*CFF(LOC(K,M))*P(J,M)

          END DO; END DO
          T(I,J) = S    ! Can pass covars in T, which returns eigenvectors
C         Lower triangle of T isn't needed by EIGS
          IF(I/=J) CY(LO(I,J)) = S
        END DO ! T gets the common-part covars CH with comms on the diagonal
      END DO   ! CY holds data correlations; from T by unities on diagonal
      CALL EIGS(NV,NF,T,NV,R,PP,NV,1,IER,6)
C        Roots of common-parts CH are now in R; eigenvectors of CH are in PP
      WRITE(6,'(" The nonzero common-parts eigenvalues are",
     +  3(:/2X,12A6))') (CLN(R(I),6,2),I=1,NF)
      WRITE(6,'(/" If satisfied, hit RETURN to compute dataspace co",
     +  "variances.  Otherwise,"/" enter anything to re-specify the",
     +  " production parameters.")')
      CALL SCANB(J,0,'B',5,WORD,1)
      IF(J/=0) GOTO 40

C Get eigenstructure of data covars  ! Not wanted in GENSCOR?
      CALL EIGS(NV,NV,CY,NVV,R,T,NV,1,IER,6) ! *** Needed to simulate sampling
C       Dataspace eigvals are now in R, with eigvecs in T
      WRITE(6,'(" The leading dataspace eigenvalues are",
     +  3(:/2X,12A6))') (CLN(R(I),6,2),I=1,MIN(2*NF,NV))
      WRITE(6,'(/" If satisfied, hit RETURN to file the dataspace ",
     +  "covariances.  Otherwise,"/" enter anything to re-specify",
     +  " the production parameters.")')
      CALL SCANB(J,0,'B',5,WORD,1)
      IF(J/=0) GOTO 40             ! ^^^Screen report wanted???
      DO J = 1,NV
        S = SQRT(MAX(0.,R(J)))
        DO I = 1,NV
          T(I,J) = T(I,J)*S
        END DO   ! ^ Making T the pattern on normalized prin. components
      END DO     !   That is, T is the Principal Gram-factor of CY = T*T'

C =============== End of maybe replacement ==========================
C      Ends with the population simple-structure Cff, communalities, and
C      common-part covariances converted to data covars by unitizing its diag.

C File population dataspace covs, source pattern and factor covs in K<code>.POP.
      OPEN(3,FILE='LASTCOV')
      WRITE(3,'(6(1X,A4))') CLN(B0,4,3), CLN(B1,4,3), CLN(PC,4,1),
     +  CLN(BL,4,3), CLN(BH,4,3), CLN(LOW,4,3)
260    WRITE(6,'(/" To write these simulated data correlations under",
     +  " filename ",A," hit RETURN."/" Otherwise, to save them unde",
     +  "r name <head>.COV, enter basename <head>."/)') F2(:LF2)
      READ(5,'(A)') WORD
      IF(LAST(WORD)==0) GOTO 280
      N = 0
262   N = N+1
      NC = ICHAR(WORD(N:N))
      IF(NC>=97 .AND. NC<=122) WORD(N:N) = CHAR(NC-32)  ! Enforce upper case
      IF(N<K .AND. NC>=48) GOTO 262
      F2 = WORD(:N)//'.COV'; LF2 = LAST(F2)
      INQUIRE(FILE=F2(:LF2),EXIST=QY)
      IF(QY) THEN
        NN = ICHAR(F2(LF2-4:LF2-4)); IF(NN>47 .AND. NN<58) GOTO 267
        WRITE(6,'("  *** NOTE: A COV-file named ",A," already exists",
     +    " in this directory."/15X,"To replace that by this one, en",
     +    "ter any letter; otherwise, hit"/15X,"RETURN to change thi",
     +    "s headname or append an index to it.")') F2(:LF2)
        CALL SCANB(J,0,'B',5,WORD,1)
        IF(J/=0) GOTO 280
267     CALL NAMM(F2,EXT,K); LF2 = LAST(F2)
        GOTO 260
      END IF
280   F3 = F2(:LF2-3)//'EIG'  ! File dataspace and source-space eigenstructure
      S = 0.; X = 0.; TOP = 0.; BOT = 99.
      DO I = 1,NV  ! Get mean,SD,Lo,Hi of communalities
        TOP = MAX(TOP,CM(I)); BOT = MIN(BOT,CM(I))
        S = S + CM(I)
        X = X + CM(I)*CM(I)
      END DO
      S = S/NV    ! Mean and SD of communalities printed below
      X = SQRT(MAX(0.,X/NV - S*S))
      OPEN(7,FILE=F2)

ccC MODA read of COV-file
cc          READ(4,*) NV, NVV, KODE, MTH (last two are optional)
cc        READ(4,*) (W1(I),I=1,NVV)  ! Input covariances
cc20      READ(4,'(A)',END=21) CH
cc        IF(CH=='N') THEN
cc          READ(4,*,ERR=21,END=21) (IDENT(I),I=1,NV)
cc          NMS = 1
cc        ELSE IF(CH=='T') THEN
cc          READ(4,*,ERR=21,END=21) (LB(I),I=1,NV)  ! Recover rawdata indices of COV variables
cc          WRITE(19) (LB(I),I=1,NV)
cc        ELSE IF(CH=='B') THEN
cc          READ(4,*,END=21,ERR=21) NB,(LB(I),I=1,NB)
cc        END IF

C   Moda/Hyball header layout
      KODE = INT(100000*RANDY())
      IF(IDENT(1)(:1)=='[') THEN
        DO I = 1,NV; IDENT(I) = '['//CF(:JJF(I))//']  '; END DO
      END IF
      WRITE(7,'(" Simulated POPULATION correlations for ",A,
     +  " variables ")') CF(:JJF(NV))
      WRITE(7,'()')
      WRITE(7,'(7(1X,A))') CF(:JJF(NV)), CF(:JJF(NVV)),
     +  CF(:JJF(KODE)), '0'
CCC      WRITE(7,'(25I6)') (NINT(1000*CY(I)),I=1,NVV)  ! Standard Hyball layout
      DO J = 1,NV  ! Pruzek's preferred covariance layout
CC        WRITE(7,'(25(1X,A5))') (CLN(CY(LO(I,J)),5,3),I=1,J)
        WRITE(7,'(25(1X,A5))') (CLN(1000*CY(LO(I,J)),5,10),I=1,J)
      END DO
      WRITE(7,'(/"Names assigned to the variables:")')
      WRITE(7,'(99(10(1X,A),:/))') (IDENT(I)(:LAST(IDENT(I))),I=1,NV)

CCCC   Pruzek's preferred layout
CCC      WRITE(7,'(2(1X,A)," L/ Simulated POPULATION correlations for ",
CCC     + A," data variables")') CF(:JJF(NV)), CF(:JJF(NV)),
CCC     + CF(:JJF(NV)) ! Pruzek's preferred heading
CCC      DO J = 1,NV  ! Pruzek's preferred covariance layout
CCC        WRITE(7,'(25(1X,A5))') (CLN(CY(LO(I,J)),5,3),I=1,J)
CCC      END DO
      WRITE(7,'(/1X,80("="))')
      WRITE(7,'(/" Pattern of ",A, " variables on ",A," factors refi",
     +  "ned from raw pattern in GENPAT file ",A,":")') CF(:JJF(NV)),
     +  CF(:JJF(NF)), F1(:LF1)
      WRITE(7,'(" Communalities (in parentheses) have Mean =",A5,
     +  ", SD =",A5,", Lo =",A5,", Hi =",A5)') CLN(S,5,3), CLN(X,5,3),
     + CLN(BOT,5,3), CLN(TOP,5,3)
      DO I = 1,NV
        WRITE(7,'(I4,". (",A4,")",30A7)') I, CLN(CM(I),4,3),
     +    (CLN(P(I,J),7,3),J=1,NF)
      END DO
      WRITE(7,'(/" Common-factor covariances:")')
      DO I = 1,NF
        WRITE(7,'(12X,20A7)') (CLN(CFF(LO(J,I)),7,3),J=1,I)
      END DO
cc      WRITE(7,'(/" Eigenvalues of the common-parts covariance ",
cc     +  "matrix:",2(/2X,25A6))') (CLN(R(I),6,-2),I=1,NF) ****** NO
      WRITE(7,'(/" Eigenvalues of the data correlation matrix:",
     +  8(/2X,20A6))') (CLN(R(I),6,-2),I=1,NV)
      WRITE(7,'(/80("-"))')
      WRITE(7,'(/" Raw source pattern ",A," was generated under ",
     +  "complexity levels ",A," and frame parameters")') F1(:LF1),
     +   CF(:JJF(KC))
      CALL SHOWP(NF,JF,B,4,7)
      WRITE(7,'(/" COV-file ",A," was finished under producion pa",
     +  "rameters"/3X," <B0,B1> = <",2(1X,A3),">, <Pc,BL,BH> = <",
     +  I3,2(1X,A3),">, LOW = ",A3)') F2(:LF2), CLN(B0,3,2),
     +  CLN(B1,3,2), NINT(PC), CLN(BL,3,2), CLN(BH,3,2), CLN(LOW,3,2)
      WRITE(7,'(/" <B0,B1> are communality bounds.  Pc% of the fact",
     +  "or correlations are oblique"/" and within bounds <BL,BH> ",
     +  "until adjusted to have smallest eigenvalue LOW.")')

      WRITE(6,'(/" KWIKCOVS'' primary output, in ASCII file ",A,
     +  ", is complete.")') F2(:LF2)

cc      IF(NV<=KUT) STOP ! ******** Why here ????
cc      WRITE(7,'(/" Loadings of these data variables on their common",
cc     +  "-parts principal factors together with"/" the unique-fac",
cc     +  "tor loadings are saved in unformatted file ",A)') F3

CC  Terminal section on blocks not checked for update errors and probably never wanted
CCC ********* Does this last part still have any point ???
CC      OPEN(8,FILE=F3,FORM='UNFORMATTED')
CC      WRITE(8) NV, F2
CC      WRITE(8) ((T(I,J),I=1,NV),J=1,NV)  ! Principal Gram-factor of data covars
CC      WRITE(8) NV, NF, JF  !  ^^^  Currently replaced by something else
CC      IF(JF<NF) THEN
CC        REWIND 9
CC        READ(9) ((P(I,J),I=1,NV),J=1,NF), (CFF(J),J=1,NFF)
CC      PP = P   ! OK ?? And why ??
CCc      DO I = 1,NV
CCc        DO J = 1,NF
CCc          PP(I,J) = P(I,J)
CCc        END DO
CCc      END DO
CC
CCC TEST LINES
CCC      E = ERR(NV,NF,JF,PP,CF,CY,NV)
CCC      WRITE(6,'(/" Max error in S.S. factor reproduction of the da",
CCC     + "ta covariances:",A6)') CLN(E,6,3)
CCC      WRITE(21,'(/" Max error in S.S. factor reproduction of the da",
CCC     + "ta covariances:",A6)') CLN(E,6,3)
CC
CCC   For each factor block in order, rotate to principal factors and partial
CCC     factors in block out of later blocks.
CC      NB = NF/JF         ! Why orthogonalize by blocks ???
CC      BIG: DO KK = 1,NB
CC        M = (KK-1)*JF    ! Last index before currently focal block
CC        MM = KK*JF       ! Last index in currently focal block
CC        DO J = 1,JF
CC          DO I = 1,J
CC            T(I,J) = CFF(LO(M+I,M+J))  ! Get eigstructure of current block covars
CC          END DO
CC        END DO
CC        CALL EIGS(JF,JF,T,NV,R,T,NV,1,IER,6)
CC
CCC TEST LINES
CCC           WRITE(21,'(/" Block",I2," eigenvalues:",10A5)') KK,
CCC         +   (CLN(R(I),5,2),I=1,JF)
CCC         +   (CLN(R(I),5,2),I=1,JF)
CC
CC        DO J = 1,JF
CC          R(J) = SQRT(MAX(0.,R(J)))
CC          DO I = 1,NV   ! Pattern on block factors' prin.axes is PP = P*T*L
CC            S = 0.
CC            DO K = 1,JF
CC              S = S + P(I,M+K)*T(K,J)
CC            END DO
CC            PP(I,M+J) = S*R(J)
CC          END DO
CC        END DO
CC        DO J = MM+1,NF  ! C12 <= Inv(D)*T'*C12
CC          DO I = 1,JF
CC            S = 0.
CC            DO K = 1,JF
CC              S = S + T(K,I)*CFF(LOC(M+K,J))
CC            END DO
CC            LL(I) = S/R(I)
CC          END DO
CC          DO I = 1,JF
CC            CFF(LO(M+I,J)) = LL(I)
CC          END DO
CC        END DO
CCC    Block factors are rotated to prin.axes; now partial out of remaining blocks
CC        IF(KK==NB) CYCLE BIG
CC        DO I = 1,NV       ! A1 <= A1 + A2*B21 = A1 + A2*C21 since C11 = I
CC          DO J = M+1,MM
CC            S = 0.
CC            DO K = MM+1,NF
CC              S = S + P(I,K)*CFF(LO(J,K))
CC            END DO
CC            PP(I,J) = PP(I,J) + S
CC          END DO
CC        END DO
CC        DO J = MM+1,NF   ! C22 <= C22 - C21*Inv(C11)*C12 = C22 - C21*C12 since C11 = I
CC          DO I = MM+1,J
CC            DO K = M+1,M+JF
CC              CFF(LO(I,J)) = CFF(LO(I,J)) - CFF(LO(K,I))*CFF(LO(K,J))
CC            END DO
CC          END DO
CC        END DO
CC        DO J = MM+1,NF
CC          DO I = M+1,M+JF
CC            CFF(LO(I,J)) = 0.
CC          END DO
CC        END DO
CCC    Get principal axes of post-regression blocks
CC        DO J = 1,NV
CC          DO I = 1,J
CC            T(I,J) = 0.
CC            DO K = M+1,MM
CC              T(I,J) = T(I,J) + PP(I,K)*PP(J,K)
CC            END DO
CC          END DO
CC        END DO
CC        CALL EIGS(NV,JF,T,NV,R,T,NV,1,IER,6)
CC        IF(LL(JF)<-.005) WRITE(6,'(" WARNING: Block",I2," eigen",
CC     +    "values are",10(1X,A5))')  KK, (CLN(LL(I),5,3),I=1,JF)
CC        DO J = 1,JF
CC          T(M+J,NV) = LL(J)   ! *** Doesn't seem to be used
CC          S = SQRT(MAX(0.,LL(J)))
CC          DO I = 1,NV
CC            PP(I,M+J) = S*T(I,J)
CC          END DO
CC        END DO
CC      ENDDO BIG  ! Finish factor block
CC      END IF
CC      WRITE(8) ((PP(I,J),I=1,NV),J=1,NF) ! Principal Gram-factor of CH
CC      WRITE(8) (SQRT(MAX(0.,1.-CM(I))),I=1,NV)  ! Unique-factor loadings
CC      WRITE(6,'(/" Production of population COV-file ",A," is com",
CC     +  "plete.")') F2(:LF2)
CC
CCC TEST LINES
CCC      E = ERR(NV,NF,JF,PP,CF,CY,NV)
CCC      IF(E>.01) WRITE(6,'(/" Max error in block-prin.factoring ",
CCC     +  "the data covariances:",A6)') CLN(E,6,3)
CC
CC      IF(JF==NF) STOP
CC      WRITE(7,'(/80("-"))')
CC      WRITE(7,'(/" Factor pattern on principal axes of residual comm",
CC     +  "on parts in each block of",I2," simple-structure factors af",
CC     +  "ter the more prominent blocks are partialled out."/)') JF
CC      DO I = 1,NV
CC        WRITE(7,'(I4,". (",A4,")",30A7)') I, CLN(CM(I),4,3),
CC     +    (CLN(P(I,J),7,3),J=1,NF)
CC      END DO
CC      WRITE(6,'(" Eigenvalues of the ",A," blocks of successiv",
CC     +  "ely orthogonalized common parts are")') CF(:JJF(NB))
CC      DO K = 1,NB
CC        M = (K-1)*JF; WRITE(6,'(3X," Block ",A,":",9(2X,A4))')
CC     +    CF(:JJF(K)), (CLN(T(M+I,NV), 4,3),I=1,JF)
CC      END DO

      STOP
      END
C
      FUNCTION JJF(N)
C Return character expression of integer N left-justified in field CF; then
C CF(:JJF(N)) writes N with exactly the right length in format specifier A.
      CHARACTER*12 CF
      INTEGER(4) K
      COMMON /CF/ CF
      K = ABS(N)
      CF = '            '
      J = 13
10    J = J-1
      CF(J:J) = CHAR(48+MOD(K,10))
      K = K/10
      IF(K>0) GOTO 10
      IF(N<0) CF(J-1:J-1) = '-'
      CF = ADJUSTL(CF)
      JJF = LEN_TRIM(CF)
      END FUNCTION
C
      SUBROUTINE CAP(WORD,L)
C This makes the first L letters in WORD all upper-case.
      CHARACTER WORD*(*)
      DO I = 1,L; N = ICHAR(WORD(I:I))
        IF(N>=97 .AND. N<=122) WORD(I:I) = CHAR(N-32)
      END DO
      RETURN
      END
C
      FUNCTION CLN(X,NFF,ND)
C Express real number X as a character string in fieldwidth NF with LD decimals
C when room, where LD = ABS(ND).  ND<0 tries to precede all positive numbers
C with a blank. LD > 9 displays zero decimals and no decimal point while
C returning X = 0 as ND-10 zeros right-justified in selected field.
C ***** WARNING: If a call of CLN prints garbage, you have forgotten to
C       declare CLN as CHARACTER*8 in the calling routine.
      PARAMETER (KW=16)
      CHARACTER CLN*8, WK(KW)
      CLN = '        '
      NF = MIN(8,NFF)  ! Limit fieldwidth to 8 chars
      LD = ABS(ND)
      IF(ABS(X)<1.0E-12) THEN !  Special for vanishingly small X
        N = MIN(NF-1,MOD(LD,10))
        IF(LD<10) CLN(NF-N:NF+1-N) = '.0'
        IF(LD<10) RETURN
        DO I = 1,N
          CLN(NF+1-I:NF+1-I) = '0'
        END DO
        RETURN
      END IF
      M = NF; IF(ABS(X)>1.) M = NF-1-INT(LOG10(ABS(X))) !  M is space free for decimal (or M-1 if X<0) )
      IF(M<0 .OR. M<1.AND.X<0.) GOTO 55
      IF(ND>=10) LD = 0; LD = MIN(LD,M)
      DO I = 1,KW
        WK(I) = ' '
      END DO
      IF(LD==0) N = NINT(ABS(X))         ! Shouldn't be needed, but is
      IF(LD/=0) N = NINT(ABS(X)*10**LD)  ! This can overflow if LD is large
      DO I = KW,KW-LD+1,-1
        WK(I) = CHAR(48+MOD(N,10))
        N = N/10
      END DO
      WK(KW-LD) = '.'
      IF(N==0 .AND. LD==0) WK(KW-LD-1) = '0'
      IF(N==0 .AND. X<0. .AND. LD>0) WK(KW-LD-1) = '-'
      DO I = KW-LD-1,2,-1
        IF(N>0) WK(I) = CHAR(48+MOD(N,10))
        IF(N>0 .AND. X<0.) WK(I-1) = '-'
        N = N/10
      END DO
      KS = KW-NF+1
25    KS = KS-1
C       Field returned will start at WK(KS+1) for initial KS
      IF(KS<=0) GOTO 50
      IF(ND>=0) THEN
        IF(WK(KS)/=' ' .AND. WK(KS+NF)/='.') GOTO 25
      ELSE
        IF(WK(KS+1)/=' '. AND. WK(KS+1)/='-' .AND. WK(KS+NF)/='.')
     +    GOTO 25
      END IF
      IF(ND>=10) KS = MAX(1,KS-1)
      DO I = 1,NF
        CLN(I:I) = WK(KS+I)
      END DO
50    IF(WK(KS)==' ') RETURN
55    CLN = ' *******'
      END FUNCTION
C
      SUBROUTINE DEKOD(CH,KRP,NV,NF,JV,JF,JG,LST,KC,MB)
C CH(1:2) is letter code for <JF,NFB,NVB>; CH(3:4) is number code for
C complexity list; KRP is complexity-1 recurrence count in master block;
C JG is number of general factors in master block.  Unpack these and report
C to screen. Complexity list is returned as a character string in LST and
c as an integer in KC.
C *** LST should get at most 15 characters, but tolerates spillover.
      CHARACTER CH*(*), LST*(*), CCF*12
      COMMON /CCF/ CCF
      KD = 26*(ICHAR(CH(1:1))-65) + (ICHAR(CH(2:2))-65)
      JF = 1 + KD/100
      NFB = 1 + MOD(KD/10,10)
      NVB = 1 + MOD(KD,10)
      K = 10*(ICHAR(CH(3:3))-48) + (ICHAR(CH(4:4))-48)
C       K is the 2-digit code for complexity levels  2.
      K = 2*K + MIN(1,KRP)  ! Expand to include complexity-1 info
      KC = 0; LL = 0
      DO L = 1,MB
        IF(MOD(K,2)>0) THEN
           KC = 10*KC + L
           LL = LL+2
           LST(LL:LL) = CHAR(48+L)
           IF(L==1 .AND. KRP>1) THEN
             J = JJF(KRP); LST(LL+1:LL+J+1) = '*'//CCF(:J); LL = LL+J+1
           END IF
C            LST lists the realized complexity levels (spaced numeric characters)
        END IF
        K = K/2
      END DO
      IF(JG>0) LST(LL+2:LL+3) = 'G'//CHAR(48+JG)
      JV = MAX(0,KRP-1)*(JF-JG)
      L = KC
20    K = MOD(L,10)
      JV = JV + KOMB(JF-JG,K)
      L = L/10
      IF(L>0) GOTO 20
      NV = JV*NVB
      NF = JF*NFB
      RETURN
      END
C
      SUBROUTINE DIST(NV,NL,NH,CV,LD)
C Entries in sym-storage array CV are presumed to have magnitudes no greater
C than unity.  CV's percent of nondiagonal entries whose first decimals round
C with point omitted to I is returned in LD(I).  NL/NH are the lowest/highest
C nonzero I.
      INTEGER LD(-10:*)
      REAL CV(*)
      LO(I,J) = J*(J-1)/2 + I
      NC = NV*(NV-1)/2
      DO I = -10,10
        LD(I) = 0
      END DO
      DO J = 2,NV
        DO I = 1,J-1
          K = NINT(10*CV(LO(I,J)))
          LD(K) = LD(K)+1
        END DO
      END DO
      DO I = -10,10
        IF(LD(I)>0) NL = I
        IF(LD(I)>0) GOTO 35
      END DO
35    DO I = 10,-10,-1
        IF(LD(I)>0) NH = I
        IF(LD(I)>0) GOTO 45
      END DO
45    CONTINUE
      DO I = -10,10
        LD(I) = NINT((100.0*LD(I))/NC)
      END DO
      IF(NL==NH) THEN
        NL = NL-1
        NH = NH+1
      END IF
      END SUBROUTINE

      SUBROUTINE JITCOV(NV,NS,CJIT,WK)  ! Tested in program TESTJIT.
C Return in CJIT a simulated sample covariance matrix for NV variables when
C sample size is NS and the population covariances are orthonormal.  The
C algorithm is by Odell & Feiveson (J. Amer. Stat. Assn., l966) with their
C ChiSq-distributed terms on the diagonal of workspace WK approximated by
C the Wilson/Hifferty formula they recommend for this.  To simulate sampling
C from a population covariance matrix CV, first generate CJIT and then
C compute A*CJIT*A' where A is a Gram-factor of CV.
C NOTE: Symmetric matrix CJIT is expressed as a symmetric-storage string of
C       its upper triangle
C ***** WARNING: JITCOV output sometimes has negative eigenvalues.
      CHARACTER CCF*12
      REAL CJIT(*), WK(*)  ! CJIT and WK must be at least length LO(NV,NV)
      COMMON /CCF/ CCF
      LO(I,J) = J*(J-1)/2 + I  ! Map upper-triangle position into sym-storage location
      IF(NS<=NV) THEN
        WRITE(6,'(" *** Error: JITCOM requires NS (now ",A,") to",
     +   " be greater than NV (now ",A,")")') CCF(:JJF(NS)),
     +   CCF(:JJF(NV))
        STOP
      END IF
      DO J = 1,LO(NV,NV)
        WK(J) = ZRAND()
      END DO
      DO J = 1,NV
        DF = FLOAT(NS-J)
        R = 2./(9.*DF)   ! For Wilson-Hifferty approximation
        WK(LO(J,J)) = DF*(1 - R + ZRAND()*SQRT(R))**3  ! Wilson-Hifferty approx.
C        WK(LO(J,J)) = WK(LO(J,J))*SQRT(2*DF) + DF ! Normality ChiSq(df) approx.
      END DO
      DO J = 1,NV
        DO I = 1,J
          IF(I==J) CJIT(LO(J,J)) = WK(LO(J,J))
          IF(I<J) CJIT(LO(I,J)) = WK(LO(I,J))*SQRT(WK(LO(I,I)))
          DO K = 1,I-1
            CJIT(LO(I,J)) = CJIT(LO(I,J)) + WK(LO(K,I))*WK(LO(K,J))
          END DO
        END DO
      END DO
      DO J = 1,LO(NV,NV)
        CJIT(J) = CJIT(J)/NS
      END DO
      END SUBROUTINE
C
      FUNCTION KOMB(K,L0)
C This returns the number of combinations of K things taken L0 at a time.
C If L0 is negative, KOMB returns the number of permutations of K things.
      L = MIN(L0,K-L0)
      IF(L0<0) L = K
      KOMB = KMB(K,L)
      IF(L0<0 .OR. K<=0) RETURN
      KOMB = KOMB/KMB(L,L)
      END FUNCTION
C
      FUNCTION KMB(K0,M)
C Return the number of ordered M-selections from K0 things.
      KMB = 1
      IF(K0<=0 .OR. M<=0 .OR. M>K0) RETURN
      K = K0
10    KMB = KMB*K
      K = K-1
      IF(K>K0-M) GOTO 10
      END FUNCTION

      SUBROUTINE GETL(KF,CH,L,N,IER)
C Find the Nth line in formatted file KF starting after omission of leading
C blanks with character string CH(:L) and position KF to read that line.
C **** Can easily extend this for more diverse searches
      CHARACTER CH*(*), WORD*80  !, CH2*2
CC      IF(N==1) CH2='st'; IF(N==2) CH2='nd'
CC      IF(N==3) CH2='rd';IF(N>3) CH2='th'
      REWIND KF; IER = 0; NN = 0
10    READ(KF,'(A)',END=50,ERR=50) WORD
      IF(LAST(WORD)<L) GOTO 10
      IF(CH(:L)/=WORD(:L)) GOTO 10
      NN = NN+1; IF(NN<N) GOTO 10
      BACKSPACE KF
      RETURN
50    IER = 1 ! Failure to find CH may set informed continuation
      END SUBROUTINE

      SUBROUTINE ISORT(N,LST)
C Sort LST integers into descending or ascending order
      INTEGER LST(*)
10    DO J = 2,N
        L = LST(J)
        DO 11 I = J-1,1,-1
         IF(LST(I)<=L) GOTO 12    ! Increasing order
C         IF(LST(I)>=L) GOTO 12    ! Decreasing order
11       LST(I+1) = LST(I)
        I = 0
12      LST(I+1) = L
      END DO
      END SUBROUTINE
C
      FUNCTION LAST(WORD)
C This left-justifies leading substring WORD(:M) of WORD, and returns its
C length as L. If the string is empty, LAST is returned as 0.
      CHARACTER WORD*(*)
      M = LEN(WORD)
      WORD(:M) = ADJUSTL(WORD(:M))
      LAST = LEN_TRIM(WORD(:ABS(M)))
      END

      SUBROUTINE LOOK(K,GET,NAME,ML,NL)
C LOOK calls DOS to delete or report files named in GET, with action set by K.
C K/2 = 0,1 initial-lists/append-lists GET in buffer file ZZZ. MOD(K,2) = 0,1
C stores/shows-to-screen the GET list. So K=0 initial-stores; K=1 initial-
C shows; K=2 append-stores; and K=3 append-shows. The last filename matching
C the GET template is returned in WORD with its length M.  NAME returns the
C list of template matches, the number of which is NL, and prints the numbered
C list to screen.  ML is max NL allowed by array allocation.
C ***** This variant uses negative input on NL to suppress screen write
      LOGICAL PRM, QY
      CHARACTER*12 GET*(*), NAME(*), WORD*40, WD*40, TMP
      CHARACTER E   ! <<<  Special characters as needed
      NOSHO = NL
      WD = GET
      M = LAST(WD)
      LL = 1
5     LL = LL+1; IF(LL<M) GOTO 5
      WD(M+2:M+LL+1) = WD(:LL); WD(40:40) = ' '  ! Shd now always have LL = M
      DO I = M+2,M+LL+1
        IF(WD(I:I)=='*' .OR. WD(I:I)=='?' ) THEN
          WD(40:40) = '!'; WD(I:I) = '!'
        END IF
      END DO
      INQUIRE(FILE=WD(M+2:M+LL+1),EXIST=QY)
      IF(.NOT.QY) OPEN(19,FILE=WD(M+2:M+LL+1)) ! Precludes no-match error message
      IF(K/2==0) CALL SYSTEM('dir '//WD(:M)//' >ZZZ')
      IF(K/2>0) CALL SYSTEM('dir '//WD(:M)//'>>ZZZ')
      IF(.NOT.QY) CLOSE(19,STATUS='DELETE')
      IF(MOD(K,2)==0) RETURN
      OPEN(4,FILE='ZZZ')
      NL = 0
10    READ(4,'(A)',END=50) WORD
      IF(WORD(10:12)=='COV') GOTO 15
      IF(WORD(25:26)==' 0' .OR. WORD(16:16)=='<') GOTO 10  ! No directory names
      E = WORD(:1); IF(E==' ' .OR. E=='.') GOTO 10
      J = ICHAR(WORD(6:6))
      IF(E=='P'.AND.(WORD(8:8)/=' '.OR.J<48.OR.J>57)) GOTO 10  ! Just 7 chars for GENPAT output
      J = ICHAR(WORD(2:2)); IF(J<48 .OR. J>57) GOTO 10
15    L = LAST(WORD(:14))
      DO I = 1,L; IF(WORD(I:I)=='!') GOTO 10; END DO
19    L = 0
20    L = L+1; IF(WORD(L+1:L+1)/=' ' .AND. L<8) GOTO 20
      IF(WORD(10:10)/=' ') WORD(L+1:L+4) = '.'//WORD(10:12)
      DO I = 1,NL  !  Eliminate possible duplications
        IF(NAME(I)(:L+4)==WORD(:L+4) .AND. NAME(I)(L+5:L+5)==' ')
     +    GOTO 10
      END DO
      NL = NL+1
      NAME(NL) = WORD(:L+4)//'     '
      M = L+4
      IF(NL<ML) GOTO 10
50    IF(NL==0) M = 0
      CLOSE(4,STATUS='DELETE')
      DO J = 2,NL
        TMP = NAME(J)
        DO I = J-1,1,-1
          IF(.NOT.PRM(NAME(I),TMP)) GOTO 55    ! Increasing order
          NAME(I+1) = NAME(I)
        END DO
        I = 0
55      NAME(I+1) = TMP
      END DO
      IF(NOSHO.LT.0) RETURN
      IF(NL>0) WRITE(6,'(20(:/4(I5,". ",A,:)))') (I,NAME(I),I=1,NL)
      IF(NL>0) WORD(:12) = NAME(NL)
      RETURN
      END

      FUNCTION PRM(NAM1,NAM2)
C Consider both base and extension when flagging which filename comes first
C in alphanumeric listing.  PRM=t says that NAM2 comes before NAM1.
      LOGICAL PRM
      CHARACTER NAM1*(*), NAM2*(*)
      PRM = .FALSE.
      L1 = LAST(NAM1); L2 = LAST(NAM2)
      N1 = L1+1; N2 = L2+1  ! Find dot positions, if any
10    N1 = N1-1; IF(NAM1(N1:N1)/='.' .AND. N1>1) GOTO 10
20    N2 = N2-1; IF(NAM2(N2:N2)/='.' .AND. N2>1) GOTO 20
      IF(N1==1) N1=L1+1; IF(N2==1) N2=L2+1  ! Where dot would be were there one
      JJ = KPRM(NAM1(:N1-1),NAM2(:N2-1))
      IF(JJ==-1) PRM = .TRUE.; IF(JJ/=0) RETURN  ! JJ=0 if basenames are same
      IF(JJ==0 .AND. (N1>L1 .OR. N2>L2)) THEN  ! An extension is blank
        IF(N2>L2) PRM = .TRUE.; RETURN
      END IF
      JJ = KPRM(NAM1(N1:L1),NAM2(N2:L2))  ! Include extension dot
      PRM = .FALSE.; IF(JJ==-1) PRM = .TRUE.
      END FUNCTION

      FUNCTION KPRM(WRD1,WRD2)
C Return value 0 if WRD1=WRD2; otherwise  -1 or +1 according to whether WRD2
C comes before or after WRD1 in alphanumeric sequence.
      INTEGER KPRM
      CHARACTER WRD1*(*), WRD2*(*)
      L1 = LAST(WRD1); L2 = LAST(WRD2)
      KPRM = 0; IF(WRD1(:L1)==WRD2(:L2)) RETURN
      N1 = L1+1; N2 = L2+1    ! Find position preceding terminal number string
10    N1 = N1-1; N = ICHAR(WRD1(N1:N1))
         IF(N>47.AND.N<58 .AND. N1>1) GOTO 10
20    N2 = N2-1; N = ICHAR(WRD2(N2:N2))
         IF(N>47.AND.N<58 .AND. N2>1) GOTO 20
      IF(LLT(WRD1(:N1),WRD2(:N2))) KPRM = 1
      IF(LLT(WRD2(:N2),WRD1(:N1))) KPRM = -1
      IF(WRD1(:N1)/=WRD2(:N2)) RETURN
      IF(N1==L1 .OR. N2==L2) THEN  ! A mumber terminus is blank
        KPRM = 1; IF(N2==L2) KPRM = -1; RETURN
      END IF
      READ(WRD1(N1+1:L1),*) K1; READ(WRD2(N2+1:L2),*) K2
      KPRM = 1; IF(K2<K1) KPRM = -1
      END FUNCTION

      SUBROUTINE MAKCOV(NF,CFF,KOV,LSET,LIST,T,LDT,R,F3,NAM) ! NAM is main program name
      LOGICAL QC, QP, QSAV
      CHARACTER*12 CF, F2, F3, NAME(40), WORD*40, WRD(NF)*3, CLN*8,
     +  EXT*3, NAM*(*)
      INTEGER KOV(NF*(NF+1)/2), LSET(*), LIST(*)
      REAL LOW, CFF(NF*(NF+1)/2), T(LDT,*), R(*)
      COMMON /CF/ CF
      LO(I,J) = J*(J-1)/2 + I
      LOC(I,J) = LO(MIN(I,J),MAX(I,J))
      NFF = LO(NF,NF)
c >>>> You can enter exogenous-factor correlations Cff in one of three ways:
c      1) Retrieve and perhaps modify a Cff for xx factors already stored.
c      2) Create a new Cff randomized under coarse control of obliquity.
c      3) Make Cff orthogonal except for boundedly randomized obliquities
c           at the specific locations (factor pairs) in Cff you want.
CCC      WRITE(6,'(" >>>> You can enter exogenous-factor correlations C",
CCC     +  "ff in one of three ways:")')
101   WRITE(6,'(6X,"1) Retrieve and perhaps modify a Cff for",A," fac",
     +  "tors already stored."/6X,"2) Create a new Cff randomized und",
     +  "er coarse control of obliquity."/6X,"3) make Cff orthogonal ",
     +  "except for boundedly randomized obliquities"/12X,"at the sp",
     +  "specific locations (factor pairs) in Cff you want."/)')
     +  CF(:JJF(NF))
      NN = -1  ! Suppress screen output from LOOK
      CALL LOOK(1,'*.C'//CF(:JJF(NF)),NAME,ML,NN)
      IF(NN==0) THEN  ! IF(MM==0) THEN
        WRITE(6,'(" No order-",A," Cff files have been archived here,",
     +    "  so your exo-correlations"/" must be created internall",
     +    "y.  Hit RETURN if prepared to continue, or enter"/" anyth",
     +    "ing to break off for now.")') CF(:JJF(NF))
      CALL SCANB(J,0,'B',5,WORD,0); IF(J/=0) STOP
      END IF
104   WRITE(6,'(/" The ",A,"-factor Cff files available here are")')
     +  CF(:JJF(NF)); L = 1
      WRITE(6,'(20(:/4(I5,". ",A,:)))') (I,NAME(I),I=1,NN)
106   LF = LAST(NAME(L))
      WRITE(6,'(/" To inspect the entries in ",A,", hit RETURN.  O",
     +  "therwise enter the index"/" of another, or any letter to ",
     +  "create more factor correlations."/)') NAME(L)(:LF)
      CALL SCANB(J,0,'I',5,WORD,1)
      IF(J<0) GOTO 130
      IF(J>0) THEN; READ(2,*) L; L=MAX(1,MIN(MM,L)); GOTO 106; END IF
      CALL START(4,NAME(L)); READ(4,*) NF
      READ(4,*) (CFF(J),J=1,NFF); AV = 0.; S = 0.
      DO J = 1,NF; V=CFF(LO(J,J)); AV=AV+V; S=S+V**2; END DO
      IF(N*S-AV**2 < .01*NF) THEN
        AV = AV/NF; DO J = 1,NFF; CFF(J) = CFF(J)/AV; END DO
      ELSE
        DO J = 1,NF; R(J) = SQRT(CFF(LO(J,J)))
          DO I = 1,J-1
            CFF(LO(I,J)) = CFF(LO(I,J))/(R(I)*R(J))
          END DO
        END DO
        DO J = 1,NF; CFF(LO(J,J)) = 1.; END DO
        WRITE(6,'(" Note: Unstandardized covariances in ",A," are co",
     +    "nverted to correlations.")') NAME(NN)(:LF)
      END IF
111   M=100; KD=2  !; IF(NF<15) THEN; M=1000; KD=3; END IF  ! QC=t flags branchback
      F3 = NAME(L); IF(QC) THEN; F3 = 'Cff'; LF = 3; END IF   ! from new Cff
      WORD = '(5(2X,5(1X,A3)))'  ! ; IF(KD==3) WORD = '(5(2X,5(1X,A4)))'
      WRITE(6,'(/" Correlations in ",A," rounded to ",A," decimals ",
     +  "with point omitted:")') F3(:LF), CF(:JJF(KD))
      DO J = 1,NF
        WRITE(6,WORD(:16)) (CLN(M*CFF(LO(I,J)),KD+1,10),I=1,MIN(J,17))
        IF(MOD(J,20)==0.AND.J<NF-2) CALL WAIT(0)
        IF(MOD(J,5)==0 .AND. J<NF) WRITE(6,'()')
      END DO  ! MM is the number of external COV-files found
      IF(QC) THEN
        WRITE(6,'(/" Hit RETURN if satisfied with this Cff, or enter ",
     +    "anything to try again.")')
        CALL SCANB(J,0,'B',5,WORD,1)
        IF(J==0) GOTO 175; GOTO 130
      END IF
      IF(.NOT.QC) WRITE(6,'(/3X,"Hit RETURN to accept these factor ",
     +  "covariances after maybe a final tweak.")')
      IF(MM==1) WRITE(6,'(3X,"Otherwise enter any letter to generate",
     +  " the factor covariances semi-randomly.")')
      IF(MM>1) WRITE(6,'(3X,"Otherwise, enter any letter to choose an",
     +  "other archived Cff or any number to"/3X,"generate the factor",
     +  " covariances semi-randomly.")')
      CALL SCANB(J,0,'R',5,WORD,1)
      IF(MM==1 .AND. J/=0) GOTO 130
      IF(MM>1) THEN; IF(J<0) GOTO 104; IF(J>0) GOTO 130; END IF
      GOTO 175  ! Goto tweak

c To create a semi-random correlation matrix Cff for XX factors, you must enter
c
c   1) an integer NC and, if NC>0, an interval <BL,BH> within limits -1 to +1
c      such that NC of the xxx distinct proper correlations are to be assigned
c      random values provisionally between BL and BH while the rest are zero.
c      Enter NC with a minus sign if you want to specify the nonzero locations;
c      otherwise, obliquities will be selected randomly.  Unless you make Cff
c      fully orthogonal by stipulating NC=0, you will also be invited to choose
c
c   2) a value LOW between 0 and 1 to be the smallest eigenvalue of the
c      created correlation matrix.
c
c      Given these parameters, GENSCOR first generates an order-NF symmetric
c   matrix Q with unit diagonal and zeros or random values between BL and BH
c   elsewhere.  It then tweaks Q into a correlation matrix Cff having the same
c   off-diagonal correlation ratios and zeros (orthogonalities) as Q while
c   making LOW its smallest eigenvalue.

130   CALL SYSTEM('cls')
      WRITE(6,'(/" To create a semi-random correlation matrix Cff f",
     +  "or ",A," factors, you must enter"//3X,"1) an integer NC an",
     +  "d, if NC>0, an interval <BL,BH> within limits -1 to +1"/6X,
     +  "such that NC of the ",A," distinct proper correlations are ",
     +  "to be assigned"/6X,"random values provisionally between BL ",
     +  "and BH while the rest are zero."/6X,"Enter NC with a minus ",
     +  "sign if you want to specify the nonzero locations;"/6X,"oth",
     +  "erwise, obliquities will be selected randomly.  Unless you ",
     +  "make Cff"/6X,"fully orthgonal by stipulating NC=0, you will",
     +  " also be invited to choose"//3X,"2) a value LOW between 0 a",
     +  "nd 1 to be the smallest eigenvalue of the"/6X,"created corr",
     +  "elation matrix.")') CF(:JJF(NF)),  CF(:JJF(NFF-NF));
     +  NT = NFF-NF; NC = 0; MM = 0
      WRITE(6,'(/6X,"Given these parameters, ",A," first generates an",
     +  " order-NF symmetric"/3X,"matrix Q with unit diagonal and zer",
     +  "os or random values between BL and BH"/3X,"elsewhere.  It th",
     +  "en tweaks Q into a correlation matrix Cff having the same"/3X,
     +  "off-diagonal correlation ratios and zeros (orthogonalities) ",
     +  "as Q while"/3X,"making LOW its smallest eigenvalue.")') NAM
133    WRITE(6,'(/3X,"The flagged count of distinct factor correlati",
     +  "ons to make oblique (nonzero)"/3X,"is now set at ",A," out of",
     +  " a maximum ",A,".  Hit RETURN if OK, or enter preferred"/3X,
     +  "number WITH NEGATIVE SIGN if you want to choose their locati",
     +  "ons in Cff."/3X,"(Your choice of NC>0 needn''t be exactly co",
     +  "rrect at this point.)"/)') CF(:JJF(SIGN(NC,MM))), CF(:JJF(NT))
      CALL SCANB(J,1,'I',5,WORD,1)
      IF(J==0) GOTO 135; IF(J<0) GOTO 133
      READ(2,*) M; NC = MIN(ABS(M),NT); IF(NC==NT) M = 0; GOTO 133
135   CFF = 0.; DO J = 1,NF; CFF(LO(J,J)) = 1.; END DO   ! Orthnorm Cff default
      QP = .FALSE.; IF(NC<0) QP = .TRUE.
      IF(NC*M>=0) THEN  ! NC is extreme or flagged for ad lib obliquities
        IF(NC==0) WRITE(6,'(" Hit RETURN to confirm that you want on",
     +    "ly orthogonal factors.")')
        IF(M==0) WRITE(6,'(" Hit RETURN to confirm that you don''t re",
     +    "quire any factor orthogonalities.")')
        IF(M>0) WRITE(6,'(" Hit RETURN to confirm that you don''t car",
     +    "e where the obliquities go in Cff.")')
        WRITE(6,'(" Otherwise, enter anything to reconsider your NC ",
     +    "setting.")')
        CALL SCANB(J,0,'B',5,WORD,1)
        IF(J/=0) GOTO 133
      END IF
      IF(NC==0) RETURN     ! Orthogonal Cff already set as default
cc        DO J = 1,NF; TWIST(J,6) = 1.; END DO  ! Orthogonal factors
cc                      ^ only in GENSCOR
      IF(NC==NT) GOTO 160    ! Start randomization
      IF(.NOT.QP) THEN ! Obliquities are to be located randomly
        CALL RANSEL(NC,NT,LIST,LSET,0) ! LIST(i) > 0 iff ith factor pair is selected
C       Sampling of factor-pair indices is without replacement
        N = 0; KOV = 0
        DO J = 2,NF; DO I = 1,J-1
           N = N+1; IF(LSET(N)>0) KOV(LO(I,J)) = 1
        END DO; END DO
        GOTO 160  ! Location of nonzeros are randomized
      END IF

C  Enter indices of factor pairs to be given nonzero correlations according
C  to the following layout which you can put on paper or just imagine:
C  Creating Cff here requires filling empty cells in just the off-diagonal
C  lower triangle of a xx by xx symmetric matrix.  (Its diagonal is pre-set
C  while its upper triangle will mirror its to-be-defined lower triangle.)
C  Make a checkmark, or imagine one, in its lower-triangle cells that you
C  want to be nonzero.  Then for each row I of this triangle from 2 to xx,
C  enter on one line of computer input this row's index followed by the column
C  indices J of its cells you have checked.  No punctuation is needed except
c  spaces, and entering J in row I when your schema calls for I in row J also
c  works.  Rows can be entered in any order and ones containing no checks
c  should be omitted.  To revise a row already installed, enter its row-index
C  I with minus sign if you want to redo it completely; otherwise, for partial
C  revision, start with positive I followed by unsigned Js that will be added
C  to this row's prior entries or negative Js that will be deleted from them.

141   WRITE(6,'("  Enter indices of factor pairs to be given nonzero ",
     +  "correlations according"/"  to the following layout which you",
     +  " can put on paper or just imagine:"/"  Creating Cff here req",
     +  "uires filling empty cells in just the off-diagonal"/"  lower",
     +  "  triangle of a ",A," by ",A," symmetric matrix. (Its diagon",
     +  "al is pre-set"/"  while its upper triangle will mirror its t",
     +  "o-be-defined lower triangle.)"/"  Make a checkmark, or imagi",
     +  "ne one, in its lower-triangle cells that you"/"  want to be ",
     +  "nonzero.  Then for each row I of this triangle from 2 to ",A,
     +  ","/"  enter on one line of computer input this row''s index",
     +  " followed by the column"/"  indices J of its cells you have",
     +  " checked.  No punctuation is needed except"/"  spaces, and ",
     +  "entering J in row I when your schema calls for I in row J a",
     +  "lso"/"  works.  Rows can be entered in any order and ones ",
     +  "containing no checks"/"  should be omitted.  To revise a r",
     "  "ow already installed, enter its row-index"/"  I with minus ",
     +  "sign if you want to redo it completely; otherwise, for part",
     +  "ial"/"  revision, start with positive I followed by unsigne",
     +  "d Js that will be added"/"  to this row''s prior entries or",
     +  " negative Js that will be deleted from them.")')
     +  (CF(:JJF(NF)),I=1,3)
      CALL WAIT(0); KOV = 0; NN = 0
      WORD = 'reconsider your NC choice.'  !; L = 26
146   IF(NN>0) WORD = 'inspect your entries.     '
      WRITE(6,'("  Enter a row index I followed by indices J, lower ",
     +  "than I, of factors to have"/"  some correlation with factor",
     +  " I.  Or hit RETURN to ",A/)') WORD(:26)
      CALL SCANB(J,0,'I',5,WORD,0); IF(J<0) GOTO 146
      JJ = J-1; IF(NN==0.AND.J==0) GOTO 130; IF(J==0) GOTO 151
148   READ(2,*) IR, (LIST(J),J=1,JJ); I = ABS(IR); IF(I<2) GOTO 146
      IF(I>NF) THEN
        WRITE(6,'(" Your order-",A," Cff doesn''t have a row ",A,
     +    ".  Try again.")') CF(:JJF(NF)), CF(:JJF(I)); GOTO 146
      END IF
      IF(IR<0) THEN   ! First wipe row I
        DO K = 1,I-1; KOV(LO(K,I)) = 0; END DO
      END IF
      DO K = 1,J-1
        L = LIST(K); M = ABS(L); IF(M==I) CYCLE
        KOV(LOC(I,M)) = MAX(0,MIN(1,L))
      END DO
      NN = 0  ! Count cell checks
      DO J = 2,NF; DO I = 1,J-1
        NN = NN + KOV(LO(I,J))
      END DO; END DO          ! Rem: NC>0 when KOV is created
      WRITE(6,'(" Enter another row of factor indices identifying no",
     +  "nzero correlations, or"/" hit RETURN to see the layout of t",
     +  "hose entered so far.  Their count, ",A,",")') CF(:JJF(NN))
      WORD(:5) = ' less'; IF(NN>NC) WORD(:5) = ' more'
      IF(NN/=ABS(NC)) WRITE(6,'(" is ",2A," than your initially decl",
     +  "ared target number, but that''s OK"/" if it''s what you wan",
     +  "t.")') CF(:JJF(ABS(NC-NN))), WORD(:5)
      IF(NN==NC) WRITE(6,'(" matches the target number you initially",
     +  " declared.")')
      CALL SCANB(J,0,'I',5,WORD,0); IF(J>0) GOTO 148
151   WRITE(6,'(" Here is the schema now entered for generating your",
     +  " Cff''s lower triangle:")')
      WRITE(6,'(" Unit variances are on the diagonal; correlations t",
     +  "o be randomized within"/" limits are shown in place by thei",
     +  "r column indices; and ""."" marks empty"/" cells that will ",
     +  "become zero unless you fill in more.")')
      DO I = 1,NF; KOV(LO(I,I)) = 1; END DO
      WRITE(6,'("  1:  1")')
      DO L = 2,NF
        WRD = '.'
        DO J = 1,L; IF(KOV(LO(J,L))>0) WRD(J) = CLN(1.*J,3,10); END DO
        WRITE(6,'(I3,":",25A3,4(:/6X,24A3))') L, (WRD(J),J=1,L-1),' 1'
        IF((NF>17.AND.NF<26.AND.MOD(L,15)==0 .OR. L>25.AND.L<50.AND.MOD
     +    (L,10)==0 .OR. L>49).AND.MOD(L,5)==0 .OR. L==NF) CALL WAIT(0)
      END DO
153   WRITE(6,'(" Hit RETURN if this layout is correct and complete.",
     +  " Otherwise, continue with"/" additions or revisions as befo",
     +  "re.")')
      CALL SCANB(J,2,'B',5,WORD,0)
      IF(J<0) GOTO 153; IF(J>0) GOTO 146
      NC = NN
160   WRITE(6,'(/" Ready to randomize oblique factor correlations be",
     +  "tween limits now set at"/" <",A3,",",A4,">.  Hit RETURN if ",
     +  "OK, or enter new pair of limits."/)') CLN(BL,3,2),CLN(BH,4,2)
      CALL SCANB(J,2,'RR',5,WORD,1)
      IF(J==0) GOTO 165
      IF(J<0) GOTO 160
      READ(2,*) X, Y
163   IF(ABS(X)>1. .OR. ABS(Y)>1.) THEN
        IF(ABS(X)>1.) X = X/10
        IF(ABS(Y)>1.) Y = Y/10
        GOTO 163
      END IF
      BL = MAX(-1.,MIN(1.,X,Y))
      BH = MIN(1.,MAX(-1.,X,Y))
      GOTO 160
165    IF(ABS(BH*BL)<.01) THEN
        WRITE(6,'(" Hit RETURN to confirm your call for all-orthogona",
     +    "l factors.  Otherwise,"/" enter anything to reconsider.")')
        CALL SCANB(J,2,'B',5,WORD,0); IF(J/=0) GOTO 160
        CFF = 0.; DO I = 1,NF; CFF(LO(I,I)) = 1.; R(I) = 1.; END DO
        LOW = 1.;  GOTO 146                 !     ^ Needed??
      END IF
C Generate raw Cff
      BD = BH-BL; QSAV = .TRUE.; NN = 0; MM = 50
      IF(NC>0 .AND. NC<NT) QSAV = .TRUE.
168    DO J = 2,NF; DO I = 1,J-1
         IF(KOV(LO(I,J))>0) CFF(LO(I,J)) = BL + BD*RANDY()
      END DO; END DO
      DO I = 1,NF; CFF(LO(I,J)) = 1.; END DO
      CALL EIGS(NF,NF,CFF,NFF,R,T,NV,0,IER,6); LOW = R(NF)
      IF(LOW>.001) GOTO 180  ! Precaution below surely never needed
      NN = NN+1; IF(NN<MM) GOTO 168
      WRITE(6,'(I6," randomly generated Cff satisfying your constrain",
     +  "ts have all been",/4X,"ill-conditioned.  Increase your range",
     +  " of admissible randomizations.")') MM; QSAV=.FALSE.; GOTO 160
C Start tweak by solving for the initial Q-matrix eigenstructure
175   CALL EIGS(NF,NF,CFF,NFF,R,T,NV,0,IER,6); LOW = R(NF)  ! Eigvecs not needed
      WRITE(6,'(3X,"Cff TWEAK:")'); QC = .TRUE.
180    WRITE(6,'(3X,"The smallest eigenvalue LOW of your factor-correl",
     +  "ation matrix is now ",A4,"."/3X,"By a multiplicative tweak t",
     +  "hat leaves the correlation ratios unchanged,"/3X,"you can de",
     +  "crease these correlations by making LOW larger, or increase ",
     +  "them"/3X,"by decreasing LOW.  Admissible limits are 1  LOW ",
     +  " 0: All the factors are"/3X,"orthogonalized by LOW = 1, whe",
     +  "reas LOW = 0 gives them a linear dependency."/)') CLN(LOW,4,3)

C   COV TWEAK:
C   The smallest eigenvalue LOW of your factor-correlation matrix is now xxxx.
c   By a multiplicative tweak that leaves the correlation ratios unchanged,
C   you can decrease these correlations by making LOW larger, or increase them
C   by decreasing Low.  Admissible limits are 1  LOW  0: All the factors are
C   orthogonalized by LOW = 1, whereas LOW = 0 gives them a linear dependency.

      WRITE(6,'(5X,"If you want to adjust your factor correlations ",
     +  "this way, enter a new"/5X,"choice of Low.  Otherwise, hit ",
     +  "RETURN to accept Cff with LOW = ",A4,"."/)') CLN(LOW,4,3)
      CALL SCANB(J,1,'R',5,WORD,1)
      IF(J==0) RETURN; IF(J<0) GOTO 180
      READ(2,*) LOW; LOW = ABS(LOW)
182    IF(LOW>1.) THEN; LOW = LOW/10; GOTO 182; END IF
      WRITE(6,'(5X,"Hit RETURN to confirm choice of ",A4," for Cf",
     +  "f''s smallest eigenvalue.")')  CLN(LOW,4,3)
      IF(LOW<.01) WRITE(6,'(5X,"(WARNING: LOW this small is unwise ",
     +  "unless you are testing limits.)")')
      WRITE(6,'(5X,"Otherwise, enter anything to choose again.")')
      CALL SCANB(J,1,'R',5,WORD,1); IF(J/=0) GOTO 180

C Theory on 1-parameter tweaking of any unit-diagonal C to fix its jth root:
C For any scalar multiplier a, Q = a(C-I)+I rescales the off-diagonal C-terms
C while leaving unities on its diagonal:  If C = TDT' is C's eigenstructure,
C Q's is Q = T(aD - a+1)T'. So if d is the jth root of C, the jth root d' of
C Q is d' = ad - a+1 = 1+a(d-1) which equals stipulated r iff a = (1-r)/(1-d).
      Z = (1-LOW)/(1-R(NF))
      DO I = 1,NF-1
        DO J = I+1,NF
          CFF(LO(I,J)) = Z*CFF(LO(I,J))
        END DO
      END DO
      DO I = 1,NF
        R(I) = 1 + Z*(R(I)-1)
      END DO
      WRITE(6,'(" Eigenvalues of your tweaked Cff are",
     +  20(:/2(4X,5A6)))') (CLN(R(I),6,2),I=1,NF)

cc      OPEN(15,FORM='UNFORMATTED') ! Save for later recovery in case roots of
cc      WRITE(15) (R(J),J=1,NF)     ! Cff are wanted (probably shd delete)

      IF(QC) THEN    ! Invite display of new COV
        WRITE(6,'(" Enter anything to inspect the finished Cff; oth",
     +    "erwise, hit RETURN to move on.")')
        READ(5,*) WORD(:1); IF(WORD(:1)/=' ') GOTO 111
      END IF
      IF(.NOT.QSAV) RETURN
      F3 = 'SIMCFF'; EXT = 'C'//CF(:JJF(NF))
187   CALL NAMM(F3,EXT,K); LF3 = LAST(F3)
      WRITE(6,'("   To save the Cff just created in an ASCII file na",
     +  "med ",A,", hit RETURN."/"   Otherwise enter a letter string",
     +  " to replace """,A,""", or any numeral to"/"   waive saving ",
     +   "this."/)')  F3(:LF3), F3(:K)
      CALL SCANB(J,1,'R',5,WORD,12)
      IF(NN==0) GOTO 190  ! Blank entry, so proceed with save
cc      READ(5,'(A)') WORD  ! SCANB has already read into WORD
      NN = LAST(WORD); CALL CAP(WORD,NN)
      DO I = 1,MIN(NN,7); M = ICHAR(WORD(I:I))
        IF(M<65 .OR. M>90) RETURN
      F3 = WORD(:I)
      END DO
      GOTO 187
190   OPEN(7,FILE=F3)   ! ************  Write Cff to file
      WRITE(7,'(A,": Simulation correlations for ",A," factors.  Un",
     +  "less modified by"/" unprogrammed editing, ",A," out of ",A,
     +  " are random over interval [",A4," - ",A4,"]")') F3(:LF3),
     +  CF(:JJF(NC)), CF(:JJF(NT)), CLN(BL,4,3), CLN(BH,4,3)
      IF(QP) WRITE(7,'(" while the rest are are zero at specified ",
     +  "locations.")')
      IF(.NOT.QP) WRITE(7,'(" while the rest are are zero at random ",
     +  "locations.")')
      WRITE(7,'(1X,A)') CF(:JJF(NF))
      DO J = 1,NF
        WRITE(7,'(20I5,4(:/20I5))')  (NINT(1000*CFF(LO(I,J))),I=1,J)
      END DO
C   Ŀ
C    If you plan to test how variation in Cff affects your simulation     
C    results, you may find it convenient at this time to create and store 
C    additional alternatives from which you can then take your pick on    
C    this or later runs.  To retrieve an older stored Cff for xx factors  
C    or to build another, enter any letter.  Otherwise, hit RETURN to     
C    finish production of simulated data covariances using raw pattern    
C    xxxxxxxx and the factor correlations in xxxxxxxxxxxx now loaded.     
C   

      WRITE(6,'(3X,"",70(""),"")'); WORD = ' '
      WRITE(6,'(3X," If you plan to test how variation in Cff affect",
     +  "s your simulation     "/3X," results, you may find it con",
     +  "venient at this time to create and store "/3X," additional",
     +  " alternatives from which you can then take your pick on    "/
     +  3X," this or later runs.  To retrieve an older stored Cff fo",
     +  "r ",A," factors",A,""/3X," or to build another, enter any ",
     +  "letter.  Otherwise, hit RETURN to     "/3X," finish produc",
     "  "tion of simulated data covariances using raw pattern    "/3X,
     +  " ",A," and the factor correlations in ",A," now loaded.",A,
     +  "")') CF(:JJF(NF)), (WORD(:1+MIN(1,NF/10))), F3(:LF3),
     +  WORD(:17-LF3)
      WRITE(6,'(3X,"",70(""),"")')
      CALL SCANB(J,0,'R',5,WORD,0)
      IF(J==0) RETURN
      IF(J<0) THEN
        WRITE(6,'(" >>>> As before, you can provide factor correlati",
     +  "ons in one of three ways:")'); GOTO 101
      END IF
      STOP  ! Undocumented exit
      END SUBROUTINE

C
      SUBROUTINE RANSEL(NS,NT,LSET,LTAG,METH)
C This randomly selects NS of integers 1 to NT, and returns the selected
C sequence in array LSET. Selection is WITHOUT replacement if METH = 0; other-
C wise, WITH replacement.  Under METH = 0, LTAG(i) = J (otherwise 0) if item i
C has been selected and has index position J when the selected items are put
C into ascending order.  Under METH > 0, LTAG(i) is the number of times item i
C has been selected while MAX is the maximum number of times any one number
C has been selected.  The random generator must have been seeded previously.
      INTEGER LSET(*), LTAG(*)
      DO I = 1,NT
        LTAG(I) = 0
        LSET(I) = I
      END DO
      IF(METH>0) GOTO 50
C Make selection WITHOUT replacement
      IF(NT<NS) THEN
        WRITE(6,'(/" *** ERROR: The range of integers sent to RAN",
     +   "SEL is smaller than the selection count.")')
        STOP
      END IF
      DO I = 0,NS-1
        I1 = I+1
        K = MIN(NT,I1+INT((NT-I)*RANDY()))
        L = LSET(K)
        LTAG(L) = K
        LSET(K) = LSET(I1)
        LSET(I1) = L
      END DO
      K = 0
      DO I = 1,NT
       IF(LTAG(I)>0) K = K+1
       IF(LTAG(I)>0) LTAG(I) = K
      END DO
      RETURN
C Make selection WITH replacement
50    DO I = 1,NS
        K = MIN(NT,1+INT(NT*RANDY()))
        LSET(I) = K
        LTAG(K) = LTAG(K)+1
      END DO
      METH = 0
      DO I = 1,NT
        METH = MAX(METH,LTAG(I))
      END DO
      END SUBROUTINE
C
      SUBROUTINE SCANB(NL,NS,SEQ,KFILE,CH,KK)
C     Copyright (c) 1989 by W. W. Rozeboom.   All rights reserved.
C This reads a string in I/O unit KFILE, cleans it for list-directed reading
C of the numbers therein, and checks whether it contains NS integers/reals in
C the sequence of Is and Rs received in SEQ if NS>0, or, if NS<1, whether
C all its numbers are of the first I/R kind listed in SEQ. (Integers are
C accepted also as reals.) NL returns 0 if the input string is blank, -1 if
C this contains only non-numeric characters, -2 if the cleaned number
C string returned in File 2 is non-null but does not match SEQ, and
C gives the total count of numbers in the returned string otherwise.
C ***** If SEQ is "B", NL returns 0 if the input line is blank, and
C       returns -1 otherwise.
C ***** In SCANB, CH returns the KFILE string's first ABS(KK)-term sequence
C       while removing this from the string scanned for numbers if KK < 0.
C       No continuation lines are allowed.
      CHARACTER AA, SEQ*(*), WA*240, WB*240, CH*(*)
      CH = '           '
      READ(KFILE,'(A)',END=999) WA
      NL = LAST(WA)
      IF(NL==0) RETURN
      IF(SEQ(1:1)=='B') NL = -1
      IF(SEQ(1:1)=='B') RETURN
      WB(NL+1:NL+1) = ' '
      IF(KK/=0) THEN    ! Otherwise, no special treatment of start string
        K = ABS(KK); CH = WA(:K)  ! Copy
        IF(KK<0) WA(:K) = ' '     ! Also wipe
      END IF
      DO I = 1,NL
        WB(I:I) = ' '
        IF(WA(I:I)<CHAR(48) .OR. WA(I:I)>CHAR(57)) CYCLE
        WB(I:I) = WA(I:I)
        WA(I:I) = '0'
      END DO
      IP = 0
      IF(WA(:3)=='-.0') WB(:1) = '-'
      IF(WA(:2)=='-0') WB(:1) = '-'
      IF(WA(:2)=='.0') WB(:1) = '.'
      IF(WA(:2)=='.0') IP = 1
      DO I = 2,NL
        IF(WA(I:I)=='-') THEN
          IF(WB(I-1:I-1)==' ' .AND. (WA(I+1:I+1)=='.'.OR.WA(I+1:I+1)
     +      =='0')) WB(I:I) = '-'
          IP = 0
        ELSE IF (WA(I:I)=='.') THEN
          IF((WA(I-1:I-1)=='0'.OR.WA(I+1:I+1)=='0') .AND. IP==0)
     +      WB(I:I) = '.'
          IF(WB(I:I) == '.') IP = 1
        ELSE IF (WA(I:I) /= '0') THEN
          IP = 0
        END IF
      END DO
C Identify Integer/Real sequence in WB and put in start of WA
      AA = 'I'
      IF(WB(1:1)=='.') AA = 'R'
      NN = 0
      DO I = 2,NL+1
        IF(WB(I:I)==' ' .AND. WB(I-1:I-1)/=' ') THEN
          NN = NN + 1
          WA(NN:NN) = AA
          AA = 'I'
        ELSE IF (WB(I:I)=='.') THEN
          AA = 'R'
        END IF
      END DO
      IF(NN==0) NL = -1
      IF(NN==0) RETURN
      AA = '+'
      IF(NS<=0 .AND. SEQ(1:1)=='R') GOTO 60
      IF(NS<=0) GOTO 50
      IF(NN<NS) GOTO 57
      DO I = 1,NS
        IF(SEQ(I:I)=='I' .AND. WA(I:I)/='I') AA = '0'
      END DO
      GOTO 55
50    DO I = 1,NN
        IF(WA(I:I)/=SEQ(1:1)) AA = '0'
      END DO
55    IF(AA=='+') GOTO 60
57    NL = -2
      IF(NL<=-2) WRITE(6,'(/" Your input does not contain the ",
     +  "number sequence requested. Try again.")')
      RETURN
60    REWIND 2
      WRITE(2,'(A)') WB(:NL)
      NL = NN
      REWIND 2
      RETURN
999   NL = 999
      END SUBROUTINE
C
      SUBROUTINE SHOW(Q,NV,C)
      REAL Q(*), C(NV,*)
      LO(I,J) = J*(J-1)/2 + I
      ID(I,J) = I*(2*NV-I+1)/2 +J+1
      DO I = 1,NV
        DO J = I,NV
          C(I,J) = Q(LO(I+1,J+1)); C(J,I) = C(I,J)
        END DO
      END DO
      DO I = 1,NV
        WRITE(6,'(2X,20 F6.2)') (C(I,J),J=1,NV)
      END DO
      END SUBROUTINE
C
      SUBROUTINE SHOWP(NF,JF,B,LDB,KF)
C Write frame-parameter specifications to file KF
      CHARACTER CLN*8, CH1
      REAL B(LDB,*)
      NFB = NF/JF
      CH1 = CHAR(48+NFB)
      WRITE(KF,'(4X,"",'//CH1//'(""),"")')
      WRITE(KF,'(4X,"    Factors  :",'//CH1//'(I3,"",I2,1X),"")')
     +  ((J-1)*JF+1,J*JF,J=1,NFB)
      WRITE(KF,'(4X,"",'//CH1//'(""),"")')
      WRITE(KF,'(4X," Hi Salient  ",'//CH1//'(A5,2X),"")')
     +  (CLN(B(1,J),5,-2),J=1,NFB)
      WRITE(KF,'(4X," Lo Salient  ",'//CH1//'(A5,2X),"")')
     +  (CLN(B(2,J),5,-2),J=1,NFB)
      WRITE(KF,'(4X," PrNeg Sal.  ",'//CH1//'(A5,2X),"")')
     +  (CLN(B(3,J),5,-2),J=1,NFB)
      WRITE(KF,'(4X," Hi nonSal.  ",'//CH1//'(A5,2X),"")')
     +  (CLN(B(4,J),5,-2),J=1,NFB)
      WRITE(KF,'(4X,"",'//CH1//'(""),"")')
      END SUBROUTINE
C
      SUBROUTINE START(J,F1)
C This opens formatted file F1 with unit-number J, and finds its first line
C beginning with a digit.
      CHARACTER F1*12, CH*80
      OPEN(J,FILE=F1)
10    READ(J,'(A)',END=50) CH
      K = 0
12    K = K+1
      L = ICHAR(CH(K:K))
      IF((L==32.OR.L==0.OR.L==45.OR.L==46).AND.K<80) GOTO 12
      IF(L<48 .OR. L>57) GOTO 10
      BACKSPACE J
      RETURN
50    WRITE(6,'(/" File ",A," is defective.")') F1
      STOP
      END
C
      FUNCTION ZRAND()
C This returns Normally distributed random Z-scores. [From Press et al, p. 203,
C whose theory is obscure at one crucial point.]
      DATA IFLG/0/   ! Saved because declared before any executables
      SAVE G
      IF(IFLG==0) THEN  ! No extra deviate is in store
10      V1 = 2.*RANDY() - 1.   ! Put uniform random number in interval [-1,1]
        V2 = 2.*RANDY() - 1.   ! Do the same for V2
        R = V1**2 + V2**2
        IF(R>=1. .OR. R==0.) GOTO 10 ! If <V1,V2> not in unit circle, try again
        F = SQRT(-2.*LOG(R)/R)
        G = V1*F      ! Save extra random deviate for next time
        ZRAND = V2*F
        IFLG = 1
      ELSE
        ZRAND = G
        IFLG = 0
      END IF
      RETURN
      END
C
      FUNCTION QFMT(F1)
C This determines the status of file F1, returning 'Y' if it is Formatted,
C 'N' if it is Not, and 'U' if it is Unknown (does not exist).
      CHARACTER QFMT, F1*(*)
      INQUIRE(FILE=F1,FORMATTED=QFMT)
      RETURN
      END
C
      FUNCTION TM(KSET)
C The value returned by function TM is seconds since last timer reset. After
C this value is determined, the timer is reset if KSET > 0 but continues to
C accumulate if KSET = 0.
      SAVE PREV
      DATA PREV/0/
      CALL TIMER(J)
      X = .01*J
      TM = X - PREV
      IF(TM<=0.) TM = TM + 86400
      IF(KSET==0) RETURN
      PREV = X
      RETURN
      END
C
      FUNCTION RANDY()
C Use the compiler's random generator for a random number in the unit interval
      DATA NEW/1/  ! Saved because declared before any executables
      IF(NEW>0) THEN
        NEW = 0
        CALL RANDOM_SEED()
      END IF
      CALL RANDOM_NUMBER(RANDY)
      END FUNCTION
C
      SUBROUTINE WAIT(N)
C  N < 0 calls for space before screen display
      IF(N>0) WRITE(6,'()')
      WRITE(6,'(" Hit RETURN to continue")')
      READ(5,'()')
      RETURN
      END


C TEST LINES
C      FUNCTION ERR(NV,NF,JF,PP,CFF,CY,NV)
C      CHARACTER CLN*8
C      REAL PP(NV,*), CFF(*), CY(*), B(5,5)
C      LO(I,J) = J*(J-1)/2 + I
C      LOC(I,J) = LO(MIN(I,J),MAX(I,J))
C      ERR = 0.
C      NB = NF/JF
C      ERR = 0.
C      WRITE(21,'(/" Next decomposition of CHH")')
C      DO 10 J = 1,NV
C       DO 10 I = 1,J-1
C        S = 0.
C        DO 50 L1 = 1,NB
C         DO 50 L2 = 1,NB
C50        B(L1,L2) = 0.
C        DO 20 K1 = 1,NF
C         L1 = 1+(K1-1)/JF
C         DO 20 K2 = 1,NF
C          L2 = 1+(K2-1)/JF
C          X = PP(I,K1)*CFF(LOC(K1,K2))*PP(J,K2)
C          B(L1,L2) = B(L1,L2) + X
C20        S = S + X
C        WRITE(21,'(" Chh(",2I3,") =",A6," (correct,",A6,
C     +   ") has block-decomposition")') I, J, CLN(S,6,3),
C     +   CLN(CY(LO(I,J)),6,3)
C        DO 60 L1 = 1,NB
C60       WRITE(21,'(10X,10A6)') (CLN(B(L1,L2),6,3),L2=1,NB)
C10    ERR = MAX(ERR,ABS(CY(LO(I,J))-S))
C      WRITE(21,'(/" Max reproduction error:",A6)') CLN(ERR,6,3)
C      RETURN
C      END

      SUBROUTINE NAMS(NV,NF,IDENT,A)
C Finish coding items' names with their pattern saliences: IDENT(J) letter-lists
C item J's salient factors in decreasing loading size up a maximum of 6, to which
C a numerical index is appended if more than two items have the same letter profile.
      CHARACTER IDENT(*)*(*), CF*12
      INTEGER TABL(NF,NV), LST(NV)
      REAL A(NV,*)
      COMMON /CF/ CF
      DO I = 1,NV
        NK = LAST(IDENT(I))
        DO K = 1,NK
          M = ICHAR(IDENT(I)(K:K))-64  ! Index of Kth factor salient for item I
          TABL(K,I) = 1000*NINT(1000.*(-ABS(A(I,M)))) + M
        END DO   ! ISORT is ascending so largest loadings are made smallest
        CALL ISORT(NK,TABL(1,I))
        DO K = 1,MIN(NK,6); IDENT(I) = '?'
          IDENT(I)(K:K) = CHAR(64+MOD(TABL(K,I),1000))
        END DO  ! Letters in name are now in decreasing salience order
      END DO
      DO I = 1,NV-1
        K = 1; M = ICHAR(IDENT(I)(1:1)) - 64
        LST(1) = 1000*NINT(1000.*(-ABS(A(I,M)))) + I
        DO J = I+1,NV
          IF(IDENT(I)==IDENT(J)) THEN
            K = K+1; M = ICHAR(IDENT(J)(1:1)) - 64
            LST(K) = 1000*NINT(1000.*(-ABS(A(J,M)))) + J
          END IF
        END DO
CC        IF(K==1) CYCLE
        CALL ISORT(K,LST)
        DO J = 1,K; L = LAST(IDENT(I))
          IDENT(MOD(LST(J),1000))(L+1:) = CF(:JJF(J))
        END DO
      END DO
      END SUBROUTINE
C
      SUBROUTINE SCHEM(NV,NF,JV,JF,CUT,A,LDA)
C Display schematized factor pattern in A with blanks for loadings smaller
C than CUT and others rounded to nearest lst decimal.
      CHARACTER*3 LN(:), FMT*40, C2*2, C1*1, CF*12
      ALLOCATABLE :: LN
      REAL A(LDA,*)
      COMMON /CF/ CF
      ALLOCATE ( LN(NF) )
5     IF(CUT>1.5) CUT = CUT/10
      IF(CUT>1.5) GOTO 5
      C2 = CF(:JJF(JF)); C1 = CF(:JJF(75/(2+3*JF)))
      FMT = '(I4,".",'//C1//'(2X,'//C2//'A),5(:/6X,'//C1//'(2X,'//
     +       C2//'A)))'
      DO I = 1,NV
        IF(MOD(I-1,JV)==0 .AND. I<NV) WRITE(6,'()')
        DO J = 1,NF
          LN(J) = '  .'
          X = ABS(A(I,J)); IF(X<=CUT) CYCLE
          IF(A(I,J)<.01) LN(J)(2:2) = '-'
          LN(J)(3:3) = CHAR(48+NINT(MIN(9.,10*X)))
        END DO
        WRITE(6,FMT) I, (LN(J), J=1,NF)
        IF(((MOD(I,10)==0.AND.NV-I>5).OR.I==NV)) CALL WAIT(1)
      END DO
      END SUBROUTINE
C
      SUBROUTINE NAMM(F1,EXT,L)
C This receives a left-justified filename in F1, solves for <base> to be the
C part of F1 prior to '.' and preceding numerals, scans the sub-directory for
C the lowest i=1,2,... such that file <base>i.EXT does not already exist, and
C returns <base>i.EXT or maybe <bas>i.EXT in F1(:12) along with char-start length
      LOGICAL QY
      CHARACTER F1*(*), EXT*3, CF*12
      COMMON /CF/ CF
      L = 0
10    L = L+1; IF(L<8 .AND. F1(L:L)/=' ' .AND. F1(L:L)/='.') GOTO 10
      I = 0
20    I = I+1; J = JJF(I); IF(I==10) L = L-1
      F1(MIN(L,7):) = CF(:JJF(I))//EXT
      INQUIRE(FILE=F1,EXIST=QY)
      IF(QY) GOTO 20
      L = L-1  ! L returns length of the base charstring
      END SUBROUTINE
